/**
 * BaumbachAssistentDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Assistent zum Hinzufügen aller Beteiligten auf einmal. Mehrere Tabs/Schritte.
 * 1.) Zuerst müssen die einfachen (Baumbach?)Beteiligten für die Klage erstellt
 * werden, ohne das Unterliegen und den Streitwert festzulegen (ggf.
 * XJustiz-Import) bzw. mit einheitlichem Streitwert. 2.) Dann können die
 * Beteiligten der Widerklage hinzugefügt werden, wieder einheitlicher
 * Streitwert. 3.) Dann kann angegeben werden, welche Beteiligten
 * gesamtschuldnerisch und zu wieviel verurteilt wurden. Erst jetzt werden die
 * eigentlichen BaumbachBeteiligten initialisiert (oder jetzt bekommen sie ihre
 * Werte).
 * 
 * @author Peter Schuster (setrok)
 * @date 22.06.2014
 * 
 * 
 */
public class BaumbachAssistentDialog extends
		HinzufuegenDialog<VerfahrensDatenContainer> {

	private static final long serialVersionUID = -170526643502912253L;

	private AbstractAction weiter = new AbstractAction("Weiter") {
		private static final long serialVersionUID = -1432886554037727631L;

		{
			putValue(SHORT_DESCRIPTION, "Weiter zum nächsten Reiter");
			putValue(MNEMONIC_KEY, KeyEvent.VK_W);
			putValue(ACTION_COMMAND_KEY, "Weiter");
			putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_W,
					InputEvent.ALT_DOWN_MASK));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			// Alle Eingaben parsen (und Daten übergeben)
			if (!parseEingabe(zustand, container != null))
				return;
			if ((mitWiderklage && zustand >= HOECHSTER_ZUSTAND - 1)
					|| (!mitWiderklage && zustand >= HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE - 1)) {
				/*
				 * Wenn der jeweils letzte Zustand erreicht ist, Verhalten wie
				 * okButton
				 */
				// Kostenrechner.zeigeFehler("Huhu, zustand ist: " + zustand);
				performOkAction(e);
			} else {
				/* Sonst dann den Zustand erhöhen */
				zustand++;
				if (container != null)
					/*
					 * Bestimmte Zustände müssen bei Import übersprungen werden:
					 */
					if ((zustand == 1 || zustand == 2))
						zustand = 3;
					else if ((zustand == 6 || zustand == 7))
						zustand = 8;
				zustandAnzeigen(zustand);
			}
		}
	};

	private boolean mitWiderklage;
	private int zustand;
	private JTabbedPane tabbedPane;
	private JPanel[] panels;
	private static final int HOECHSTER_ZUSTAND = 11;
	private static final int HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE = 6;
	private VerfahrensDatenContainer container;
	private JTextArea jta;
	private JSpinner spiKlaegerZahl;
	private JSpinner spiBeklagtenZahl;
	private ScalableBeteiligtenTableModel sbtmKlaeger;
	private Object[][] objectsKlaeger;
	private JTable tabKlaegerGenus;
	private JScrollPane scpKlaegerGenus;
	private Object[][] objectsBeklagte;
	private ScalableBeteiligtenTableModel sbtmBeklagte;
	private SimpleBeteiligtenTable tabBeklagtenGenus;
	private JScrollPane scpBeklagteGenus;
	private JCheckBox chbBkSwEinheitlich;
	private JFormattedTextField ftfBkStreitwert;
	private SimpleBeteiligtenTable tabBeklagtenSw;
	private JScrollPane scpBkSw;
	private JCheckBox chBkGesamtschuldner;
	private JList<Beteiligter> liBkGesamtschuldner;
	private JScrollPane scpBeklagtenAuswahlListe;
	private JFormattedTextField ftfBkGesamtUnterliegen;
	private SimpleBeteiligtenTable tbBkUnt;
	private JScrollPane scpBkUnt;
	private JList<Beteiligter> liBkWk;
	private JScrollPane scpBkWk;
	private JSpinner spiDwbkAnz;
	private ScalableBeteiligtenTableModel sbtmDrittwiderbeklagte;
	private Object[][] objectsDwbk;
	private SimpleBeteiligtenTable tbDwbkGenus;
	private JScrollPane scpDwbkGenus;
	private JCheckBox chbWkEinheitlich;
	private JFormattedTextField ftfWkSw;
	private SimpleBeteiligtenTable tabKlSw;
	private JScrollPane scpKlSw;
	private SimpleBeteiligtenTable tabDwbkSw;
	private JScrollPane scpDwbkSw;
	private JCheckBox chbWkGesamtschuld;
	private JList<Beteiligter> liWkGesamtschuldner;
	private JFormattedTextField ftfWkUnterliegen;
	private SimpleBeteiligtenTable tbKlDarueber;
	private JScrollPane scpKlDarueber;
	private SimpleBeteiligtenTable tbDwbkDarueber;
	private JScrollPane scpDwbkDarueber;
	private int klaegerZahl;
	private int beklagtenZahl;
	private int dwbkZahl;
	private double bkStreitwert;
	private Beteiligter[] beteiligteBeklagte;
	private double bkUnterliegen = 0.0;
	private int[] bkGesamtIndices = new int[0];
	private double bkGesamtUnterliegen = 0;
	private int bkWiderklaegerIndex;
	private JCheckBox chbKlWk;
	private boolean bkGesamt = false;
	private double wkStreitwert;
	private boolean klWk = false;
	private Beteiligter[] beteiligteWiderbeklagte;
	private double wkGesamtUnterliegen = 0;
	private double wkUnterliegen = 0;
	private boolean wkGesamt = false;
	private int[] wkGesamtIndices = null;

	/**
	 * Der Konstruktor für den Standardfall ohne Xjustiz-Import
	 * 
	 * @param owner
	 *            the JFrame from which the dialog is displayed
	 * @param mitWiderklage
	 *            sollen auch die Schritte für die Widerklage durchgeführt
	 *            werden?
	 */
	public BaumbachAssistentDialog(JFrame owner, boolean mitWiderklage) {
		super(owner, "Assistent für die Baumbach'sche Kostenformel");
		this.mitWiderklage = mitWiderklage;
		this.zustand = 0;
		this.container = null;
		klaegerZahl = 1;
		beklagtenZahl = 2;
		dwbkZahl = 0;
		// hier einen Default-Wert für den Streitwert nehmen
		bkStreitwert = 10000;
		wkStreitwert = 0;
	}

	/**
	 * Ein Konstruktor, der Verfahrensdaten vom XJustiz-Import annimmt und dann
	 * ein paar Felder schon einmal vorbelegen kann.
	 * 
	 * @param owner
	 *            the JFrame from which the dialog is displayed
	 * @param container
	 *            ein KostenTenorContainer aus einem Xjustiz-Import
	 */
	public BaumbachAssistentDialog(JFrame owner,
			VerfahrensDatenContainer container) {
		super(owner,
				"Assistent für die Baumbach'sche Kostenformel (Xjustiz-Import)");
		if (container == null)
			throw new IllegalArgumentException(
					"Leere Datei oder Datei konnte nicht gelesen werden!");
		// this.mitWiderklage =
		// (BerechnungLadenSpeichernDialog.gibtsKlaeger(container) &&
		// container.streitgenossen.klaeger
		// .enthaeltAnWiderklageBeteiligte())
		// || (BerechnungLadenSpeichernDialog.gibtsBeklagte(container) &&
		// container.streitgenossen.beklagte
		// .enthaeltAnWiderklageBeteiligte())
		// || BerechnungLadenSpeichernDialog.gibtsDrittwiderbeklagte(container);
		this.mitWiderklage = container.streitgenossen.widerklage;
		this.zustand = 0;
		this.container = container;
		if (!BerechnungLadenSpeichernDialog.gibtsKlaeger(container)
				|| container.streitgenossen.klaeger.size() != 1)
			throw new IllegalArgumentException("Nur ein Kläger erlaubt!");
		klaegerZahl = container.streitgenossen.klaeger.size();
		klWk = (mitWiderklage && container.streitgenossen.klaeger
				.enthaeltAnWiderklageBeteiligte());
		/* Gibt's überhaupt Beklagte und wenn ja, maximal einen Widerkläger? */
		if (!BerechnungLadenSpeichernDialog.gibtsBeklagte(container))
			throw new IllegalArgumentException("Datei enthält keine Beklagten!");
		else if (container.streitgenossen.beklagte
				.zaehleAnWiderklageBeteiligte() > 1)
			throw new IllegalArgumentException("Nur ein Widerkläger erlaubt!");
		beklagtenZahl = container.streitgenossen.beklagte.size();
		if (BerechnungLadenSpeichernDialog.gibtsDrittwiderbeklagte(container))
			dwbkZahl = container.streitgenossen.drittwiderbeklagte.size();
		else
			dwbkZahl = 0;
		/*
		 * Gibt es Widerkläger ohne Widerbeklagte oder Widerbeklagte ohne
		 * Widerkläger?
		 */
		if (this.mitWiderklage) {
			if (container.streitgenossen.beklagte
					.zaehleAnWiderklageBeteiligte() > 0
					&& container.streitgenossen.klaeger
							.zaehleAnWiderklageBeteiligte() < 1 && dwbkZahl < 1)
				throw new IllegalArgumentException(
						"Datei enthält Widerkläger ohne Widerbeklagte!");
			if ((container.streitgenossen.klaeger
					.zaehleAnWiderklageBeteiligte() > 0 || dwbkZahl > 0)
					&& container.streitgenossen.beklagte
							.zaehleAnWiderklageBeteiligte() < 1)
				throw new IllegalArgumentException(
						"Datei enthält Widerbeklagte ohne Widerkläger!");
		}
		// hier einen Default-Wert für den Streitwert nehmen (z.B. Xjustiz)
		bkStreitwert = container.allgemein.streitwert;
		wkStreitwert = container.allgemein.streitwert;
	}

	/**
	 * Diese Methode erstellt zum Abschluss BaumbachBeteiligtenListen
	 * 
	 * @return und einen KostenTenorContainer zurückgeben
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected VerfahrensDatenContainer baueRueckgabewert() {
		/*
		 * Erst wenn der letzte Zustand erreicht ist, darf gebaut werden (sollte
		 * eigentlich von parseEingabe() geprüft werden
		 */
		BaumbachBeteiligtenListe klaeger = konvertiere(objectsKlaeger);
		BaumbachBeteiligtenListe beklagte = konvertiere(objectsBeklagte);
		BaumbachBeteiligtenListe drittwiderbeklagte;
		String aktenzeichen;
		double streitwert;
		// Höchsten Streitwert ermitteln
		if (wkStreitwert > bkStreitwert)
			streitwert = wkStreitwert;
		else
			streitwert = bkStreitwert;
		// streitwert = vdc.allgemein.streitwert;
		if (container != null) {
			aktenzeichen = container.allgemein.aktenzeichen;
		} else {
			aktenzeichen = "";
		}
		if (mitWiderklage) {
			drittwiderbeklagte = konvertiere(objectsDwbk);
		} else {
			drittwiderbeklagte = null;
		}
		if (bkGesamt) {
			/* BaumbachGesamtschuldnerschaft bk bauen */
			// TODO Ist genusNumerus Plural?
			beklagte.add(new BaumbachGesamtschuldnerschaft(
					Beteiligter.BEKLAGTE, Beteiligter.MAENNLICH
							+ Beteiligter.PLURAL, bkGesamtUnterliegen, false,
					bkGesamtIndices, beklagte));
		}
		/*
		 * BaumbachGesamtschuldnerschaft kl+dbwk bauen, sofern Voraussetzungen.
		 * Zur Sicherheit (auch wenn's eigentlich schon früher abgefragt wird)
		 * auch Bedingungen gegen NullPointerExceptions
		 */
		if (wkGesamt && drittwiderbeklagte != null
				&& drittwiderbeklagte.size() > 0) {
			BaumbachBeteiligtenListe zwischen = new BaumbachBeteiligtenListe();
			zwischen.addAll(klaeger);
			zwischen.addAll(drittwiderbeklagte);
			drittwiderbeklagte.add(new BaumbachGesamtschuldnerschaft(
					Beteiligter.DRITTWIDERBEKLAGTE, Beteiligter.MAENNLICH
							+ Beteiligter.PLURAL, wkGesamtUnterliegen, true,
					wkGesamtIndices, zwischen));
		}
		return BerechnungLadenSpeichernDialog.erstelleContainer(klaeger,
				drittwiderbeklagte, beklagte, streitwert, aktenzeichen);
	}

	/**
	 * Die Methode initialisiert die allgemeinen JComponents für den
	 * Inhaltsbereich des Dialogs und wird vom Konstruktor selbsttätig
	 * aufgerufen. In der abgeleiteten Klasse baut die Methode eine
	 * {@link javax.swing.JTabbedPane} auf, mit so vielen Tabs wie Zuständen,
	 * die in einem Array JPanel[] panels aufbewahrt werden.
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#baueContentPane()
	 */
	@Override
	protected void baueContentPane() {
		tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		// BorderLayout.NORTH führt zu Darstellungsfehlern beim Vergrößern
		getContentPane().add(tabbedPane, BorderLayout.CENTER);
		tabbedPane.setBorder(BORDER);
		panels = new JPanel[HOECHSTER_ZUSTAND];
		// for-Schleife zum Anlegen der Reiter
		for (int i = 0; i < panels.length; i++) {
			panels[i] = new JPanel();
			panels[i].setBorder(BORDER);
			panels[i].setLayout(new GridBagLayout());
		}
	}

	/**
	 * Füllt die Tabs mit Oberflächenelemente
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		// Jeder Schritt ist ein einzelnes JPanel im tabbedPane

		getButtonPane().add(new JButton(weiter));
		addToRootInputMap(weiter);

		// Nur je eine sbtm-Variable für kl, bk, dwbk!

		// Panel 0:
		tabbedPane.addTab("Start", null, panels[0], "Willkommenstext anzeigen");
		String willkommen = "Willkommen!\n\n"
				+ "Der Assistent führt Sie durch die notwendigen Schritte, \n"
				+ "um den Baumbach'schen Kostentenor berechnen zu können.\n\n"
				+ "Klicken Sie \"Weiter\", um zu beginnen.";
		if (container != null)
			willkommen = "Willkommen!\n\n"
					+ "Sie möchten die Beteiligtendaten aus dem Verfahren "
					+ container.allgemein.aktenzeichen
					+ " importieren.\n"
					+ "Der Assistent unternimmt mit Ihnen die noch verbleibenden Schritte, um die noch fehlenden Daten zu erfragen, \n"
					+ "mit denen ein Baumbach'scher Kostentenor berechnet werden kann.\n\n"
					+ "Klicken Sie \"Weiter\", um zu beginnen.";
		jta = new JTextArea(willkommen);
		jta.setEditable(false);
		panels[0].add(jta, GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1,
				GridBagConstraints.BOTH, false));

		// Panel 1:
		// Kann bei Import übersprungen werden
		tabbedPane.addTab("Beteiligtenanzahl (1)", null, panels[1],
				"Anzahl der Kläger und Beklagten");
		JLabel lblKlaegerZahl = new JLabel("Anzahl der Kläger: ",
				SwingConstants.RIGHT);
		panels[1].add(lblKlaegerZahl, GitterBeutelBeschraenkungen.getInstance(
				0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		spiKlaegerZahl = new JSpinner(new SpinnerNumberModel(klaegerZahl, 0,
				100, 1));
		setMnemonicLabelFor(lblKlaegerZahl, spiKlaegerZahl, KeyEvent.VK_K);
		spiKlaegerZahl.setEnabled(false);
		panels[1].add(spiKlaegerZahl, GitterBeutelBeschraenkungen.getInstance(
				1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblBeklagtenZahl = new JLabel("Anzahl der Beklagten: ",
				SwingConstants.RIGHT);
		panels[1].add(lblBeklagtenZahl, GitterBeutelBeschraenkungen
				.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		spiBeklagtenZahl = new JSpinner(new SpinnerNumberModel(beklagtenZahl,
				0, 100, 1));
		setMnemonicLabelFor(lblBeklagtenZahl, spiBeklagtenZahl, KeyEvent.VK_B);
		panels[1].add(spiBeklagtenZahl, GitterBeutelBeschraenkungen
				.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Panel 2
		// kann bei Import übersprungen werden
		tabbedPane.addTab("-genus (2)", null, panels[2],
				"Bezeichnung von Kläger und Beklagten");

		JLabel lblGenusKlaeger = new JLabel("Bezeichnung der Kläger");
		panels[2].add(lblGenusKlaeger, GitterBeutelBeschraenkungen.getInstance(
				0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		objectsKlaeger = new Object[1][4];
		sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger,
				Beteiligter.KLAEGER);
		tabKlaegerGenus = new SimpleBeteiligtenTable(sbtmKlaeger,
				Beteiligter.KLAEGER);
		scpKlaegerGenus = new JScrollPane(tabKlaegerGenus);
		setMnemonicLabelFor(lblGenusKlaeger, scpKlaegerGenus, KeyEvent.VK_K);
		panels[2].add(scpKlaegerGenus, GitterBeutelBeschraenkungen.getInstance(
				0, 1, 1, 1, GridBagConstraints.BOTH, false));

		JLabel lblBeklagtenGenus = new JLabel("Bezeichnung der Beklagten");
		panels[2].add(lblBeklagtenGenus, GitterBeutelBeschraenkungen
				.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		objectsBeklagte = new Object[2][3];
		sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte,
				Beteiligter.BEKLAGTE);
		tabBeklagtenGenus = new SimpleBeteiligtenTable(sbtmBeklagte,
				Beteiligter.BEKLAGTE);
		scpBeklagteGenus = new JScrollPane(tabBeklagtenGenus);
		setMnemonicLabelFor(lblBeklagtenGenus, scpBeklagteGenus, KeyEvent.VK_B);
		panels[2].add(scpBeklagteGenus, GitterBeutelBeschraenkungen
				.getInstance(1, 1, 1, 1, GridBagConstraints.BOTH, false));

		// Panel 3
		tabbedPane.addTab("Klagewert (3)", null, panels[3],
				"Streitwert der Beklagten (für alle einheitlich?)");
		chbBkSwEinheitlich = new JCheckBox(
				"Die Beklagten wurden alle in derselben Höhe in Anspruch genommen",
				true);
		chbBkSwEinheitlich.setMnemonic(KeyEvent.VK_H);
		chbBkSwEinheitlich.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				/*
				 * : Muss auf Änderungen reagieren: Wenn selected, dann das
				 * Textfeld aktivieren und Table deaktivieren; wenn nicht
				 * selected, dann umgekehrt.
				 */
				boolean b = ((JCheckBox) e.getSource()).isSelected();
				ftfBkStreitwert.setEnabled(b);
				tabBeklagtenSw.setEnabled(!b);
			}
		});
		panels[3].add(chbBkSwEinheitlich, GitterBeutelBeschraenkungen
				.getInstance(0, 0, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblBkStreitwert = new JLabel("Streitwert: ",
				SwingConstants.RIGHT);
		panels[3].add(lblBkStreitwert, GitterBeutelBeschraenkungen.getInstance(
				0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfBkStreitwert = new JFormattedTextField(formatter);
		// Vorbelegen mit dem Wert aus Xjustiz, sofern bekannt
		ftfBkStreitwert.setText(Double.toString(bkStreitwert));
		ftfBkStreitwert.setColumns(40);
		setMnemonicLabelFor(lblBkStreitwert, ftfBkStreitwert, KeyEvent.VK_S);
		// "Weiter"-Action der InputMap hinzufügen und mit VK_ENTER verknüpfen
		setEnterAction(ftfBkStreitwert, weiter);
		panels[3].add(ftfBkStreitwert, GitterBeutelBeschraenkungen.getInstance(
				1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte,
				Beteiligter.BEKLAGTE, 10000.0);
		tabBeklagtenSw = new SimpleBeteiligtenTable(sbtmBeklagte,
				Beteiligter.BEKLAGTE);
		scpBkSw = new JScrollPane(tabBeklagtenSw);
		panels[3].add(scpBkSw, GitterBeutelBeschraenkungen.getInstance(0, 2, 2,
				1, GridBagConstraints.BOTH, false));
		/*
		 * Noch einmal auf ausgewählt setzen, damit der ChangeListener ausgelöst
		 * wird
		 */
		chbBkSwEinheitlich.setSelected(true);

		// Panel 4
		tabbedPane
				.addTab("Gesamtschuld (4)",
						null,
						panels[4],
						"Wurden die Beklagten (ganz oder z.T.) gesamtschuldnerisch verurteilt? Auswahl der Beklagten");
		chBkGesamtschuldner = new JCheckBox(
				"Die Beklagten wurden ganz oder z.T. gesamtschuldnerisch verurteilt",
				true);
		chBkGesamtschuldner.setMnemonic(KeyEvent.VK_G);
		/*
		 * Muss nach dem Mnemonic, weil das Zuweisen eines Mnemonic wohl das
		 * ChangeEvent auch auslöst.
		 */
		chBkGesamtschuldner.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				/*
				 * Wenn nicht ausgewhält, muss die Liste ausgegraut werden und
				 * die Werte / Markierung geleert werden
				 */
				boolean b = ((JCheckBox) e.getSource()).isSelected();
				liBkGesamtschuldner.setEnabled(b);
				ftfBkGesamtUnterliegen.setEnabled(b);
				if (!b) {
					liBkGesamtschuldner.setSelectedIndex(-1);
					ftfBkGesamtUnterliegen.setText("");
				}
			}
		});
		panels[4].add(chBkGesamtschuldner, GitterBeutelBeschraenkungen
				.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblBkGesamtschuldner = new JLabel(
				"Gesamtschuldnerisch verurteilte Beklagten auswählen");
		panels[4].add(lblBkGesamtschuldner, GitterBeutelBeschraenkungen
				.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Hier muss ich wohl die erste Spalte nehmen...
		liBkGesamtschuldner = new JList<Beteiligter>(new Beteiligter[0]);
		liBkGesamtschuldner
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		liBkGesamtschuldner.setCellRenderer(new BeteiligtenRenderer());
		scpBeklagtenAuswahlListe = new JScrollPane(liBkGesamtschuldner);
		setMnemonicLabelFor(lblBkGesamtschuldner, scpBeklagtenAuswahlListe,
				KeyEvent.VK_V);
		panels[4].add(scpBeklagtenAuswahlListe, GitterBeutelBeschraenkungen
				.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));

		JLabel lblBkGesamtUnterliegen = new JLabel("Höhe der Verurteilung",
				SwingConstants.RIGHT);
		panels[4].add(lblBkGesamtUnterliegen, GitterBeutelBeschraenkungen
				.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfBkGesamtUnterliegen = new JFormattedTextField(formatter);
		ftfBkGesamtUnterliegen.setText("10");
		ftfBkGesamtUnterliegen.setColumns(40);
		setMnemonicLabelFor(lblBkGesamtUnterliegen, ftfBkGesamtUnterliegen,
				KeyEvent.VK_H);
		setEnterAction(ftfBkGesamtUnterliegen, weiter);
		panels[4].add(ftfBkGesamtUnterliegen, GitterBeutelBeschraenkungen
				.getInstance(1, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		/* Und einmal die Auswahl setzen, um ein ChangeEvent auszulösen */
		chBkGesamtschuldner.setSelected(true);

		// Panel 5
		// ==> sonst Unterliegen 0
		tabbedPane
				.addTab("Darüber hinaus (5)", null, panels[5],
						"Wurden einige Beklagte darüber hinaus verurteilt? Welche, zu wie viel?");

		JLabel lblBkUnt = new JLabel(
				"Folgende Beklagte wurden darüber hinaus zu weiterer Zahlung verurteilt:");
		panels[5].add(lblBkUnt, GitterBeutelBeschraenkungen.getInstance(0, 0,
				1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte,
				Beteiligter.BEKLAGTE, 10000.0, 0.0);
		tbBkUnt = new SimpleBeteiligtenTable(sbtmBeklagte, Beteiligter.BEKLAGTE);
		scpBkUnt = new JScrollPane(tbBkUnt);
		setMnemonicLabelFor(lblBkUnt, scpBkUnt, KeyEvent.VK_D);
		panels[5].add(scpBkUnt, GitterBeutelBeschraenkungen.getInstance(0, 2,
				2, 1, GridBagConstraints.BOTH, false));

		// AB HIER NUR WENN (mitWiderklage == true)
		if (mitWiderklage == false) {
			zustandAnzeigen(zustand);
			panelsAusblendenBeiImport();
			return;
		}

		// Panel 6
		// kann bei Import übersprungen werden
		tabbedPane.addTab("Widerklage (6)", null, panels[6],
				"Welcher Beklagte klagt? Wie viele Drittwiderbeklagten?");
		JLabel lblBkWk = new JLabel(
				"Bitte wählen Sie den Beklagten aus, der auch Widerkläger ist.");
		panels[6].add(lblBkWk, GitterBeutelBeschraenkungen.getInstance(0, 0, 3,
				1, GridBagConstraints.HORIZONTAL, false));

		liBkWk = new JList<Beteiligter>();
		liBkWk.setCellRenderer(new BeteiligtenRenderer());
		liBkWk.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scpBkWk = new JScrollPane(liBkWk);
		setMnemonicLabelFor(lblBkWk, scpBkWk, KeyEvent.VK_B);
		panels[6].add(scpBkWk, GitterBeutelBeschraenkungen.getInstance(0, 1, 3,
				1, GridBagConstraints.BOTH, false));

		JLabel lblDwbkAnz = new JLabel("Wie viele Drittwiderbeklagte gibt es?",
				SwingConstants.RIGHT);
		panels[6].add(lblDwbkAnz, GitterBeutelBeschraenkungen.getInstance(0, 2,
				1, 1, GridBagConstraints.HORIZONTAL, false));
		spiDwbkAnz = new JSpinner(new SpinnerNumberModel(dwbkZahl, 0, 100, 1));
		setMnemonicLabelFor(lblDwbkAnz, spiDwbkAnz, KeyEvent.VK_D);
		panels[6].add(spiDwbkAnz, GitterBeutelBeschraenkungen.getInstance(1, 2,
				1, 1, GridBagConstraints.HORIZONTAL, false));

		// Abfragen, ob Wk auch gegen den Kläger geht
		chbKlWk = new JCheckBox("auch der Kläger ist Widerbeklagter", true);
		chbKlWk.setMnemonic(KeyEvent.VK_U);
		panels[6].add(chbKlWk, GitterBeutelBeschraenkungen.getInstance(2, 2, 1,
				1, GridBagConstraints.HORIZONTAL, false));

		// Panel 7
		// kann bei Import übersprungen werden
		tabbedPane.addTab("Drittwiderklage (7)", null, panels[7],
				"Bezeichnung der Drittwiderbeklagten");
		JLabel lblDwbkGenus = new JLabel("Genus der Drittwiderbeklagten");
		panels[7].add(lblDwbkGenus, GitterBeutelBeschraenkungen.getInstance(0,
				0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		objectsDwbk = new Object[dwbkZahl][4];
		sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk,
				Beteiligter.DRITTWIDERBEKLAGTE);
		tbDwbkGenus = new SimpleBeteiligtenTable(sbtmDrittwiderbeklagte,
				Beteiligter.DRITTWIDERBEKLAGTE);
		scpDwbkGenus = new JScrollPane(tbDwbkGenus);
		setMnemonicLabelFor(lblDwbkGenus, scpDwbkGenus, KeyEvent.VK_G);
		panels[7].add(scpDwbkGenus, GitterBeutelBeschraenkungen.getInstance(0,
				1, 1, 1, GridBagConstraints.BOTH, false));

		// Panel 8
		tabbedPane
				.addTab("Widerklagewert (8)", null, panels[8],
						"Wurden Widerbeklagte in derselben Höhe verklagt? Welche (jeweils)?");
		chbWkEinheitlich = new JCheckBox(
				"Alle Widerbeklagten wurden in derselben Höhe in Anspruch genommen",
				true);
		chbWkEinheitlich.setMnemonic(KeyEvent.VK_H);
		chbWkEinheitlich.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				/*
				 * Muss auf Änderungen so reagieren, dass entweder das Textfeld
				 * aktiviert werden muss (bei isSelected(true)) oder die beiden
				 * Tabellen (bei isSelected(false)) - die jeweils andere Gruppe
				 * muss dann deaktiv werden.
				 */
				boolean b = ((JCheckBox) e.getSource()).isSelected();
				ftfWkSw.setEnabled(b);
				ftfWkSw.setText(Double.toString(wkStreitwert));
				/*
				 * Die Klägertabelle muss unveränderlich sein, wenn der Kläger
				 * nicht involviert ist.
				 */
				if (klWk)
					tabKlSw.setEnabled(!b);
				else
					tabKlSw.setEnabled(false);
				tabDwbkSw.setEnabled(!b);
				if (!b)
					ftfWkSw.setText("");
			}
		});
		panels[8].add(chbWkEinheitlich, GitterBeutelBeschraenkungen
				.getInstance(0, 0, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblWkSw = new JLabel("Streitwert der Widerklage:",
				SwingConstants.RIGHT);
		panels[8].add(lblWkSw, GitterBeutelBeschraenkungen.getInstance(0, 1, 1,
				1, GridBagConstraints.HORIZONTAL, false));

		ftfWkSw = new JFormattedTextField(formatter);
		ftfWkSw.setText(Double.toString(wkStreitwert));
		setMnemonicLabelFor(lblWkSw, ftfWkSw, KeyEvent.VK_S);
		setEnterAction(ftfWkSw, weiter);
		panels[8].add(ftfWkSw, GitterBeutelBeschraenkungen.getInstance(1, 1, 1,
				1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblKlSw = new JLabel("Streitwert für den Kläger:");
		panels[8].add(lblKlSw, GitterBeutelBeschraenkungen.getInstance(0, 2, 1,
				1, GridBagConstraints.HORIZONTAL, false));

		sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger,
				Beteiligter.KLAEGER, 0.0);
		tabKlSw = new SimpleBeteiligtenTable(sbtmKlaeger, Beteiligter.KLAEGER);
		scpKlSw = new JScrollPane(tabKlSw);
		setMnemonicLabelFor(lblKlSw, scpKlSw, KeyEvent.VK_K);
		panels[8].add(scpKlSw, GitterBeutelBeschraenkungen.getInstance(0, 3, 1,
				1, GridBagConstraints.BOTH, false));

		JLabel lblDwbkSw = new JLabel(
				"Streitwert bzgl. der Drittwiderbeklagten:");
		panels[8].add(lblDwbkSw, GitterBeutelBeschraenkungen.getInstance(1, 2,
				1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk,
				Beteiligter.DRITTWIDERBEKLAGTE, 0.0);
		tabDwbkSw = new SimpleBeteiligtenTable(sbtmDrittwiderbeklagte,
				Beteiligter.DRITTWIDERBEKLAGTE);
		scpDwbkSw = new JScrollPane(tabDwbkSw);
		setMnemonicLabelFor(lblDwbkSw, scpDwbkSw, KeyEvent.VK_D);
		panels[8].add(scpDwbkSw, GitterBeutelBeschraenkungen.getInstance(1, 3,
				1, 1, GridBagConstraints.BOTH, false));

		// Panel 9
		tabbedPane
				.addTab("Gesamtschuld (9)",
						null,
						panels[9],
						"Wurden die Widerbeklagten z.T. gesamtschuldnerisch verurteilt? Welche Widerbeklagten, in welcher Höhe?");
		chbWkGesamtschuld = new JCheckBox(
				"Einige Widerbeklagte wurden gesamtschuldnerisch verurteilt.",
				true);
		chbWkGesamtschuld.setMnemonic(KeyEvent.VK_E);
		chbWkGesamtschuld.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				/*
				 * Wenn nicht ausgewählt, muss die Liste ausgegraut werden und
				 * die Werte / Markierung geleert werden
				 */
				boolean b = ((JCheckBox) e.getSource()).isSelected();
				liWkGesamtschuldner.setEnabled(b);
				ftfWkUnterliegen.setEnabled(b);
				if (!b) {
					liWkGesamtschuldner.setSelectedIndex(-1);
					ftfWkUnterliegen.setText(Double
							.toString(wkGesamtUnterliegen));
				}

			}
		});
		panels[9].add(chbWkGesamtschuld, GitterBeutelBeschraenkungen
				.getInstance(0, 0, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblWkGesamtschuldner = new JLabel(
				"Folgende Widerbeklagte wurde gesamtschuldnerisch verurteilt");
		panels[9].add(lblWkGesamtschuldner, GitterBeutelBeschraenkungen
				.getInstance(0, 1, 2, 1, GridBagConstraints.HORIZONTAL, false));

		liWkGesamtschuldner = new JList<Beteiligter>();
		liWkGesamtschuldner.setCellRenderer(new BeteiligtenRenderer());
		setMnemonicLabelFor(lblWkGesamtschuldner, liWkGesamtschuldner,
				KeyEvent.VK_F);
		liWkGesamtschuldner
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		panels[9].add(liWkGesamtschuldner, GitterBeutelBeschraenkungen
				.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));

		JLabel lblWkUnterliegen = new JLabel("Höhe der Verurteilung",
				SwingConstants.RIGHT);
		panels[9].add(lblWkUnterliegen, GitterBeutelBeschraenkungen
				.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfWkUnterliegen = new JFormattedTextField(formatter);
		ftfWkUnterliegen.setText(Double.toString(wkGesamtUnterliegen));
		setMnemonicLabelFor(lblWkUnterliegen, ftfWkUnterliegen, KeyEvent.VK_H);
		setEnterAction(ftfWkUnterliegen, weiter);
		panels[9].add(ftfWkUnterliegen, GitterBeutelBeschraenkungen
				.getInstance(1, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Panel 10
		tabbedPane
				.addTab("Darüber hinaus (10)",
						null,
						panels[10],
						"Wurden einzelne Widerbeklagten darüber hinaus einzeln verurteilt? Welche, in welcher Höhe?");
		JLabel lblKlDarueber = new JLabel(
				"Folgende Kläger wurden darüber hinaus verurteilt:");
		panels[10].add(lblKlDarueber, GitterBeutelBeschraenkungen.getInstance(
				0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger,
				Beteiligter.KLAEGER, wkStreitwert, wkUnterliegen);
		tbKlDarueber = new SimpleBeteiligtenTable(sbtmKlaeger,
				Beteiligter.KLAEGER);
		scpKlDarueber = new JScrollPane(tbKlDarueber);
		setMnemonicLabelFor(lblKlDarueber, scpKlDarueber, KeyEvent.VK_K);
		panels[10].add(scpKlDarueber, GitterBeutelBeschraenkungen.getInstance(
				0, 1, 1, 1, GridBagConstraints.BOTH, false));

		JLabel lblDwbkDarueber = new JLabel(
				"Folgende Drittwiderbeklagte wurden darüber hinaus verurteilt:");
		panels[10].add(lblDwbkDarueber, GitterBeutelBeschraenkungen
				.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk,
				Beteiligter.DRITTWIDERBEKLAGTE, wkStreitwert, wkUnterliegen);
		tbDwbkDarueber = new SimpleBeteiligtenTable(sbtmDrittwiderbeklagte,
				Beteiligter.DRITTWIDERBEKLAGTE);
		scpDwbkDarueber = new JScrollPane(tbDwbkDarueber);
		setMnemonicLabelFor(lblDwbkDarueber, scpDwbkDarueber, KeyEvent.VK_D);
		panels[10].add(scpDwbkDarueber, GitterBeutelBeschraenkungen
				.getInstance(1, 1, 1, 1, GridBagConstraints.BOTH, false));

		zustandAnzeigen(zustand);
		panelsAusblendenBeiImport();
	}

	/**
	 * Die Methode setzt alle Tabs außer den aktuellen und den Start-Tab auf
	 * deaktiv
	 * 
	 * @param index
	 *            Index des zu aktivierenden Tabs
	 */
	private void zustandAnzeigen(int index) {
		for (int i = 0; i < (mitWiderklage ? HOECHSTER_ZUSTAND
				: HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE) && panels[i] != null; i++) {
			if (i == 0) {
				tabbedPane.setEnabledAt(i, true);
			} else if (i == index) {
				tabbedPane.setEnabledAt(i, true);
				tabbedPane.setSelectedIndex(i);
			} else {
				tabbedPane.setEnabledAt(i, false);
			}
		}
	}

	/**
	 * Die Methode darf erst ganz zum Schluss true melden, abhängig davon also,
	 * ob eine Widerklage vorliegt oder nicht. Wenn keine Widerklage vorliegt,
	 * ist der Vorgang nach dem Panel 5 fertig, mit Widerklage nach dem Panel
	 * 10.
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	protected boolean parseEingabe() {
		if (mitWiderklage) {
			System.out.println(zustand);
			return zustand == HOECHSTER_ZUSTAND - 1 && parseEingabe(10);
		} else {
			return zustand == HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE - 1
					&& parseEingabe(5);
		}
	}

	/**
	 * Überprüft die Eingabe und übergibt sie an das nächste Tab
	 * 
	 * @param tab
	 *            die Nummer des Tabulators
	 * @return true, wenn das Überprüfen und Übergeben klappt
	 */
	private boolean parseEingabe(int tab) {
		/*
		 * Zu jedem Panel überlegen, was ausgelesen und was in das nächste Panel
		 * geschrieben werden muss. Mögliche Exceptions und RuntimeExceptions
		 * müssen abgefangen werden
		 */
		boolean b = false;
		switch (tab) {
		case 0:
		default:
			// Willkommen: nix zu prüfen
			b = true;
			break;
		case 1:
			// Anzahl Kläger und Beklagte
			try {
				klaegerZahl = ((SpinnerNumberModel) spiKlaegerZahl.getModel())
						.getNumber().intValue();
				// Integer.parseInt(spiKlaegerZahl.getText());
				beklagtenZahl = ((SpinnerNumberModel) spiBeklagtenZahl
						.getModel()).getNumber().intValue();
				// Integer.parseInt(spBeklagtenZahl.getText());
				if (klaegerZahl == 1 && beklagtenZahl > 0) {
					objectsKlaeger = new Object[klaegerZahl][4];
					objectsBeklagte = new Object[beklagtenZahl][4];
					sbtmKlaeger = new ScalableBeteiligtenTableModel(
							objectsKlaeger, Beteiligter.KLAEGER);
					tabKlaegerGenus.setModel(sbtmKlaeger);
					// ((BeteiligtenTable) tabKlaegerGenus).setWidth();
					sbtmBeklagte = new ScalableBeteiligtenTableModel(
							objectsBeklagte, Beteiligter.BEKLAGTE);
					tabBeklagtenGenus.setModel(sbtmBeklagte);
					// ((BeteiligtenTable) tabBeklagtenGenus).setWidth();
					b = true;
				} else
					b = false;
			} catch (IllegalArgumentException e) {
				if (e instanceof NumberFormatException)
					Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
							HinzufuegenDialog.FEHLER_ZAHLENFORMAT_KL_BK, e);
				else
					Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
							HinzufuegenDialog.FEHLER_BEIM_ANLEGEN_KL_BK, e);
				// e.printStackTrace();
				b = false;
			}
			break;
		case 2:
			/* Bezeichnung von Kläger und Beklagten (nix zu beachten) */
			objectsKlaeger = sbtmKlaeger.getValues();
			objectsBeklagte = sbtmBeklagte.getValues();
			ftfBkStreitwert.setText(Double.toString(bkStreitwert));
			sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte,
					Beteiligter.BEKLAGTE, bkStreitwert);
			tabBeklagtenSw.setModel(sbtmBeklagte);
			chbBkSwEinheitlich.setSelected(false);
			chbBkSwEinheitlich.setSelected(true);
			// Kläger werden für eine der Widerklage-Tabs gebraucht!
			b = true;
			break;
		case 3:
			/*
			 * Streitwert der Beklagten (für alle einheitlich?) Prüfen, ob
			 * einheitlich oder nicht
			 */
			boolean bkEinheitlich = chbBkSwEinheitlich.isSelected();
			try {
				if (bkEinheitlich) {
					/*
					 * wenn einheitlich: speichern und auf alle Bk übernehmen
					 * und alle Werte ziehen --> für Panel 5
					 */
					bkStreitwert = Double
							.parseDouble(ftfBkStreitwert.getText());
					sbtmBeklagte.setEachStreitwert(bkStreitwert, true);
					sbtmBeklagte.fireTableRowsUpdated(0,
							sbtmBeklagte.getRowCount() - 1);
					objectsBeklagte = sbtmBeklagte.getValues();
				} else {
					/*
					 * sonst: getValues und höchsten Streitwert für schlechte
					 * Tage zur Seite legen --> für den zurückzugebenden
					 * KostenTenorContainer
					 */
					objectsBeklagte = sbtmBeklagte.getValues();
					bkStreitwert = sbtmBeklagte.getHoechstenStreitwert();
					ftfBkStreitwert.setText(Double.toString(bkStreitwert));
				}
				/*
				 * Panel 4 vorbereiten: Array Beteiligter[], nur erste Spalte
				 * --> wird auch für Panel 6 gebraucht
				 */
				beteiligteBeklagte = sbtmBeklagte.getBeteiligterColumn();
				liBkGesamtschuldner.setListData(beteiligteBeklagte);
				ftfBkGesamtUnterliegen.setText(Double
						.toString(bkGesamtUnterliegen));
				/* Panel 5 schon einmal vorbereiten */
				sbtmBeklagte = new ScalableBeteiligtenTableModel(
						objectsBeklagte, Beteiligter.BEKLAGTE, bkStreitwert,
						bkUnterliegen);
				tbBkUnt.setModel(sbtmBeklagte);
				b = true;
			} catch (NumberFormatException e) {
				Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
						HinzufuegenDialog.FEHLER_ZAHLENFORMAT_SW, e);
				// e.printStackTrace();
				b = false;
			}
			break;
		case 4:
			bkGesamt = chBkGesamtschuldner.isSelected();
			/*
			 * Prüfen: Häkchen bei gesamtschuldnerischer Verurteilung gesetzt?
			 * Wenn ja, dann muss das Unterliegen auch über 0 sein, sonst ist
			 * die Eingabe kappes.
			 */
			if (bkGesamt) {
				try {
					bkGesamtIndices = liBkGesamtschuldner.getSelectedIndices();
					bkGesamtUnterliegen = Double
							.parseDouble(ftfBkGesamtUnterliegen.getText());
					if (bkGesamtIndices != null && bkGesamtIndices.length > 1
							&& bkGesamtUnterliegen > 0
							&& bkGesamtUnterliegen <= bkStreitwert)
						b = true;
					else {
						Kostenrechner
								.zeigeFehler(
										FehlerDialog.FEHLER_BLANKO,
										HinzufuegenDialog.FEHLER_BK_GESAMTSCHULDNERSCHAFT,
										new IllegalArgumentException(
												HinzufuegenDialog.FEHLER_BK_GESAMTSCHULDNERSCHAFT));
					}
				} catch (NumberFormatException e) {
					Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
							HinzufuegenDialog.FEHLER_ZAHLENFORMAT, e);
					// e.printStackTrace();
				}
			} else {
				// sonst: alles egal, keine Plausi zu prüfen
				b = true;
			}
			break;
		case 5:
			/*
			 * Wurden einige Beklagte darüber hinaus verurteilt? Welche, zu wie
			 * viel?
			 */
			/*
			 * Prüfen, ob einer der Beteiligten ein höheres Unterliegen als der
			 * streitwert hat (ggfs. zusammen mit der gesamtschuldnerischen
			 * Verurteilung)
			 */
			b = true;
			for (int row = 0; row < sbtmBeklagte.getRowCount(); row++) {
				double d = ((Double) sbtmBeklagte.getValueAt(row, 3))
						.doubleValue();
				if (d > bkStreitwert)
					b = false;
				if (chBkGesamtschuldner.isSelected()
						// && Arrays.asList(bkGesamtIndices).contains(row)
						&& contains(bkGesamtIndices, row)
						&& (d + bkGesamtUnterliegen) > bkStreitwert)
					b = false;
			}
			if (!b)
				Kostenrechner
						.zeigeFehler(
								FehlerDialog.FEHLER_BLANKO,
								HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT,
								new IllegalArgumentException(
										HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT));
			else {
				/*
				 * Für die Rückgabe muss anWiderklageBeteiligt erst einmal auf
				 * false gesetzt werden, wo es nicht gesetzt wurde
				 */
				sbtmBeklagte.setEachAnwiderklagebeteiligt(false, false);
				// Dann Werte auslesen
				objectsBeklagte = sbtmBeklagte.getValues();
				if (mitWiderklage) {
					// Wenn Widerklage
					if (container == null) {
						// (und sofern nicht xjustiz-import) Panel 6 bauen
						liBkWk.setListData(beteiligteBeklagte);
						// spiDwbkAnz.setText(Integer.toString(dwbkZahl));
						((SpinnerNumberModel) spiDwbkAnz.getModel())
								.setValue(new Integer(dwbkZahl));
					} else {
						// Wenn xjustiz-import: nix, Panel 6 schon angelegt
					}
				} else {
					// wenn keine Widerklage vorhanden => hier fertig!
				}
			}
			break;
		case 6:
			// Welcher Beklagte klagt? Wie viele Drittwiderbeklagten?
			try {
				bkWiderklaegerIndex = liBkWk.getSelectedIndex();
				dwbkZahl = ((SpinnerNumberModel) spiDwbkAnz.getModel())
						.getNumber().intValue();
				// Integer.parseInt(spiDwbkAnz.getText());
				klWk = chbKlWk.isSelected();
				/*
				 * Mindestens ein Bk muss ausgewählt sein, dwbkZahl darf nicht
				 * negativ sein und wenn schon mitWiderklage, dann muss der
				 * Kläger widerbeklagt sein oder wenigstens ein Dwbk vorhanden
				 * sein
				 */
				if (bkWiderklaegerIndex > -1 && dwbkZahl > -1
						&& (klWk || dwbkZahl > 0)) {
					/*
					 * Beim markierten Beklagten die vierte Spalte (index 3,
					 * anWiderklageBeteiligt) auf true setzen. Hier gehe ich
					 * direkt über die Werte und nicht den Umweg über ein sbtm,
					 * weil an dieser Stelle keine Tabelle angezeigt wird.
					 */
					objectsBeklagte[bkWiderklaegerIndex][3] = Boolean.TRUE;
					/*
					 * Schreiben, ob Wk auch gegen den Kläger geht. Auch hier
					 * direkt über die Werte, weil eine Tabelle dazu gerade
					 * nicht angezeigt wird.
					 */
					objectsKlaeger[0][3] = new Boolean(klWk);
					objectsDwbk = new Object[dwbkZahl][4];
					sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(
							objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE);
					// Drittwiderbeklagte sind immer an der Wk beteiligt
					sbtmDrittwiderbeklagte.setEachAnwiderklagebeteiligt(true,
							false);
					tbDwbkGenus.setModel(sbtmDrittwiderbeklagte);
					b = true;
				} else {
					Kostenrechner
							.zeigeFehler(
									FehlerDialog.FEHLER_BLANKO,
									HinzufuegenDialog.FEHLER_NIX_WIDERKLAEGER_WIDERBEKLAGTER,
									new IllegalArgumentException(
											HinzufuegenDialog.FEHLER_NIX_WIDERKLAEGER_WIDERBEKLAGTER));
					b = false;
				}
			} catch (NumberFormatException e) {
				Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
						HinzufuegenDialog.FEHLER_ZAHLFORMAT_DWBK, e);
				// e.printStackTrace();
				b = false;
			}
			break;
		case 7:
			/* Bezeichnung der Drittwiderbeklagten (nix zu prüfen) */
			/*
			 * Denkt auch daran, dass man Dwbk auch weglassen kann
			 */
			if (dwbkZahl > 0) {
				objectsDwbk = sbtmDrittwiderbeklagte.getValues();
				sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(
						objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE,
						wkStreitwert);
				tabDwbkSw.setModel(sbtmDrittwiderbeklagte);
			} else {
				// schon gut
			}
			b = true;
			/*
			 * Für Panel 8 werden dann wieder die Klägerdaten gebraucht:
			 * angezeigt werden sollen sie immer. Der ChangeListener der
			 * checkbox sorgt dafür, dass die Kläger nur bearbeitet werden
			 * können, wenn der Kläger was mit der Wk am Hut hat.
			 */
			if (klWk) {
				sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger,
						Beteiligter.KLAEGER, wkStreitwert);
			} else {
				sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger,
						Beteiligter.KLAEGER, 0);
			}
			tabKlSw.setModel(sbtmKlaeger);
			/*
			 * Einmal den ChangeListener aufmerksam machen, der schreibt auch
			 * den wkStreitwert ins Textfeld.
			 */
			chbWkEinheitlich.setSelected(false);
			chbWkEinheitlich.setSelected(true);
			break;
		case 8:
			/*
			 * Wurden Widerbeklagte in derselben Höhe verklagt? Welche
			 * (jeweils): Hier achtet die Prüfung immer darauf, dass es einen
			 * Kläger geben kann, der nicht widerbeklagt ist bzw. dass es
			 * möglicherweise keine Dwbk gibt.
			 */
			boolean wkEinheitlich = chbWkEinheitlich.isSelected();
			try {
				if (wkEinheitlich) {
					/* wenn einheitlich */
					wkStreitwert = Double.parseDouble(ftfWkSw.getText());
					if (klWk) {
						// Wenn nicht bei Wk dabei, dann auch nichts setzen
						sbtmKlaeger.setEachStreitwert(wkStreitwert, true);
						objectsKlaeger = sbtmKlaeger.getValues();
					}
					if (dwbkZahl > 0) {
						// Man kann Dwbk auch weglassen
						sbtmDrittwiderbeklagte.setEachStreitwert(wkStreitwert,
								true);
						objectsDwbk = sbtmDrittwiderbeklagte.getValues();
					}
				} else {
					/* wenn nicht einheitlich */
					double klWkStreitwert = 0;
					double dwbkStreitwert = 0;
					if (klWk) {
						// Wenn nicht bei Wk dabei, dann auch nichts lesen
						objectsKlaeger = sbtmKlaeger.getValues();
						klWkStreitwert = sbtmKlaeger.getHoechstenStreitwert();
					}
					if (dwbkZahl > 0) {
						// Man kann Dwbk auch weglassen
						objectsDwbk = sbtmDrittwiderbeklagte.getValues();
						dwbkStreitwert = sbtmDrittwiderbeklagte
								.getHoechstenStreitwert();
					}
					// Höheren Streitwert von beiden speichern
					wkStreitwert = (klWkStreitwert > dwbkStreitwert) ? klWkStreitwert
							: dwbkStreitwert;
					ftfWkSw.setText(Double.toString(wkStreitwert));
				}
				if (wkStreitwert <= 0) {
					Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
							HinzufuegenDialog.FEHLER_WK_STREITWERT,
							new IllegalArgumentException(
									HinzufuegenDialog.FEHLER_WK_STREITWERT));
					b = false;
					break;
				} else {
					b = true;
				}
				/*
				 * Immer eine Liste aus Kl (auch wenn nicht wbk) und Dwbk bauen,
				 * damit die Indexe beim Bauen einer
				 * BaumbachGesamtschuldnerschaft stimmen. Umständlicher Weg mit
				 * einer for-each-Schleife, aber Arrays.asList() gibt eine
				 * unveränderliche Liste zurück, sodass der erste Weg nicht
				 * geklappt hat...
				 */
				ArrayList<Beteiligter> tempWiderbeklagte = new ArrayList<Beteiligter>();
				for (Beteiligter bb : sbtmKlaeger.getBeteiligterColumn())
					tempWiderbeklagte.add(bb);
				for (Beteiligter bb : sbtmDrittwiderbeklagte
						.getBeteiligterColumn())
					tempWiderbeklagte.add(bb);
				beteiligteWiderbeklagte = tempWiderbeklagte
						.toArray(new Beteiligter[tempWiderbeklagte.size()]);
				liWkGesamtschuldner.setListData(beteiligteWiderbeklagte);
				ftfWkUnterliegen.setText(Double.toString(wkGesamtUnterliegen));
				if (klWk) {
					sbtmKlaeger = new ScalableBeteiligtenTableModel(
							objectsKlaeger, Beteiligter.KLAEGER, wkStreitwert,
							wkUnterliegen);
					tbKlDarueber.setModel(sbtmKlaeger);
					tbKlDarueber.setEnabled(true);
				} else {
					/*
					 * Wenn Kl nicht wbk, dann darf man für ihn auch nichts
					 * eingeben
					 */
					tbKlDarueber.setEnabled(false);
				}
				if (dwbkZahl > 0) {
					sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(
							objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE,
							wkStreitwert, wkUnterliegen);
					tbDwbkDarueber.setModel(sbtmDrittwiderbeklagte);
					tbDwbkDarueber.setEnabled(true);
				} else {
					tbDwbkDarueber.setEnabled(false);
				}
				chbWkGesamtschuld.setSelected(false);
				chbWkGesamtschuld.setSelected(true);
				chbWkGesamtschuld
						.setSelected(beteiligteWiderbeklagte.length > 1);
			} catch (NumberFormatException e) {
				b = false;
				Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
						HinzufuegenDialog.FEHLER_ZAHLENFORMAT_SW, e);
				// e.printStackTrace();
			}
			break;
		case 9:
			/*
			 * Wurden die Widerbeklagten z.T. gesamtschuldnerisch verurteilt?
			 * Welche Widerbeklagten, in welcher Höhe? Prüfen: Häkchen bei
			 * gesamtschuldnerischer Verurteilung gesetzt? Wenn ja, dann muss
			 * das Unterliegen auch über 0 sein, sonst ist die Eingabe kappes.
			 */
			wkGesamt = chbWkGesamtschuld.isSelected();
			if (wkGesamt) {
				try {
					wkGesamtIndices = liWkGesamtschuldner.getSelectedIndices();
					wkGesamtUnterliegen = Double.parseDouble(ftfWkUnterliegen
							.getText());
					if (wkGesamtIndices != null && wkGesamtIndices.length > 1
							&& wkGesamtUnterliegen > 0
							&& wkGesamtUnterliegen <= wkStreitwert)
						b = true;
					else {
						b = false;
						Kostenrechner
								.zeigeFehler(
										FehlerDialog.FEHLER_BLANKO,
										HinzufuegenDialog.FEHLER_WK_GESAMTSCHULDNERSCHAFT,
										new IllegalArgumentException(
												HinzufuegenDialog.FEHLER_WK_GESAMTSCHULDNERSCHAFT));
					}
					/*
					 * Wenn der Kläger nicht widerbeklagt ist, darf er nicht
					 * ausgewählt werden
					 */
					if (!klWk && contains(wkGesamtIndices, 0)) {
						b = false;
						Kostenrechner
								.zeigeFehler(
										FehlerDialog.FEHLER_BLANKO,
										HinzufuegenDialog.FEHLER_KLAEGER_NIX_WBK,
										new IllegalArgumentException(
												HinzufuegenDialog.FEHLER_KLAEGER_NIX_WBK));
					}
				} catch (NumberFormatException e) {
					b = false;
					Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
							HinzufuegenDialog.FEHLER_ZAHLENFORMAT, e);
					// e.printStackTrace();
				}
			} else {
				/*
				 * Wenn keine Gesamtschuldnerschaft angehakt, keine Plausis zu
				 * prüfen.
				 */
				b = true;
			}
			/* Panel 10 ist schon unter case 8 gebaut worden. */
			break;
		case 10:
			/*
			 * Wurden einzelne Widerbeklagten darüber hinaus einzeln verurteilt?
			 * Welche, in welcher Höhe?
			 */

			/*
			 * Prüfen, ob einer der Beteiligten ein höheres Unterliegen als der
			 * streitwert hat (ggfs. zusammen mit der gesamtschuldnerischen
			 * Verurteilung)
			 */
			b = true;
			/*
			 * Wenn Kl nicht wbk, dann darf man für ihn auch nichts eingeben,
			 * sonst prüfen, ob Unterliegen den Streitwert überschreitet
			 */
			if (klWk)
				for (int row = 0; row < sbtmKlaeger.getRowCount(); row++) {
					double d = ((Double) sbtmKlaeger.getValueAt(row, 3))
							.doubleValue();
					if (d > wkStreitwert)
						b = false;
					else if (chbWkGesamtschuld.isSelected()
							&& contains(wkGesamtIndices, row)
							&& (d + wkGesamtUnterliegen) > wkStreitwert)
						b = false;
				}
			/* Dran denken, dass man Dwbk auch weglassen kann */
			if (dwbkZahl > 0)
				for (int row = 0; row < sbtmDrittwiderbeklagte.getRowCount(); row++) {
					int absRow = row + klaegerZahl;
					double d = ((Double) sbtmDrittwiderbeklagte.getValueAt(row,
							3)).doubleValue();
					if (d > wkStreitwert)
						b = false;
					else if (chbWkGesamtschuld.isSelected()
							&& contains(wkGesamtIndices, absRow)
							&& (d + wkGesamtUnterliegen) > wkStreitwert)
						b = false;
				}
			if (!b)
				Kostenrechner
						.zeigeFehler(
								FehlerDialog.FEHLER_BLANKO,
								HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT,
								new IllegalArgumentException(
										HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT));
			// Wenn Widerklage erhoben: hier fertig!
			break;
		}
		return b;
	}

	/**
	 * Die Variante von parseEingabe, die nach Import von Xjustiz genutzt werden
	 * soll. Überprüft die Eingabe und übergibt sie an das nächste Tab, füllt
	 * dieses aber, wenn möglich mit Daten aus Xjustiz.
	 * 
	 * @param tab
	 *            die Nummer des Tabulators
	 * @param ueberspringen
	 *            true = es sollen die bei Xjustiz-Import unnötigen Tabs
	 *            übersprungen werden
	 * @return true, wenn zum nächsten, nicht zu überspringenden Tab gewechselt
	 *         werden kann
	 */
	private boolean parseEingabe(int tab, boolean ueberspringen) {
		if (!ueberspringen)
			return parseEingabe(tab);
		else {
			/*
			 * Der Konstruktor für vdc überprüft schon, ob genug Kläger und
			 * Beklagte da sind und setzt die Anzahl dieser Beteiligten.
			 * 
			 * Um das eigentliche Überspringen muss sich der Weiter-Button
			 * kümmern.
			 * 
			 * Bei den zu überspringenden Tabs werden die zweidimensionalen
			 * Object-Arrays gefüllt, die sbtm-Modelle angelegt, setModel bei
			 * den JTables ausgeführt und die jeweiligen Streitwert-Textfelder
			 * gefüllt.
			 */
			if (tab == 0) {
				boolean b = parseEingabe(tab);
				/*
				 * Klägerdaten auslesen (werden an der Oberfläche dann erstmal
				 * nicht gebraucht), Beklagtendaten auslesen und für Panel 3
				 * setzen
				 */
				objectsKlaeger = konvertiere(container.streitgenossen.klaeger);
				objectsBeklagte = konvertiere(container.streitgenossen.beklagte);
				ftfBkStreitwert.setText(Double.toString(bkStreitwert));
				sbtmBeklagte = new ScalableBeteiligtenTableModel(
						objectsBeklagte, Beteiligter.BEKLAGTE, bkStreitwert);
				tabBeklagtenSw.setModel(sbtmBeklagte);
				chbBkSwEinheitlich.setSelected(false);
				chbBkSwEinheitlich.setSelected(true);
				return b;
			} else if (tab == 5) {
				boolean b = parseEingabe(tab);
				/*
				 * Drittwiderbeklagtendaten auslesen, Kläger- und Dwbk-Daten für
				 * Panel 8 setzen.
				 */
				ftfWkSw.setText(Double.toString(wkStreitwert));
				if (klWk) {
					sbtmKlaeger = new ScalableBeteiligtenTableModel(
							objectsKlaeger, Beteiligter.KLAEGER, wkStreitwert);
				} else {
					sbtmKlaeger = new ScalableBeteiligtenTableModel(
							objectsKlaeger, Beteiligter.KLAEGER, 0);
				}
				tabKlSw.setModel(sbtmKlaeger);
				if (BerechnungLadenSpeichernDialog
						.gibtsDrittwiderbeklagte(container))
					objectsDwbk = konvertiere(container.streitgenossen.drittwiderbeklagte);
				else
					objectsDwbk = new Object[0][4];
				sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(
						objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE,
						wkStreitwert);
				tabDwbkSw.setModel(sbtmDrittwiderbeklagte);
				chbWkEinheitlich.setSelected(false);
				chbWkEinheitlich.setSelected(true);
				return b;
			} else {
				return parseEingabe(tab);
			}
		}
	}

	/**
	 * Die Methode sorgt dafür, dass bei Xjustiz-Import die nicht benötigten
	 * Panels/Tabs auch nicht stören. Zwar kann man diese Tabs nicht komplett
	 * ausblenden, aber zumindest werden die Titles auf null gesetzt, so dass
	 * sie leer sind. Der Assistent erscheint dadurch nicht mehr so groß.
	 * 
	 */
	private void panelsAusblendenBeiImport() {
		if (container == null)
			return;
		panels[1].setVisible(false);
		tabbedPane.setTitleAt(1, null);
		panels[2].setVisible(false);
		tabbedPane.setTitleAt(2, null);
		if (mitWiderklage) {
			tabbedPane.setTitleAt(6, null);
			panels[6].setVisible(false);
			tabbedPane.setTitleAt(7, null);
			panels[7].setVisible(false);
		}
	}

	/**
	 * Die Methode prüft, ob in einem Array int[] ein bestimmter int enthalten
	 * ist.
	 * 
	 * @param indices
	 *            das zu durchsuchende Array int[]
	 * @param index
	 *            der int-Wert, der enthalten sein könnte
	 * @return true, wenn index in indices enthalten ist.
	 */
	private boolean contains(int[] indices, int index) {
		boolean contains = false;
		for (int i : indices)
			if (i == index)
				contains = true;
		return contains;
	}

	/**
	 * Konvertiert einen BaumbachBeteiligten in ein Object-Array aus
	 * Beteiligter, null, null, Boolean. Streitwert und Unterliegen werden hier
	 * bewusst mit null aufgefüllt, weil die auf jeden Fall vom Anwender gesetzt
	 * werden sollen.
	 * 
	 * @param bb
	 *            ein BaumbachBeteiligter
	 * @return ein Object-Array aus Beteiligter, null, null, Boolean
	 */
	private Object[] konvertiere(BaumbachBeteiligter bb) {
		if (bb != null)
			return new Object[] { new Beteiligter(bb), (Double) null,
					(Double) null, new Boolean(bb.isAnWiderklageBeteiligt()) };
		else
			return null;
	}

	/**
	 * Konvertiert ein Array Object[4] aus Beteiligter, Double, Double, Boolean
	 * in einen BaumbachBeteiligten und liest dabei alles aus.
	 * 
	 * @param obb
	 *            ein Array Object[4] aus Beteiligter, Double, Double, Boolean
	 * @return einen BaumbachBeteiligten
	 */
	private BaumbachBeteiligter konvertiere(Object[] obb) {
		if (obb != null && obb.length == 4) {
			double sw = obb[1] == null ? 0.0 : ((Double) obb[1]).doubleValue();
			double unt = obb[2] == null ? 0.0 : ((Double) obb[2]).doubleValue();
			boolean isWk = obb[3] == null ? false : ((Boolean) obb[3])
					.booleanValue();
			return new BaumbachBeteiligter((Beteiligter) obb[0], sw, unt, isWk);
		} else
			return null;
	}

	/**
	 * Konvertiert eine BaumbachBeteiligtenListe in ein Array Object[][] aus
	 * Beteiligtendaten
	 * 
	 * @param bbl
	 *            eine BaumbachBeteiligtenListe
	 * @return ein Array Object[][] aus Beteiligtendaten
	 */
	private Object[][] konvertiere(BaumbachBeteiligtenListe bbl) {
		if (bbl != null) {
			Object[][] zwischen = new Object[bbl.size()][4];
			for (int index = 0; index < bbl.size(); index++) {
				Object[] o = konvertiere(bbl.get(index));
				if (o != null)
					zwischen[index] = o;
			}
			return zwischen;
		} else
			return null;
	}

	/**
	 * Konvertiert ein Array Object[][] aus Beteiligtendaten in eine
	 * BaumbachBeteiligtenListe
	 * 
	 * @param obbl
	 *            ein Array Object[][] aus Beteiligtendaten, bei dem für jede
	 *            Zeile die Anforderungen von
	 *            {@link eu.gronos.kostenrechner.BaumbachAssistentDialog#konvertiere(Object[])
	 *            konvertiere(Object[])} erfüllt sind
	 * @return eine BaumbachBeteiligtenListe
	 */
	private BaumbachBeteiligtenListe konvertiere(Object[][] obbl) {
		if (obbl != null) {
			BaumbachBeteiligtenListe zwischen = new BaumbachBeteiligtenListe();
			for (Object[] o : obbl) {
				BaumbachBeteiligter bb = konvertiere(o);
				if (bb != null)
					zwischen.add(bb);
			}
			return zwischen;
		} else
			return null;
	}
}
