/**
 * EmailUtils.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.Desktop;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;

/**
 * Hält statische Methoden für Feedback-E-Mails bereit
 *
 * @author Peter Schuster (setrok)
 * @date 28.03.2016
 *
 */
class EmailUtils {

	private static final String E_MAIL_FEHLER_URI_SYNTAX = "URI-Syntax falsch: ";
	private static final String E_MAIL_FEHLER_ENCODING = "Zeichenkodierung nicht unterstützt: ";
	private static final String E_MAIL_FEHLER_TEXT = "Fehler beim Aufruf des Standard-E-Mail-Programms: ";
	private static final String E_MAIL_FEHLER_TITLE = "E-Mail-Fehler";
	/**
	 * PETER_EMAIL speichert die E-Mail-Adresse des Programmierers als String.
	 */
	static final String PETER_EMAIL = "webmaster@kostentenor.de";
	/**
	 * SUBJECT2 speichert die URI-Einleitung für die Betreffzeile als String.
	 */
	protected static final String SUBJECT2 = "?subject=";
	/**
	 * BODY2 speichert die URI-Einleitung für den E-Mail-Text als String.
	 */
	protected static final String BODY2 = "&BODY=";
	/**
	 * MAILTO speichert die URI-Einleitung für E-Mails als String.
	 */
	static final String MAILTO = "mailto:";

	/**
	 * Die Methode dient dazu, zum E-Mail-Versand eine Zeichenkette URL-konform
	 * zu kodieren.
	 * 
	 * @param str
	 *            die zu kodierende Zeichenkette
	 * @url "http://www.2ality.com/2010/12/simple-way-of-sending-emails-in-java.html"
	 * @return die kodierte Zeichenkette
	 */
	protected static final String urlEncode(String str) {
		try {
			return URLEncoder.encode(str, "UTF-8").replace("+", "%20");
		} catch (UnsupportedEncodingException uee) {
			throw new RuntimeException(uee);
		}
	}

	/**
	 * emailSchreiben lässt eine Beschwerde-E-Mail im Standard-E-Mail-Client
	 * schreiben.
	 * 
	 * @param subject
	 *            die Betreffzeile der E-Mail als String
	 * @param body
	 *            der Inhalt der E-Mail als String
	 * 
	 * @url "http://stackoverflow.com/questions/12449654/clickable-email-address-as-a-jlabel-in-java"
	 * @url "http://www.2ality.com/2010/12/simple-way-of-sending-emails-in-java.html"
	 */
	static final void emailSchreiben(final String subject, final String body) {
		// final URI mailtoURI = new URI("mailTo", PETER_EMAIL + subject
		// + body, null);
		try {
			final URI mailtoURI = new URI(EmailUtils.MAILTO + EmailUtils.PETER_EMAIL + EmailUtils.SUBJECT2
					+ urlEncode(subject) + EmailUtils.BODY2 + urlEncode(body));
			// Launches the mail composing window of the user default mail
			// client
			Desktop.getDesktop().mail(mailtoURI);
			// den Browser aufzurufen dauert länger:
			// Desktop.getDesktop().browse(mailtoURI);
		} catch (UnsupportedEncodingException uee) {
			Kostenrechner.zeigeFehler(E_MAIL_FEHLER_TITLE,
					E_MAIL_FEHLER_TEXT + E_MAIL_FEHLER_ENCODING + uee.getLocalizedMessage(), uee);
		} catch (URISyntaxException use) {
			Kostenrechner.zeigeFehler(E_MAIL_FEHLER_TITLE,
					E_MAIL_FEHLER_TEXT + E_MAIL_FEHLER_URI_SYNTAX + use.getLocalizedMessage(), use);
		} catch (IOException ioe) {
			Kostenrechner.zeigeFehler(E_MAIL_FEHLER_TITLE, E_MAIL_FEHLER_TEXT + ioe.getLocalizedMessage(), ioe);
		}
	}

	/**
	 * NN speichert TODO als String.
	 */
	static final String NEUER_ABSATZ = "\n\n";
}
