/**
 * FehlerDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Eine Klasse, die Fehlermeldungen ausführlicher darstellt.
 *
 * @author Peter Schuster (setrok)
 * @date 26.08.2014
 *
 */
class FehlerDialog extends HinzufuegenDialog<Boolean> {

	private static final long serialVersionUID = 306283812188663774L;
	private String meldung;
	// private StackTraceElement[] stackTrace;
	// private String localizedMessage;
	// this.stackTrace = stackTrace;
	// this.localizedMessage = localizedMessage;
	private Throwable throwable;
	static final String FEHLER_BLANKO = "Fehler";

	/**
	 * emailSchreibenTenorFalsch lässt eine Beschwerde-E-Mail im
	 * Standard-E-Mail-Client über {@link TenorUtils.emailSchreiben(String,
	 * String)} schreiben. Dabei wird der Text des Tenors und noch XML-Daten
	 * mitgeschickt.
	 * 
	 */
	private final AbstractAction emailSchreibenFehler = new BeschriebeneAktion("Beim Autor beschweren",
			"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail an den Programmierer zu verfassen</html>",
			KeyEvent.VK_E, "e-mail") {

		private static final long serialVersionUID = -4095891796997774737L;

		@Override
		public void actionPerformed(ActionEvent e) {
			final String subject = "Fehlermeldung aus kostentenor.de " + Kostenrechner.KOSTENRECHNER_VERSION;
			// Im E-Mail-Text wird der Stacktrace mitgeschickt.
			final String fehler = getFehlerText().toString();
			final String body = "Bitte beschreiben Sie, wie es zu dem Fehler gekommen ist: " + EmailUtils.NEUER_ABSATZ
					+ "Debug-Informationen für den Programmierer" + EmailUtils.NEUER_ABSATZ + fehler;
			EmailUtils.emailSchreiben(subject, body);
		}
	};

	/**
	 * Der Konstruktor initialisiert den Dialog und gibt eine StackTrace aus.
	 * 
	 * @param title
	 *            den Titel des Dialogs
	 * @param meldung
	 *            die Fehlermeldung
	 * @param throwable
	 *            das {@link Throwable} (Oberklasse von {@link Exception}), das
	 *            geworfen wurde.
	 */
	public FehlerDialog(String title, String meldung, Throwable throwable) {
		super(Kostenrechner.selbst, title);
		this.meldung = meldung;
		this.throwable = throwable;
		super.setOkButtonToBeFocussed(true);
		throwable.printStackTrace();
	}

	/**
	 * Die Methode füllt das contentPane mit einem Icon-Label, einer JTextArea
	 * für die eigentliche Meldung und einer JTextArea im JScrollPane für den
	 * StackTrace.
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		JPanel meldungPane = new JPanel();
		meldungPane.setLayout(new FlowLayout(FlowLayout.LEFT));

		JLabel lblIcon = new JLabel(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-rot.png")));
		JTextArea jtaMeldung = new JTextArea(meldung);
		jtaMeldung.setRows(2);
		jtaMeldung.setEditable(false);
		jtaMeldung.setOpaque(false);

		StringBuffer sb = getFehlerText();
		JTextArea jtaStackTrace = new JTextArea();
		jtaStackTrace.setText(sb.toString());
		jtaStackTrace.setCaretPosition(0);
		jtaStackTrace.setForeground(Color.RED);
		jtaStackTrace.setEditable(false);
		jtaStackTrace.setRows(7);
		JScrollPane scpStackTrace = new JScrollPane(jtaStackTrace);
		scpStackTrace.setVisible(Kostenrechner.debug);

		meldungPane.add(lblIcon);
		meldungPane.add(jtaMeldung);
		contentPanel.add(meldungPane,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		contentPanel.add(scpStackTrace,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.BOTH, true));

		getButtonPane().add(new JButton(emailSchreibenFehler));
		addToRootInputMap(emailSchreibenFehler);
	}

	/**
	 * Die Methode dient dazu, die Message, LocalizedMessage und den StackTrace
	 * eines Fehlers in einen StringBuffer zu schreiben.
	 * 
	 * @return LocalizedMessage und StackTrace als StringBuffer
	 */
	StringBuffer getFehlerText() {
		StringBuffer sb = new StringBuffer();
		if (throwable != null) {
			sb.append(throwable.getMessage() + "\n");
			sb.append(throwable.getLocalizedMessage() + "\n");
			for (StackTraceElement ste : throwable.getStackTrace()) {
				sb.append("\t" + ste.toString() + "\n");
			}
		}
		return sb;
	}

	@Override
	protected boolean parseEingabe() {
		return true;
	}

	@Override
	protected Boolean baueRueckgabewert() {
		return Boolean.TRUE;
	}

}
