/**
 * StreitwertReduktionTest.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.KeyboardFocusManager;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.print.PrinterException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTable.PrintMode;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.BadLocationException;
import javax.swing.text.NumberFormatter;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.xml.bind.JAXB;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import eu.gronos.kostenrechner.VerfahrensDatenContainer.ForderungsStaffelung;
import eu.gronos.kostenrechner.VerfahrensDatenContainer.Teilklageruecknahme;

/**
 * Mit dem Tool k&ouml;nnen Hauptsache- und Kostentenor in einfachen F&auml;llen
 * einer Teilklager&uuml;cknahme, einem streitgenossenschaftlichem
 * Teilunterliegen und einer Staffelung von Hauptklage und Hilfsaufrechnung
 * erzeugt werden.
 * 
 * @author Peter Schuster (setrok)
 * @version 0.5.5
 * 
 * @since 0.5.5 (2016-03-28) E-Mail-Schaltfläche im TenorDialog, die auch den
 *        Tenor und die XML-Daten mitschickt; E-Mail-Menüpunkt im Hauptmenü;
 *        E-Mail-Versand bei Fehlermeldungen.
 * 
 * @since 0.5.2 und 0.5.3 Fehlerbeseitigung (0.5.4 übersprungen)
 * 
 * @since 0.5.1 Fehlerbeseitigung. {@link GebuehrenTabelle} fit für {@link JAXB}
 *        gemacht.
 * 
 * @since 0.5 Auch bei Teilklagerücknahme wird auf Wunsch § 92 II ZPO beachtet.
 * 
 * @since 0.4.8 VerfahrensDatenContainer durch einen neuen ersetzt. Beim Laden
 *        von Xjustiz wird auch GenusNumerus der Beteiligten-Auswahllisten auf
 *        den anderen Registerkarten richtig gesetzt. SKKTX-Dateien können als
 *        Kommandozeilen-Parameter übergeben werden. Der
 *        {@link XmlTransferHandler} nimmt jetzt zum Drag'n'Drop auch
 *        skktx-Dateien und von allen Registerkarten.
 * 
 * @since 0.4.7 Laden und speichern möglich (skktx - xml).
 * 
 * @since 0.4.6 Breite des TenorDialogs auf 120 Spalten begrenzt. Links sind
 *        klickbar.
 * 
 * @since 0.4.5 Vorläufige Vollstreckbarkeit auch bei der Baumbach'schen
 *        Kostenformel
 * 
 * @since 0.4.2 Neuer "Keine-Umsatzsteuer"-Tatbestand. In der Gebührentabelle
 *        können Einträge verschoben werden. Langfassung der GPL kann angezeigt
 *        werden.
 * 
 * @since 0.4.1 Bugfix: Drag'n'Drop funktionierte nicht mehr mit
 *        Auslagetatbeständen. AngriffsVerteidigungsmittelStaffelung und
 *        StreitwertReduktion haben Vollstreckbarkeitstenor.
 * 
 * @since 0.4 Neue Klassen VollstreckbarkeitsObjekt und VollstreckbarkeitsListe.
 *        Freigabe unter GPL 3.0. Annahme der GPL zum Programmstart n&ouml;tig.
 * 
 * @since 0.3.1 Bugfix beim Setzen des Streitwerts durch BaumbachAssistenten.
 *        Durch FocusListener wird jede Änderung des niedrigeren Streitwerts
 *        nach Letztlich Obsiegt zurück geschrieben. Bessere Wortstellung im
 *        Hauptsachetenor.
 * 
 * @since 0.3.0 GebührenTatbestand als Basisklasse war in StreitwertReduktion
 *        und Kindklassen noch nicht angekommen; jetzt aber schon. :) Neues
 *        Interface Tabulierend und neue Klasse TabulierendTableModel, von
 *        AngriffsVerteidigungsmittelStaffelung, BaumbachFormel und
 *        StreitwertReduktion unterstützt. Der tabellarische Teil der Begründung
 *        des Tenors wird jetzt auch als Tabelle eingefügt.
 * 
 * @since 0.2.9 (2014-09-19) Eingabefelder für Zahlen jetzt rechtsbündig. Bei
 *        Xjustiz-Import (und bei sonstiger Verwendung des Assistenten) werden
 *        auch die Streitwert-/Obsiegen-Felder bei Teilklagerücknahme und bei
 *        der freien Gebührenberechnung gefüllt.
 * 
 * @since 0.2.8 (2014-08-21) Tenorierend aufgeteilt in simpleres Ober-Interface
 *        Tenorierend und abgeleitetes Interface UrteilsTenorierend mit dem
 *        gewohnten Umfang. Umsatzsteuer auf Rechtsanwaltsverg&uuml;tung wird
 *        unterst&uuml;tzt. Neue Funktionalit&auml;t Geb&uuml;hrenaufstellung.
 *        Neue Klasse GebuehrenErhoehungsTatbestand. Neue Klasse FehlerDialog.
 *        Die OK-Schaltfläche kann automatisch den Focus bekommen. Mehr
 *        GebührenTatbestände. Neue Klasse BeschriebeneAktion, um bei
 *        AbstractActions bereits im Konstruktor values zu setzen. Bei
 *        UrteilsTenorierend kann man eine Begründung implementieren, was
 *        AngriffsVerteidigungsmittelStaffelung, BaumbachFormel und
 *        StreitwertReduktion auch machen. In TenorDialog kann man übers
 *        Kontextmenü die Begründung einfügen. Reset in
 *        AngriffsVerteidigungsmittelStaffelung eingebaut, damit die Berechnung
 *        auch noch ein zweites Mal (nach Änderung der Werte) klappt.
 * 
 * @since 0.2.7 (2014-08-17) Freien Gebührenrechner hinzugefügt. Die
 *        GebührenRechner geben eine Liste der Gebührensprünge als ArrayList
 *        heraus. Tabelle kann gedruckt und in die Zwischenablage kopiert
 *        werden.
 * 
 * @since 0.2.6 (2014-08-14) Konstanten extrahiert. Test-Unit erweitert.
 *        GebuehrenTabelle aufgeteilt in eine erweiterte
 *        ArrayList&lt;GebuehrenZeile&gt; namens GebuehrenTabelle und einen
 *        GebuehrenRechner als Oberklasse mit Berechnungsmethoden. Rundung beim
 *        Vergleich der identischen Quoten. GitterBeutelBeschränkungen zu einem
 *        Quasi-Singleton gemacht und leicht angepasst.
 * 
 * @since 0.2.5 (2014-08-11) Tooltips für Spaltenüberschriften.
 * 
 * @since 0.2.2 (2014-08-08) Tabellen weitergehend editierbar: Renderer und
 *        Editor für Widerklage (Boolean), DefaultEditor für Beteiligte. Mehr
 *        Plausis. Die Gesamtschuldnerschaft selbst kann entfernt werden.
 * 
 * @since 0.2.1 (2014-08-05) Bugfix im GttmNachher-Kontextmenü und
 *        setViewportFill bei allen Tabellen. ACTION_COMMAND_KEY in Actions.
 *        Kommentiert. JFormattedTextFields im Hauptfenster werden jetzt auch
 *        auf Focus markiert. Streitgenossen in Tabellen sind jetzt auch
 *        nachträglich editierbar.
 * 
 * @since 0.2.0 (2014-08-04) Verschieben von Gebühren in andere Tabelle über
 *        Kontextmenü und über Drag'n'Drop. Xjustiz-Import per Drag'n'Drop.
 * 
 * @since 0.1.9 (2014-08-02) BeteiligtenRenderer lässt lfdNr in Ruhe, wenn schon
 *        gesetzt. Registerkarte 3 (Hilfsaufrechnung) aktiviert und gefüllt.
 *        Staffelung mit Klage und Hilfsaufrechnung kann jetzt berechnet werden.
 *        Bezeichner in ComboBox doch wieder mit Artikel. Tabelle für
 *        Forderungen auch editierbar.
 * 
 * @since 0.1.8 (2014-07-29) GebührensatzRenderer. Beim Zurücksetzen wird das
 *        Anzeigen von Widerklagen jetzt auch richtig zurückgesetzt.
 *        Design-Fehler beim Skalieren im BaumbachAssistentDialog behoben. Mehr
 *        Plausis im Xjustiz-Import über BaumbachAssistenten. Neue Klassen
 *        Forderung, KlageForderung und AufrechnungsForderung. Neue Interfaces
 *        RowList und RowHandler. BeteiligtenRenderer und
 *        SimpleBeteiligtenRenderer zusammengeführt.
 * 
 * @since 0.1.7 (2014-07-24) Neuer Hilfetext, "darüber hinaus" auch mit
 *        geschachtelten Gesamtschuldnerschaften. Kontextmenü und
 *        "Eintrag entfernen" auch in den Gebühren-Tabellen. Komfortablere
 *        Eingabe der Beteiligtenzahlen im BaumbachAssistenten durch JSpinner.
 *        Beim Aufruf ist die Übergabe des Dateinamens einer XJustiz-Datei
 *        möglich.
 * 
 * @since 0.1.6 (2014-07-20) EuroRenderer auch im Hauptfenster, TenorDialog
 *        erweitert HinzufügenDialog (schönere Buttons, Fenstergrößere besser
 *        angepasst), Abfrage vor Beenden. BaumbachFormel sortiert die
 *        Klageabweisung (und "im Übrigen") besser in den Hauptsachetenor.
 *        "darüber hinaus" jetzt im Hauptsachetenor und im Kostentenor, aber nur
 *        für einfache BaumbachBeteiligten. Gleichlautende Kostenverteilung wird
 *        in der Baumbachformel im selben Satz ausgegeben. Kontextmenüs für die
 *        Beteiligten-JTables. Alles zurücksetzen setzt auch Widerklagen wieder
 *        zurück.
 * 
 * @since 0.1.5 (2014-07-17) BaumbachAssistent mit XJustiz-Import
 * 
 * @since 0.1.4 (2014-07-08) XJustiz-Import
 * 
 * @since 0.1.3 (2014-07-04) Gesamtschuldnerschaft extrahieren
 * 
 * @since 0.1.2 (2014-07-01) Tastenkürzel für Tabs, Tabellen in Tab-Reihenfolge
 * 
 * @since 0.1.1 (2014-06-29) Schönere Dialoge durch neue Oberklasse
 *        HinzufuegenDialog&lt;E&gt;. Bei Vorliegen einer Widerklage wird bei
 *        Streitwertbeschluss auch eine Summe gebildet, sofern Klage und
 *        Widerklage nicht wirtschaftlich identisch sind. Mehr Tastenkürzel und
 *        Kurzbeschreibungen.
 * 
 * @todo TODO Benutzung von Höhe der Inanspruchnahme, Streitwert etc sowie von
 *       Unterliegen, Obsiegen und Begründetheit vereinheitlichen.
 * 
 * @todo TODO Benutzung von long und double vereinheitlichen (eher Richtung
 *       long)
 * 
 * @todo TODO die ArrayList soll auch die Nummerierung und die Referenzen
 *       verwalten
 * 
 * @todo TODO Tastaturbedienung im Hauptfenster und allen Dialogen,
 *       Tabulatorreihenfolge
 */
public class Kostenrechner extends JFrame {
	private static final String KOSTENRECHNER_TITLE = "Schusters kleiner Kostentenor - Zivilprozessuale Kostenberechnung";
	static final String KOSTENRECHNER_VERSION = " v0.5.5";
	private static final long serialVersionUID = 1240469319173224562L;

	static final BeteiligtenRenderer BETEILIGTEN_RENDERER = new BeteiligtenRenderer();
	static final BeteiligtenRenderer BETEILIGTEN_RENDERER_OHNE_LFDNR = new BeteiligtenRenderer(true);
	private static final TooltipHeaderRenderer HEADER_RENDERER = new TooltipHeaderRenderer();
	private static final WiderklageTableRendererEditor WIDERKLAGE_RENDERER_EDITOR = new WiderklageTableRendererEditor();
	private static final ForderungenRenderer FORDERUNGEN_RENDERER = new ForderungenRenderer();
	static final EuroRenderer EURO_RENDERER = new EuroRenderer();
	private static final NumberFormat FORMAT = NumberFormat.getInstance();
	private static final NumberFormatter FORMATTER = new NumberFormatter(FORMAT);
	private static final NumberFormatter FORMATTER_INT = new NumberFormatter(NumberFormat.getIntegerInstance());
	private static final String SUMME_EUR = "Summe: %,.2f EUR";
	private static final EuroEditor EURO_EDITOR = new EuroEditor(FORMATTER);
	private static final String FORDERUNGEN_POPUPLABEL = "Angriffs-/Verteidigungsmittel...";
	private static final String NACHHER_POPUPLABEL = "Gebühren nachher...";
	private static final String VORHER_POPUPLABEL = "Gebühren vorher...";
	private static final String DRITTWIDERBEKLAGTE_POPUPLABEL = "Drittwiderbeklagte...";
	private static final String KLAEGER_POPUPLABEL = "Kläger...";
	private static final String BEKLAGTE_POPUPLABEL = "Beklagte...";
	private static final String RECHNER_POPUPLABEL = "Gebührentatbestände...";
	private static final String STREITWERTE_POPUPLABEL = "Streitwerte...";
	private static final String FEHLER_NIX_PARTEIEN_LANG = "Die Berechnung erfordert einen Kläger und mindestens einen Beklagten.";
	private static final String FEHLER_NIX_PARTEIEN = "Fehler: Keine Kläger und/oder Beklagten vorhanden";
	private static final String FEHLER_KEINE_GEBUEHREN_LANG = "Bitte geben Sie zunächst ein welche Gebühren vor und/oder nach der Streitwertreduktion angefallen waren.";
	private static final String FEHLER_KEINE_GEBUEHREN = "Fehler: Keine Gebühren angefallen!";
	private static final String FEHLER_GESAMTSCHULDNERSCHAFT = "Fehler beim Hinzufügen einer Gesamtschuldnerschaft";
	private static final String FEHLER_GESAMTSCHULD_NIX_LOESCHEN = "Einträge können nicht entfernt werden, wenn bereits eine Gesamtschuldnerschaft vorhanden ist.";
	private static final String FEHLER_BERECHNUNG = "Fehler bei der Berechnung";
	private static final String FEHLER_DRUCKEN = "Fehler beim Drucken";
	private static final int GEBUEHREN_ANERKENNTNIS = 101;
	private static final int GEBUEHREN_SAEUMNIS = 102;
	private static final int GEBUEHREN_STANDARD = 103;
	private static final int GEBUEHREN_2INSTANZ = 104;
	/**
	 * Der boolesche Wert gibt an, ob der {@link FehlerDialog} auch den
	 * {@link Throwable#getStackTrace() StackTrace} anzeigen soll.
	 */
	static boolean debug = false;
	/**
	 * Der int mit dem Namen ctrlMask ermöglicht, dass für Windows u.a. die
	 * Strg-Taste bei Tastenkombinationen auf dem Mac durch Cmd ersetzt wird.
	 */
	static int ctrlMask = ActionEvent.CTRL_MASK;
	private JPanel contentPane;
	private JTable tabKlaeger;
	private JTable tabBeklagte;
	private JTable tabDrittwiderbeklagte;
	private JTable tabNachher;
	private JTable tabVorher;
	private GebuehrenTatbestandTableModel gttmNachher;
	private GebuehrenTatbestandTableModel gttmVorher;
	// private JTextPane txtpnKostentenor_1;
	private JFormattedTextField ftfStreitwertVorher;
	private JFormattedTextField ftfStreitwertNachher;
	private JFormattedTextField ftfObsiegen;

	JComboBox<Beteiligter> cbBeklagte;
	JComboBox<Beteiligter> cbKlaeger;
	// private JComboBox<GebuehrenTatbestand> cbGebuehrenListe;
	private BeteiligtenTableModel btmKlaeger;
	private BeteiligtenTableModel btmDrittwiderbeklagte;
	private BeteiligtenTableModel btmBeklagte;
	private boolean widerKlageAnzeigen;
	// private JCheckBox cbKlAlsGesamtschuldner;
	// private JCheckBox cbBkAlsGesamtschuldner;
	// private JFormattedTextField ftfAuslagenHoehe;
	private JPanel pnHilfsaufrechnung;
	private JTabbedPane tabbedPane;
	private JButton btnWiderklageAnzeigen;
	private JScrollPane scpDwbk;
	private JButton btnDrittwiderbeklagterNeu;
	private JButton btnKlDwblGesamtsch;
	protected boolean wirtschaftlichIdentisch = false;
	private JTextPane txtpnPanic;
	private JScrollPane scpPanic;
	static Kostenrechner selbst = null;
	private JCheckBoxMenuItem mi92;
	private JPopupMenu pmBeklagtenKontextmenu;
	private JPopupMenu pmKlaegerKontextmenu;
	private JPopupMenu pmDwbkKontextmenu;
	private JPopupMenu pmGttmVorher;
	private JPopupMenu pmGttmNachher;
	private JComboBox<Beteiligter> cbAngriffsKlaeger;
	private JComboBox<Beteiligter> cbVerteidigungsBeklagter;
	private ForderungenTableModel ftm;
	private JTable tbForderungen;
	private JPopupMenu pmForderungen;
	private JTable tabStreitwertTabelle;
	private JFormattedTextField ftfStreitwert;
	private JTable tabGebuehrenTatbestaende;
	private GebuehrenTatbestandTableModel gttmRechner;
	private JPopupMenu pmGttmRechner;
	private JTextField tfGebuehrenSumme;
	private JPopupMenu pmStreitwert;

	// private SimpleAttributeSet attr;

	/**
	 * tabNext: AbstractAction, die verhindert, dass die JTable die
	 * Tabulartortaste "frisst".
	 * 
	 * 
	 * @url "http://www.coderanch.com/t/342895/GUI/java/JTable-focus" * aus: Big
	 *      Moose Saloon, A friendly place for programming greenhorns!
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	AbstractAction aaTabNext = new AbstractAction() {
		private static final long serialVersionUID = -6095777609039741786L;

		public void actionPerformed(ActionEvent ae) {
			KeyboardFocusManager.getCurrentKeyboardFocusManager().focusNextComponent();
		}
	};

	/**
	 * AbstractAction, die verhindert, dass die JTable Umsch+Tabulartortaste
	 * "frisst".
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	AbstractAction aaTabBefore = new AbstractAction() {
		private static final long serialVersionUID = -244188598744791079L;

		public void actionPerformed(ActionEvent ae) {
			KeyboardFocusManager.getCurrentKeyboardFocusManager().focusPreviousComponent();
		}
	};
	/**
	 * Der AbstractAction mit dem Namen widerKlageAktivierenAction ruft einen
	 * kleinen HinzufuegenDialog für die Widerklage auf.
	 */
	private final AbstractAction widerKlageAktivierenAction = new BeschriebeneAktion("Widerklagen aktivieren",
			"<html><b>Widerklage</b><br>Aktiviert die Eingabeschnittstellen für Widerklagen</html>", KeyEvent.VK_W,
			"widerklage-aktivieren") {
		private static final long serialVersionUID = 7753262660993032427L;

		/**
		 * Ein kleiner HinzufuegenDialog, der auch nach wirtschaftlicher
		 * Identität fragt
		 */
		HinzufuegenDialog<boolean[]> hd = new HinzufuegenDialog<boolean[]>(Kostenrechner.this,
				"Widerklage aktivieren") {
			private static final long serialVersionUID = -9111863513392608271L;
			private JCheckBox chbWiderklage;
			private JCheckBox chbWirtschaftlichIdentisch;

			@Override
			protected boolean[] baueRueckgabewert() {
				return new boolean[] { chbWiderklage.isSelected(), chbWirtschaftlichIdentisch.isSelected() };
			}

			@Override
			protected void fuelleContentPane() {
				chbWiderklage = new JCheckBox("Widerklage aktivieren", true);
				contentPanel.add(chbWiderklage,
						GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
				chbWirtschaftlichIdentisch = new JCheckBox("Widerklage wirtschaftlich identisch", false);
				chbWirtschaftlichIdentisch.setToolTipText("<html><b>§ 45 Abs. 1 Satz 3 GKG</b>"
						+ "<br>Betreffen die Ansprüche im Fall des Satzes 1 oder 2 (Klage und Widerklage, hilfsweise geltend gemachte Ansprüche)"
						+ "<br> denselben Gegenstand, ist nur der Wert des höheren Anspruchs maßgebend.</html>");
				contentPanel.add(chbWirtschaftlichIdentisch,
						GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
			}

			@Override
			protected boolean parseEingabe() {
				// Keine Plausibilitäten zu beachten diesmal
				return true;
			}

		};

		@Override
		public void actionPerformed(ActionEvent ae) {
			boolean[] b = hd.showDialog();
			if (b == null)
				return;
			setWiderKlageAnzeigen(b[0]);
			btmKlaeger.setWiderKlageAnzeigen(b[0]);
			btmDrittwiderbeklagte.setWiderKlageAnzeigen(b[0]);
			btmBeklagte.setWiderKlageAnzeigen(b[0]);
			wirtschaftlichIdentisch = b[1];
		}
	};
	/**
	 * Der AbstractAction mit dem Namen neueGebuehrNachher fügt einen neuen
	 * Gebührentatbestand in die Tabelle für die nach Gebühren ein, die nach der
	 * Streitwertreduktion entstanden sind.
	 */
	private final AbstractAction neueGebuehrNachherAction = new BeschriebeneAktion("Neue Gebühr (nachher)",
			"<html><b>Neue Gebühr (nachher)</b><br>Fügt einen neuen Gebührentatbestand in die Tabelle für die nach Gebühren ein, die nach der Streitwertreduktion entstanden sind.</html>",
			KeyEvent.VK_N, "gebuehr-nachher") {
		private static final long serialVersionUID = 7170038539836176410L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			GebuehrenTatbestand gt = new GebuehrHinzufuegenDialog(Kostenrechner.this, "Neue Gebühr (nachher)")
					.showDialog();
			// erzeugeGebuehrenTatbestand(cbGebuehrenListe,
			// ftfAuslagenHoehe);
			if (gt != null)
				gttmNachher.addRow(gt);
		}
	};
	/**
	 * Der AbstractAction mit dem Namen neueGebuehrVorher fügt einen neuen
	 * Gebührentatbestand in die Tabelle der vor der Streitwertreduktion
	 * entstandenen ein
	 */
	private final AbstractAction neueGebuehrVorherAction = new BeschriebeneAktion("Neue Gebühr (vorher)",
			"<html><b>Neue Gebühr (vorher)</b><br>Fügt einen neuen Gebührentatbestand in die Tabelle der vor der Streitwertreduktion entstandenen ein.</html>",
			KeyEvent.VK_V, "gebuehr-vorher") {
		private static final long serialVersionUID = 7170038539836176410L;

		@Override
		public void actionPerformed(ActionEvent e) {
			GebuehrenTatbestand gt = new GebuehrHinzufuegenDialog(Kostenrechner.this, "Neue Gebühr (vorher)")
					.showDialog();
			// erzeugeGebuehrenTatbestand(cbGebuehrenListe,
			// ftfAuslagenHoehe);
			if (gt != null)
				gttmVorher.addRow(gt);
		}

	};
	/**
	 * Der AbstractAction mit dem Namen berechneQuotenMethode erzeugt den
	 * Kostentenor nach der Quotenmethode
	 */
	private final AbstractAction quotenMethodeAction = new BeschriebeneAktion("Berechne (Quotenmethode)",
			"<html><b>Berechne Quotenmethode</b><br>Erzeugt den Kostentenor nach der Quotenmethode.</html>",
			KeyEvent.VK_Q, "quotenmethode-tenor") {
		private static final long serialVersionUID = -7701909446599455048L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			if (!pruefeGebuehrenTabellen()) {
				Throwable e = new IllegalArgumentException(FEHLER_KEINE_GEBUEHREN_LANG);
				new FehlerDialog(FEHLER_KEINE_GEBUEHREN, e.getLocalizedMessage(), e).showDialog();
				return;
			}
			long[] zwischen = parseVorherNachherLetztendlich();
			StreitwertReduktion quoten = new QuotenMethode((Beteiligter) cbKlaeger.getSelectedItem(),
					(Beteiligter) cbBeklagte.getSelectedItem(), gttmVorher.toArray(), gttmNachher.toArray(),
					new long[] { zwischen[0], zwischen[1] }, zwischen[2], mi92.isSelected());
			final VerfahrensDatenContainer container = erstelleContainer();
			new TenorDialog(Kostenrechner.this, quoten, container).showDialog();
		}
	};
	/**
	 * Der AbstractAction mit dem Namen berechneMehrkostenMethode erzeugt den
	 * Kostentenor nach der Mehrkostenmethode.
	 */
	private final AbstractAction mehrkostenMethodeAction = new BeschriebeneAktion("Berechne (Mehrkostenmethode)",
			"<html><b>Berechne Mehrkostenmethode</b><br>Erzeugt den Kostentenor nach der Mehrkostenmethode.</html>",
			KeyEvent.VK_M, "mehrkostenmethode-tenor") {
		private static final long serialVersionUID = -5053128615441379386L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			if (!pruefeGebuehrenTabellen()) {
				Throwable e = new IllegalArgumentException(FEHLER_KEINE_GEBUEHREN_LANG);
				new FehlerDialog(FEHLER_KEINE_GEBUEHREN, e.getLocalizedMessage(), e).showDialog();
				return;
			}
			long[] zwischen = parseVorherNachherLetztendlich();
			StreitwertReduktion mehrkosten = new MehrkostenMethode((Beteiligter) cbKlaeger.getSelectedItem(),
					(Beteiligter) cbBeklagte.getSelectedItem(), gttmVorher.toArray(), gttmNachher.toArray(),
					new long[] { zwischen[0], zwischen[1] }, zwischen[2], mi92.isSelected());
			final VerfahrensDatenContainer container = erstelleContainer();
			new TenorDialog(Kostenrechner.this, mehrkosten, container).showDialog();
		}
	};
	/**
	 * Der AbstractAction mit dem Namen baumbachBerechnen erzeugt einen
	 * Kostentenor nach der Baumbach'schen Kostenformel.
	 */
	private final AbstractAction baumbachAction = new BeschriebeneAktion("Baumbach'schen Kostentenor berechnen",
			"<html><b>Baumbach'schen Kostentenor berechnen</b><br>Erzeugt einen Kostentenor nach der Baumbach'schen Kostenformel.</html>",
			KeyEvent.VK_T, "baumbach-tenor") {
		private static final long serialVersionUID = 8105321861085009090L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			if (!pruefeBaumbachTabellen()) {
				Throwable e = new IllegalArgumentException(FEHLER_NIX_PARTEIEN_LANG);
				new FehlerDialog(FEHLER_NIX_PARTEIEN, e.getLocalizedMessage(), e).showDialog();
				return;
			}
			BaumbachBeteiligtenListe bts = null;
			/*
			 * Wenn Dwbk vorhanden, dann die Werte aller drei Tabellen
			 * zusammenfügen, ansonsten nur die von Klägern und Beklagten.
			 */
			if (btmDrittwiderbeklagte.getRowCount() < 1)
				bts = new BaumbachBeteiligtenListe(btmKlaeger.getAllValues(), btmBeklagte.getAllValues());
			else
				bts = new BaumbachBeteiligtenListe(btmKlaeger.getAllValues(), btmDrittwiderbeklagte.getAllValues(),
						btmBeklagte.getAllValues());
			BaumbachFormel bf;
			try {
				bf = new BaumbachFormel(bts, mi92.isSelected(), wirtschaftlichIdentisch);
				new TenorDialog(Kostenrechner.this, bf, erstelleContainer()).showDialog();
			} catch (IllegalArgumentException iae) {
				new FehlerDialog(FehlerDialog.FEHLER_BLANKO, FEHLER_BERECHNUNG, iae).showDialog();
			}
		}
	};
	/**
	 * Die AbstractAction ist eine Action zum Ermitteln des Tenors auf
	 * Registerkarte Hilfsaufrechnung (3).
	 */
	private AbstractAction angriffsVerteidigungsmittelStaffelungAction = new BeschriebeneAktion("Tenor ermitteln",
			"Ermittelt den Kostentenor und Streitwert bei Staffelung von Hauptsache und Aufrechnungen.", KeyEvent.VK_T,
			"aufrechnung-tenor") {
		private static final long serialVersionUID = 2748367225462023438L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			if (cbAngriffsKlaeger == null || cbVerteidigungsBeklagter == null || ftm == null || mi92 == null)
				return;
			Beteiligter kl = (Beteiligter) cbAngriffsKlaeger.getSelectedItem();
			Beteiligter bk = (Beteiligter) cbVerteidigungsBeklagter.getSelectedItem();
			ArrayList<Forderung> forderungen = ftm.getAllValues();
			boolean b92ii = mi92.isSelected();
			try {
				AngriffsVerteidigungsmittelStaffelung avs = new AngriffsVerteidigungsmittelStaffelung(forderungen, kl,
						bk, b92ii);
				final VerfahrensDatenContainer container = erstelleContainer();
				new TenorDialog(Kostenrechner.this, avs, container).showDialog();
			} catch (IllegalArgumentException e) {
				new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
				// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
				// e.printStackTrace();
			}
		}

	};
	/**
	 * Die AbstractAction fügt ein neues Angriffs- oder Verteidigungsmittel zur
	 * Tabelle hinzu
	 */
	private AbstractAction neueForderungAction = new BeschriebeneAktion("Angriffs-/Verteidigungsmittel hinzufügen",
			"Fügt ein neues Angriffs- oder Verteidigungsmittel zur Tabelle hinzu", KeyEvent.VK_H, "forderung-neu") {
		private static final long serialVersionUID = -25587531439255593L;

		@Override
		public void actionPerformed(ActionEvent e) {
			Forderung forderung = new ForderungHinzufuegen(Kostenrechner.this).showDialog();
			if (forderung != null)
				ftm.addRow(forderung);
		}

	};

	/**
	 * Der AbstractAction mit dem Namen bkGesamtschuldner fügt eine
	 * gesamtschuldnerische Verurteilung zur Tabelle der Beklagten hinzu.
	 */
	private AbstractAction bkGesamtschuldnerAction = new BeschriebeneAktion("Gesamtschuldnerische Beklagte neu",
			"<html><b>Gesamtschuldnerische Beklagten neu</b><br>Fügt eine gesamtschuldnerische Verurteilung zur Tabelle der Beklagten hinzu.</html>",
			KeyEvent.VK_G, "gesamtschuldnerschaft-beklagte") {
		private static final long serialVersionUID = 3348162188610732190L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			try {
				GesamtschuldnerDialog gesamtschuldnerDialog = new GesamtschuldnerDialog(Kostenrechner.this,
						btmBeklagte.getAllValues());
				BaumbachGesamtschuldnerschaft bt = gesamtschuldnerDialog.showDialog();
				if (bt != null)
					btmBeklagte.addRow(bt);
			} catch (IllegalArgumentException iae) {
				// Meldung, wenn noch keine Beteiligten in Tabelle
				new FehlerDialog(FEHLER_GESAMTSCHULDNERSCHAFT, iae.getLocalizedMessage(), iae).showDialog();
			}
		}

	};
	/**
	 * Der AbstractAction mit dem Namen klDwbkGesamtsch fügt eine
	 * gesamtschuldnerische Verurteilung auf Kläger-/Drittwiderbeklagtenseite
	 * hinzu.
	 */
	private final AbstractAction klDwbkGesamtschAction = new BeschriebeneAktion("Gesamtschuldnerische Kl./Drwbk. neu",
			"<html><b>Gesamtschuldnerische Kläger und Drittwiderbeklagte neu</b>"
					+ "<br>Fügt eine gesamtschuldnerische Verurteilung auf Kläger-/Drittwiderbeklagtenseite hinzu."
					+ "<br>Vorher bitte die einzelnen Beteiligten hinzufügen.</html>",
			KeyEvent.VK_L, "beklagte-gesamtschuldner") {
		private static final long serialVersionUID = -6637467913455344657L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			try {
				GesamtschuldnerDialog gesamtschuldnerDialog = new GesamtschuldnerDialog(Kostenrechner.this,
						btmKlaeger.getAllValues(), btmDrittwiderbeklagte.getAllValues());
				BaumbachGesamtschuldnerschaft bt = gesamtschuldnerDialog.showDialog();
				if (bt != null)
					btmDrittwiderbeklagte.addRow(bt);
			} catch (IllegalArgumentException iae) {
				// Meldung, wenn noch keine Beteiligten in Tabelle
				new FehlerDialog(FEHLER_GESAMTSCHULDNERSCHAFT, iae.getLocalizedMessage(), iae).showDialog();
			}
		}

	};
	/**
	 * Der AbstractAction mit dem Namen gesamtschuldnerschaftExtrahieren geht
	 * nach Beteiligtentypen aufgeteilt durch die Beteiligten einer Tabelle,
	 * bestimmt den gesamtschuldnerischen Sockelbetrag bestimmen, erzeugt dann
	 * BaumbachGesamtschuldner und zieht von allen Beteiligten diesen
	 * Sockelbetrag ab. {@link KeyEvent#VK_V}
	 */
	private final AbstractAction gesamtschuldnerschaftExtrahierenAction = new BeschriebeneAktion(
			"Gesamtschuldnerische Verurteilung ermitteln",
			"Geht die Beteiligten einer Seite durch und guckt, ob sie z.T. gesamtschuldnerisch verurteilt wurden",
			KeyEvent.VK_V, "extrahiere-gesamtschuld") {
		private static final long serialVersionUID = -2835734772816646654L;

		@Override
		public void actionPerformed(ActionEvent e) {
			BeteiligtenTableModel btm = btmBeklagte;
			// TODO ALLES PRÜFEN!
			BaumbachBeteiligtenListe beteiligte = btm.getAllValues();
			if (beteiligte == null || beteiligte.size() < 2)
				return;
			BaumbachGesamtschuldnerschaft gesamtschuldnerschaft = beteiligte
					.extrahiereGesamtschuldnerschaft(btm.getBeteiligtenTyp());
			if (gesamtschuldnerschaft != null) {
				btm.addRow(gesamtschuldnerschaft);
				// Rekursion, sofern Rückgabe nicht null
				actionPerformed(e);
				// Und das Model muss was von der Änderung mitbekommen.
				btm.fireTableDataChanged();
			}
		}

	};
	/**
	 * Der AbstractAction mit dem Namen gesamtschuldnerschaftDwbkExtrahieren
	 * geht die Beteiligten einer Seite durch und guckt, ob sie z.T.
	 * gesamtschuldnerisch verurteilt wurden. {@link KeyEvent#VK_W}
	 */
	private final AbstractAction gesamtschuldnerschaftDwbkExtrahierenAction = new BeschriebeneAktion(
			"Gesamtschuldnerische Verurteilung (Widerbeklagte) ermitteln",
			"Geht die Beteiligten einer Seite durch und guckt, ob sie z.T. gesamtschuldnerisch verurteilt wurden",
			KeyEvent.VK_W, "extrahiere-widerklage-gesamtschuld") {
		private static final long serialVersionUID = 8105321861085009090L;

		@Override
		public void actionPerformed(ActionEvent e) {
			// TODO ALLES PRÜFEN!
			// WICHTIG: Vorher echte Kopie anlegen!
			BaumbachBeteiligtenListe beteiligte = new BaumbachBeteiligtenListe();
			beteiligte.addAll(btmKlaeger.getAllValues());
			beteiligte.addAll(btmDrittwiderbeklagte.getAllValues());
			if (beteiligte == null || beteiligte.size() < 2)
				return;
			BaumbachGesamtschuldnerschaft gesamtschuldnerschaft = beteiligte
					.extrahiereGesamtschuldnerschaft(btmDrittwiderbeklagte.getBeteiligtenTyp());
			if (gesamtschuldnerschaft != null) {
				btmDrittwiderbeklagte.addRow(gesamtschuldnerschaft);
				System.out.println("Schleife");
				// Rekursion, sofern Rückgabe nicht null: Bei Dwbk nicht gut...
				actionPerformed(e);
				// Und das Model muss was von der Änderung mitbekommen.
				btmKlaeger.fireTableDataChanged();
				btmDrittwiderbeklagte.fireTableDataChanged();
			}
		}

	};

	/**
	 * Der AbstractAction mit dem Namen beklagterNeu fügt einen neuen
	 * (einzelnen) Beklagten hinzu. {@link KeyEvent#VK_B}
	 */
	private final AbstractAction neuerBeklagterAction = new BeschriebeneAktion("Beklagter neu",
			"<html><b>Beklagter neu</b><br>Fügt einen neuen (einzelnen) Beklagten hinzu.</html>", KeyEvent.VK_B,
			"beklagter-neu") {
		private static final long serialVersionUID = -6505409262360418812L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			beteiligtenHinzufuegen(btmBeklagte);
		}

	};
	/**
	 * Der AbstractAction mit dem Namen drittwiderbeklagterNeu fügt einen neuen
	 * Drittwiderbeklagten hinzu. {@link KeyEvent#VK_D}
	 */
	private final AbstractAction neuerDrittwiderbeklagterAction = new BeschriebeneAktion("Drittwiderbeklagter neu",
			"<html><b>Drittwiderbeklagter neu</b><br>Fügt einen neuen Drittwiderbeklagten hinzu.</html>", KeyEvent.VK_D,
			"drittwiderbeklagter-neu") {
		private static final long serialVersionUID = -3610891307085826193L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			beteiligtenHinzufuegen(btmDrittwiderbeklagte);
		}

	};
	/**
	 * Der AbstractAction mit dem Namen klgrNeu fügt einen neuen Kläger hinzu.
	 * {@link KeyEvent#VK_K}
	 */
	private final AbstractAction neuerKlaegerAction = new BeschriebeneAktion("Kläger neu",
			"<html><b>Kläger neu</b><br>Fügt einen neuen Kläger hinzu</html>", KeyEvent.VK_K, "klaeger-neu") {
		private static final long serialVersionUID = -2835734772816646654L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			beteiligtenHinzufuegen(btmKlaeger);
		}

	};

	/**
	 * Der AbstractAction dient dazu, im Menü aufgerufen zu werden. Sie setzt
	 * alle Tabellen wieder in den Ursprungszustand zurück und schaltet dazu
	 * auch die Anzeige von Widerklagen aus. {@link KeyEvent#VK_Z}
	 */
	private final AbstractAction tabellenZuruecksetzenAction = new BeschriebeneAktion("Alle Einträge zurücksetzen",
			"Leert die Tabelle der Gebührentatbestände und die Tabellen der Beteiligten", KeyEvent.VK_Z,
			"neue-berechnung", KeyStroke.getKeyStroke(KeyEvent.VK_N, ctrlMask)) {
		private static final long serialVersionUID = 8105321861085009090L;

		@Override
		public void actionPerformed(ActionEvent e) {
			setzeStandardGebuehren(false);
			/*
			 * widerklageAnzeigen muss richtig gesetzt werden, bevor die
			 * Tabellen gesetzt werden
			 */
			setWiderKlageAnzeigen(false);
			setzeBaumbachTabellen();
			setzeForderungsTabellen();
			setzeGebuehrenRechnerTabelle(0);
		}

	};
	/**
	 * Der AbstractAction mit dem Namen standardSzenario leert die Tabelle mit
	 * den Gebührentatbeständen und \nfüllt sie mit der Konstellation
	 * Teilklagerücknahme vor Anfall der Terminsgebühr. {@link KeyEvent#VK_F}
	 */
	AbstractAction klageruecknahmeTerminAction = new BeschriebeneAktion("Fall Teilklagerücknahme vor/im Termin",
			"Leert die Tabelle mit den Gebührentatbeständen und \nfüllt sie mit der Konstellation Teilklagerücknahme vor Anfall der Terminsgebühr",
			KeyEvent.VK_F, "standard-gebuehren") {
		private static final long serialVersionUID = -7826777167427931422L;

		@Override
		public void actionPerformed(ActionEvent e) {
			setzeStandardGebuehren(true);
			tabbedPane.setSelectedIndex(0);
		}
	};
	/**
	 * Der AbstractAction mit dem Namen beenden beendet Kostentenor.
	 * {@link KeyEvent#VK_E}
	 */
	AbstractAction beendenAction = new BeschriebeneAktion("Ende", "Beendet Kostentenor", KeyEvent.VK_E, "quit") {
		private static final long serialVersionUID = -2268718567794925181L;

		@Override
		public void actionPerformed(ActionEvent e) {
			Kostenrechner.beendeKostenrechner();
		}

	};

	/** AbstractAction, nur für den Assistenten! {@link KeyEvent#VK_A} */
	AbstractAction baumbachAssistentAction = new BeschriebeneAktion("Beteiligte anlegen über Assistenten",
			"Der Assistent vereinfacht das Hinzufügen von Beteiligten durch eine geführte Abfrage der Daten.",
			KeyEvent.VK_A, "baumbach-assistent") {
		private static final long serialVersionUID = 3372100502718641247L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			BaumbachAssistentDialog bad = new BaumbachAssistentDialog(Kostenrechner.this, widerKlageAnzeigen);
			VerfahrensDatenContainer container = bad.showDialog();
			leseContainer(container);
			// setzeBaumbachTabellen(vdc);
			// setzeAlleStreitwertTextfelder(vdc);
		}
	};

	/**
	 * Der AbstractAction importiert eine XJustiz-Datei und ruft dann den
	 * Baumbach-Assistenten auf. {@link KeyEvent#VK_X}
	 */
	AbstractAction xmlLesenAction = new BeschriebeneAktion("XJustiz importieren über Assistenten",
			"Liest einen XJustiz-Datensatz (XML) ein", KeyEvent.VK_X, "xjustiz",
			KeyStroke.getKeyStroke(KeyEvent.VK_O, ctrlMask + ActionEvent.SHIFT_MASK)) {
		private static final long serialVersionUID = 3372100502718641247L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			File auswahl = null;
			VerfahrensDatenContainer container = null;
			BaumbachAssistentDialog bad = null;
			JFileChooser jfc = new JFileChooser();
			jfc.setFileFilter(new FileNameExtensionFilter("XML", "xml"));
			int ergebnis = jfc.showOpenDialog(null);
			if (ergebnis == JFileChooser.APPROVE_OPTION) {
				auswahl = jfc.getSelectedFile();
				container = xJustizLesen(auswahl);
			} else {
				bad = new BaumbachAssistentDialog(Kostenrechner.this, widerKlageAnzeigen);
				container = bad.showDialog();
				// setzeAlleStreitwertTextfelder(vdc);
			}
			// setzeBaumbachTabellen(vdc);
			leseContainer(container);
		}
	};

	/**
	 * skktxSpeichern ist ne Action, die speichert die Daten aller
	 * Registerkarten für spätere Berechnungen ab
	 */
	private Action skktxSpeichern = new BeschriebeneAktion("Berechnung speichern",
			"Speichert die Daten aller Registerkarten für spätere Berechnungen ab", KeyEvent.VK_S, "save",
			KeyStroke.getKeyStroke(KeyEvent.VK_S, ctrlMask)) {
		private static final long serialVersionUID = 8642800626569995070L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			final VerfahrensDatenContainer container = erstelleContainer();
			final BerechnungLadenSpeichernDialog db = new BerechnungLadenSpeichernDialog();
			db.speicherXmlDialog(Kostenrechner.this, container);
		}
	};

	/**
	 * skktxLaden = Lädt die Daten aller Registerkarten aus einer früher
	 * gespeicherten Berechnung (Strg+O)
	 */
	private Action skktxLaden = new BeschriebeneAktion("Berechnung öffnen",
			"Lädt die Daten aller Registerkarten aus einer früher gespeicherten Berechnung", KeyEvent.VK_F, "open",
			KeyStroke.getKeyStroke(KeyEvent.VK_O, ctrlMask)) {

		private static final long serialVersionUID = -2225698692818848010L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			final BerechnungLadenSpeichernDialog db = new BerechnungLadenSpeichernDialog();
			final VerfahrensDatenContainer container = db.ladeXmlDialog(Kostenrechner.this);
			if (container != null)
				leseContainer(container);
		}
	};

	/**
	 * emailSchreibenXML lässt eine Beschwerde-E-Mail im Standard-E-Mail-Client
	 * über {@link TenorUtils.emailSchreiben(String, String)} schreiben. Dabei
	 * werden XML-Daten mitgeschickt. {@link KeyEvent.VK_T}
	 * 
	 */
	private final AbstractAction emailSchreibenXML = new BeschriebeneAktion("Fehlermeldung an Autor",
			"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail mit Fehlerbehebungsdaten an den Programmierer zu verfassen</html>",
			KeyEvent.VK_T, "e-mail-plus") {

		private static final long serialVersionUID = -4300105137631815898L;

		@Override
		public void actionPerformed(ActionEvent e) {
			final VerfahrensDatenContainer container = erstelleContainer();
			final String subject = "Fehlverhalten in kostentenor.de " + Kostenrechner.KOSTENRECHNER_VERSION;
			// Wäre schön, wenn noch XML-Daten mitgeschickt würden.
			final String xmlString = BerechnungLadenSpeichernDialog.toXmlString(container);
			final String body = "Wenn mit den eingegebenen Daten ein Fehler aufgetreten sein sollte, beschreiben Sie das Fehlverhalten des Programms bitte!\n\n"
					+ xmlString;
			EmailUtils.emailSchreiben(subject, body);
		}
	};

	/**
	 * emailSchreibenAllgemein lässt eine Beschwerde-E-Mail im
	 * Standard-E-Mail-Client über {@link TenorUtils.emailSchreiben(String,
	 * String)} schreiben. {@link KeyEvent.VK_E}
	 * 
	 */
	private final AbstractAction emailSchreibenAllgemein = new BeschriebeneAktion("E-Mail an Autor",
			"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail an den Programmierer zu verfassen</html>",
			KeyEvent.VK_E, "e-mail") {

		private static final long serialVersionUID = 618282849118692220L;

		@Override
		public void actionPerformed(ActionEvent e) {
			final String subject = "Kommentar zu kostentenor.de " + Kostenrechner.KOSTENRECHNER_VERSION;
			final String body = "Mein Kommentar zu Schusters kleinem Kostentenor:\n\n";
			EmailUtils.emailSchreiben(subject, body);
		}
	};

	/**
	 * Der AllzweckHorcher FENSTER_HORCHER speichert Routinen, die beim
	 * Schließen des Fensters eine Abfrage ausführen.
	 */
	private static final WindowListener FENSTER_HORCHER = new AllzweckHorcher() {

		@Override
		public void windowClosing(WindowEvent e) {
			Kostenrechner.beendeKostenrechner();
		}

		@Override
		public void windowClosed(WindowEvent e) {
			Kostenrechner.beendeKostenrechner();
		}

	};
	/**
	 * Soll jede Änderung des niedrigeren Streitwerts nach Letztlich Obsiegt
	 * zurückschreiben.
	 */
	private final FocusListener streitwertListener = new FocusListener() {

		private long alterStreitwert = -1;

		/**
		 * Wenn das Textfeld den Focus bekommt: Wert zwischenspeichern, um eine
		 * Änderung erkennen zu können
		 */
		@Override
		public void focusGained(FocusEvent fe) {
			if (ftfStreitwertNachher == null)
				return;
			String s = ftfStreitwertNachher.getText();
			alterStreitwert = -1;
			/* Erst gucken, ob's ein long ist */
			try {
				alterStreitwert = Long.parseLong(s);
			} catch (NumberFormatException nfe) {
				/* Sonst gucken, ob's ein double ist */
				try {
					alterStreitwert = (long) Double.parseDouble(s);
				} catch (NumberFormatException nfe2) {
					alterStreitwert = -1;
				}
			}
		}

		/**
		 * Wenn das Textfeld den Focus verliert: Gucken, ob sich der Wert seit
		 * Focus-Erlangung geändert hat. Wenn ja: auch das letztliche Obsiegen
		 * ändern.
		 */
		@Override
		public void focusLost(FocusEvent fe) {
			if (ftfStreitwertNachher == null || ftfObsiegen == null)
				return;
			long neuerStreitwert = alterStreitwert;
			String s = ftfStreitwertNachher.getText();
			/* Erst gucken, ob's ein long ist */
			try {
				neuerStreitwert = Long.parseLong(s);
			} catch (NumberFormatException nfe) {
				/* Sonst gucken, ob's ein double ist */
				try {
					neuerStreitwert = (long) Double.parseDouble(s);
				} catch (NumberFormatException nfe2) {
					neuerStreitwert = alterStreitwert;
				}
			}
			if (neuerStreitwert != alterStreitwert) {
				ftfObsiegen.setText(ftfStreitwertNachher.getText());
			}
		}

	};

	/**
	 * Der AbstractAction mit dem Namen ueberAction zeigt Informationen über das
	 * Programm an. {@link KeyEvent#VK_U}
	 */
	private AbstractAction ueberAction = new BeschriebeneAktion("Über " + Kostenrechner.KOSTENRECHNER_TITLE,
			"Zeigt Informationen über das Programm an.", KeyEvent.getExtendedKeyCodeForChar('Ü'), "about") {
		private static final long serialVersionUID = -7370964238649394812L;

		@Override
		public void actionPerformed(ActionEvent e) {
			Kostenrechner.ueberKostenrechner();
		}

	};
	/**
	 * Der AbstractAction kann JPopupMenus hinzugefügt werden. Soll für die
	 * Beteiligtentabellen und die Gebührentabellen einsetzbar sein.
	 * {@link KeyEvent#VK_E}
	 */
	private final AbstractAction eintragEntfernenAktion = new BeschriebeneAktion("Eintrag entfernen",
			"Entfernt den markierten Eintrag aus der Tabelle", KeyEvent.VK_E, "remove") {
		private static final long serialVersionUID = 8001075205760187469L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			Object o = ae.getSource();
			if (!(o instanceof JMenuItem)) {
				return;
			}
			// Herausfinden, welche Tabelle ruft
			JPopupMenu popup = (JPopupMenu) ((JMenuItem) o).getParent();
			if (popup.getLabel() == VORHER_POPUPLABEL)
				entferneTabellenEintrag(tabVorher, gttmVorher);
			if (popup.getLabel() == NACHHER_POPUPLABEL)
				entferneTabellenEintrag(tabNachher, gttmNachher);
			if (popup.getLabel() == BEKLAGTE_POPUPLABEL)
				entferneBeteiligtenEintrag(tabBeklagte, btmBeklagte);
			if (popup.getLabel() == KLAEGER_POPUPLABEL)
				entferneBeteiligtenEintrag(tabKlaeger, btmKlaeger);
			if (popup.getLabel() == DRITTWIDERBEKLAGTE_POPUPLABEL)
				entferneBeteiligtenEintrag(tabDrittwiderbeklagte, btmDrittwiderbeklagte);
			if (popup.getLabel() == FORDERUNGEN_POPUPLABEL)
				entferneTabellenEintrag(tbForderungen, ftm);
			if (popup.getLabel() == RECHNER_POPUPLABEL)
				entferneTabellenEintrag(tabGebuehrenTatbestaende, gttmRechner);
		}
	};

	/**
	 * Der AbstractAction mit dem Namen gebuehrenTbVerschiebenAction entfernt
	 * den markierten Eintrag aus dieser Tabelle und fügt ihn der jeweils
	 * anderen hinzu. Soll ins Kontextmenü. {@link KeyEvent#VK_T}
	 */
	private final AbstractAction gebuehrenTbVerschiebenAction = new BeschriebeneAktion(
			"Gebühr in die andere Tabelle verschieben",
			"Entfernt den markierten Eintrag aus dieser Tabelle und fügt ihn der jeweils anderen hinzu.", KeyEvent.VK_T,
			"verschiebe-gebuehr") {
		private static final long serialVersionUID = 2748367225462023438L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			Object o = ae.getSource();
			if (!(o instanceof JMenuItem)) {
				return;
			}
			// Herausfinden, welche Tabelle ruft
			JPopupMenu popup = (JPopupMenu) ((JMenuItem) o).getParent();
			if (popup.getLabel() == VORHER_POPUPLABEL)
				verschiebeGebuehrenTatbestand(tabVorher, tabNachher);
			if (popup.getLabel() == NACHHER_POPUPLABEL)
				verschiebeGebuehrenTatbestand(tabNachher, tabVorher);
		}
	};

	/**
	 * Der AbstractAction mit dem Namen neueGebuehrNachher fügt einen neuen
	 * Gebührentatbestand in die Tabelle für die Gebühren zur freien Berechnung
	 * ein. {@link KeyEvent#VK_R}
	 */
	private final AbstractAction neueRvgVerguetungAction = new BeschriebeneAktion("Neue Vergütung nach RVG",
			"<html><b>Neue Gebühr</b><br>Fügt einen neuen Vergütungstatbestand nach RVG in die Tabelle für die Gebühren zur freien Berechnung ein.</html>",
			KeyEvent.VK_R, "gebuehr-rvg") {
		private static final long serialVersionUID = -7542463988551747020L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			GebuehrenTatbestand gt = new GebuehrHinzufuegenDialog(Kostenrechner.this, "Neue Vergütung (RVG)",
					AnwaltsGebuehrenTabelle.getAllGebuehrenTatbestaende()).showDialog();
			if (gt != null)
				gttmRechner.addRow(gt);
		}
	};
	/**
	 * Der AbstractAction mit dem Namen neueGebuehrNachher fügt einen neuen
	 * Gebührentatbestand in die Tabelle für die Gebühren zur freien Berechnung
	 * ein. {@link KeyEvent#VK_N}
	 */
	private final AbstractAction neueGkgGebuehrAction = new BeschriebeneAktion("Neue Gebühr nach GKG",
			"<html><b>Neue Gebühr</b><br>Fügt einen neuen Gebührentatbestand nach GKG in die Tabelle für die Gebühren zur freien Berechnung ein.</html>",
			KeyEvent.VK_N, "gebuehr-gkg") {
		private static final long serialVersionUID = -7542463988551747020L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			GebuehrenTatbestand gt = new GebuehrHinzufuegenDialog(Kostenrechner.this, "Neue Gebühr (GKG)",
					GerichtsGebuehrenTabelle.getAllGebuehrenTatbestaende()).showDialog();
			if (gt != null)
				gttmRechner.addRow(gt);
		}
	};
	/**
	 * gebuehrenBerechnung erstellt eine Aufstellung der Gebühren und ihrer
	 * Summe zu dem ausgewählten Streitwert. {@link KeyEvent#VK_E}
	 */
	private final AbstractAction gebuehrenAufstellungAction = new BeschriebeneAktion(
			"Ausführliche Gebührenaufstellung erstellen",
			"Erstellt eine Aufstellung der Gebühren und ihrer Summe zu dem ausgewählten Streitwert.", KeyEvent.VK_E,
			"gebuehren-aufstellung") {
		private static final long serialVersionUID = 4793754967839528087L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			requestFocus();
			String streitwert = "";
			Tenorierend t = null;
			if (ftfStreitwert != null && gttmRechner != null) {
				try {
					streitwert = ftfStreitwert.getText();
					t = new GebuehrenBerechnungsAufstellung(gttmRechner.getAllValues(), Long.parseLong(streitwert));
					final VerfahrensDatenContainer container = erstelleContainer();
					new TenorDialog(Kostenrechner.this, t, container).showDialog();
				} catch (NumberFormatException e) {
					new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
				} catch (NullPointerException | IllegalArgumentException e) {
					new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
				}
			}
		}
	};
	/**
	 * Der AbstractAction mit dem Namen standardSzenario leert die Tabelle mit
	 * den Gebührentatbeständen des Rechners und füllt sie mit der Konstellation
	 * Gebühren bei Säumnis. {@link KeyEvent#VK_S}
	 */
	private final AbstractAction saeumnisGebuehrenAction = new BeschriebeneAktion("Gebühren bei Säumnis",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation Gebühren bei Säumnis",
			KeyEvent.VK_S, "saeumnis-gebuehren") {
		private static final long serialVersionUID = -5007484323415490185L;

		@Override
		public void actionPerformed(ActionEvent e) {
			setzeGebuehrenRechnerTabelle(GEBUEHREN_SAEUMNIS);
			tabbedPane.setSelectedIndex(3);
		}
	};
	/**
	 * Der AbstractAction mit dem Namen standardSzenario leert die Tabelle mit
	 * den Gebührentatbeständen des Rechners und füllt sie mit der Konstellation
	 * Gebühren bei Anerkenntnis. {@link KeyEvent#VK_G}
	 */
	private final AbstractAction anerkenntnisGebuehrenAction = new BeschriebeneAktion("Gebühren bei Anerkenntnis",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation Gebühren bei Anerkenntnis",
			KeyEvent.VK_G, "anerkenntnis-gebuehren") {

		private static final long serialVersionUID = -4172206260165408350L;

		@Override
		public void actionPerformed(ActionEvent e) {
			setzeGebuehrenRechnerTabelle(GEBUEHREN_ANERKENNTNIS);
			tabbedPane.setSelectedIndex(3);
		}
	};
	/**
	 * normaleGebuehren leert die Tabelle mit den Gebührentatbeständen des
	 * Rechners und füllt sie mit der Konstellation und füllt sie mit der
	 * Konstellation eines normalen erstinstanzlichen Verfahrens
	 * {@link KeyEvent#VK_UNDEFINED}
	 */
	private final AbstractAction normaleGebuehrenAction = new BeschriebeneAktion("Gebühren im Normalfall",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation eines normalen erstinstanzlichen Verfahrens",
			KeyEvent.VK_H, "normal-gebuehren") {
		private static final long serialVersionUID = 5567272748853539208L;

		@Override
		public void actionPerformed(ActionEvent e) {
			setzeGebuehrenRechnerTabelle(GEBUEHREN_STANDARD);
			tabbedPane.setSelectedIndex(3);
		}
	};
	/**
	 * mitInstanzGebuehren leert die Tabelle mit den Gebührentatbeständen des
	 * Rechners und füllt sie mit der Konstellation und füllt sie mit der
	 * Konstellation eines normalen erstinstanzlichen Verfahrens mit
	 * anschließender Berufung {@link KeyEvent#VK_UNDEFINED}
	 */
	private final AbstractAction mitInstanzGebuehrenAction = new BeschriebeneAktion("Gebühren einschließlich Berufung",
			"Leert die Tabelle mit den Gebührentatbeständen des Rechners und \nfüllt sie mit der Konstellation eines normalen erst- und zweitinstanzlichen Verfahrens",
			KeyEvent.VK_B, "mit-instanz-gebuehren") {
		private static final long serialVersionUID = 1670640402679506238L;

		@Override
		public void actionPerformed(ActionEvent e) {
			setzeGebuehrenRechnerTabelle(GEBUEHREN_2INSTANZ);
			tabbedPane.setSelectedIndex(3);
		}
	};
	/**
	 * Der AllzweckHorcher mit dem Namen aktualisierungsHorcher stellt sicher,
	 * dass das Streitwert-JFormattedTextfield und die dazugehörige JTable immer
	 * denselben Streitwert haben.
	 */
	private final AllzweckHorcher aktualisierungsHorcher = new AllzweckHorcher() {
		@Override
		public void keyReleased(KeyEvent ke) {
			Object source = ke.getSource();
			if (ke.getKeyCode() == KeyEvent.VK_ENTER) {
				if (source instanceof JTable)
					ftfAnTableAnpassen((JTable) source);
				else if (source instanceof JFormattedTextField)
					tableAnFtfAnpassen((JFormattedTextField) source);
			}
		}

		@Override
		public void keyTyped(KeyEvent ke) {
			keyReleased(ke);
		}

		@Override
		public void mouseClicked(MouseEvent me) {
			Object source = me.getSource();
			if (source instanceof JTable && me.getClickCount() > 1)
				ftfAnTableAnpassen((JTable) source);
		}

		@Override
		public void focusLost(FocusEvent fe) {
			Object source = fe.getSource();
			if (source instanceof JTable)
				ftfAnTableAnpassen((JTable) source);
			else if (source instanceof JFormattedTextField)
				tableAnFtfAnpassen((JFormattedTextField) source);
		}

		@Override
		public void valueChanged(ListSelectionEvent lse) {
			Object source = lse.getSource();
			if (source instanceof ListSelectionModel)
				ftfAnTableAnpassen(tabStreitwertTabelle);
		}

		@Override
		public void tableChanged(TableModelEvent tme) {
			errechneGebuehrenSumme();
		}

		private void tableAnFtfAnpassen(JFormattedTextField ftf) {
			long streitwert = 0;
			if (ftf.isEditValid())
				try {
					ftf.commitEdit();
					Object value = ftf.getValue();
					if (value instanceof Long)
						streitwert = ((Long) value).longValue();
					else if (value instanceof Double)
						streitwert = Math.round(((Double) value).doubleValue());
					else if (value instanceof Float)
						streitwert = Math.round(((Float) value).doubleValue());
					else if (value instanceof Integer)
						streitwert = ((Integer) value).longValue();
					else if (value instanceof Number)
						streitwert = Math.round(((Number) value).doubleValue());
					else
						return;
					// streitwert = Long.parseLong(ftf.getText());
					int zeile = ((StreitwertTableModel) tabStreitwertTabelle.getModel()).rowAtStreitwert(streitwert);
					if (zeile > -1)
						tabStreitwertTabelle.changeSelection(zeile, -1, false, false);
					errechneGebuehrenSumme();
				} catch (ParseException | NumberFormatException e) {
					new FehlerDialog(FehlerDialog.FEHLER_BLANKO, "Falsches Eingabeformat!", e).showDialog();
				}
		}

		private void ftfAnTableAnpassen(JTable table) {
			final int selectedRow = table.getSelectedRow();
			if (selectedRow < 0)
				return;
			final Long valueAt = (Long) table.getModel().getValueAt(selectedRow, 0);
			ftfStreitwert.setText(valueAt.toString());
			errechneGebuehrenSumme();
		}

	};
	/**
	 * Die AbstractAction druckeTabelle druckt den Inhalt der Streitwerttabelle
	 */
	private final AbstractAction druckeSwTabelleAction = new BeschriebeneAktion("Tabelle drucken",
			"Druckt den Inhalt der Streitwerttabelle", KeyEvent.VK_D, "print-table",
			KeyStroke.getKeyStroke(KeyEvent.VK_P, ctrlMask)) {
		private static final long serialVersionUID = 8046359053768482197L;

		@Override
		public void actionPerformed(ActionEvent e) {
			try {
				boolean fertig = tabStreitwertTabelle.print(PrintMode.FIT_WIDTH, new MessageFormat("Streitwerttabelle"),
						new MessageFormat(KOSTENRECHNER_TITLE + KOSTENRECHNER_VERSION + "\t Seite {0}"));
				if (fertig)
					;
			} catch (PrinterException pe) {
				new FehlerDialog(FehlerDialog.FEHLER_BLANKO, FEHLER_DRUCKEN, pe).showDialog();
			}
		}
	};
	/**
	 * kopiereSwTabelle speichert eine Aktion als AbstractAction, die den Inhalt
	 * der Streitwerttabelle in die Zwischenablage kopiert.
	 */
	private final AbstractAction kopiereSwTabelleAction = new BeschriebeneAktion("Streitwerttabelle kopieren",
			"Kopiert den Inhalt der Streitwerttabelle in die Zwischenablage", KeyEvent.VK_K, "copy-table",
			KeyStroke.getKeyStroke(KeyEvent.VK_C, ctrlMask)) {
		private static final long serialVersionUID = 5567272748853539208L;

		@Override
		public void actionPerformed(ActionEvent ae) {
			// Werte zwischenspeichern
			ListSelectionModel model = tabStreitwertTabelle.getSelectionModel();
			int selection = tabStreitwertTabelle.getSelectedRow();
			// Alles markieren
			tabStreitwertTabelle.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
			tabStreitwertTabelle.selectAll();
			// kopieren down vote
			Action copy = tabStreitwertTabelle.getActionMap().get("copy");
			ActionEvent nev = new ActionEvent(tabStreitwertTabelle, ActionEvent.ACTION_PERFORMED, "");
			copy.actionPerformed(nev);
			// Werte wieder zurücksetzen
			tabStreitwertTabelle.setSelectionModel(model);
			tabStreitwertTabelle.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			tabStreitwertTabelle.changeSelection(selection, -1, false, false);
		}
	};
	/**
	 * lizenzAnzeigenAktion zeigt die Kurzform der Lizenzbedingungen an als
	 * Action.
	 */
	private final Action lizenzAnzeigenAktion = new BeschriebeneAktion("Lizenz",
			"Zeigt die Kurzform der Lizenzbedingungen an.", KeyEvent.VK_L) {

		private static final long serialVersionUID = -798271727754981044L;

		@Override
		public void actionPerformed(ActionEvent e) {
			new LizenzAkzeptanzHinzufuegen(Kostenrechner.this, false).showDialog();
		}
	};

	/**
	 * Launch the application.
	 * 
	 */
	public static void main(final String[] args) {
		// Herausfinden, auf welchem OS ich laufe!
		if (System.getProperty("os.name").contains("Windows") && "\\".equals(System.getProperty("file.separator")))
			setWindowsLookAndFeel();
		if (System.getProperty("os.name").contains("Mac OS X"))
			setMacOsLookAndFeel();
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					// Kostenrechner frame = new Kostenrechner();
					// frame.setVisible(true);
					selbst = new Kostenrechner();
					debug = parseDebug(args);
					selbst.setVisible(true);
					/*
					 * XJustiz-Datei kann als Parameter übergeben werden. Nach
					 * Aufbau des JFrame wird dann der XML-Parser und dann der
					 * Assistent gestartet.
					 */
					File xml = null;
					if (args != null && args.length > 0) {
						xml = parseDateiname(args);
						selbst.nimmXmlDatei(xml);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Constructs a new frame that is initially invisible. This constructor sets
	 * the component's locale property to the value returned by
	 * JComponent.getDefaultLocale.
	 */
	public Kostenrechner() {
		// selbst = this;
		super(KOSTENRECHNER_TITLE + KOSTENRECHNER_VERSION);
		/*
		 * Dank an JavaBeginners, Mit einem NumberFormatter lässt sich ein
		 * TextFeld so formatieren, dass nur Zahlen als Eingabe akzeptiert
		 * werden. Die Grundlage hierfür bildet ein JFormattedTextField.
		 * 
		 * @url
		 * "http://www.javabeginners.de/Swing/Swing-Elemente/Textkomponenten/Textfeld_fuer_Zahlen.php"
		 */
		FORMAT.setGroupingUsed(false);
		// formatter.setAllowsInvalid(false);
		// setTitle(KOSTENRECHNER_TITLE + KOSTENRECHNER_VERSION);
		// setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		// setBounds(100, 100, 849, 617);
		setSize(849, 617);
		setLocationByPlatform(true);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		contentPane.setLayout(new BorderLayout(0, 0));

		tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		contentPane.add(tabbedPane);

		fuelleStreitwertReduktionPanel();

		fuelleBaumbachPanel();

		fuelleHilfsaufrechnungPanel();

		fuelleGebuehrenRechnerPanel();

		fuelleHilfePanel();

		baueMenuBar();

		definiereIcons();

		setTransferHandler(new XmlTransferHandler(this));

		addWindowListener(FENSTER_HORCHER);

		copyrightNoergler();
	}

	/**
	 * Wenn die Lizenzvereinbarung noch nicht akzeptiert wurde, dann nörgeln.
	 * Wenn nicht akzeptiert wird, dann Programm beenden. Sonst Vermerk in
	 * Registry/Preferences und Programmstart.
	 */
	void copyrightNoergler() {
		if (!LizenzAkzeptanzHinzufuegen.ausRegistryLesen()) {
			boolean akzeptiert = new LizenzAkzeptanzHinzufuegen(this).showDialog() == Boolean.TRUE ? true : false;
			if (akzeptiert) {
				LizenzAkzeptanzHinzufuegen.inRegistrySchreiben(true);
			} else {
				System.exit(1);
			}
		} else {
		}
	}

	/**
	 * Die Methode baut eine Menüzeile
	 */
	private void baueMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		JMenu menu = new JMenu("Aktionen");
		menu.setMnemonic(KeyEvent.VK_A);
		menuBar.add(menu);

		// TODO Tastenkürzel doppelt?
		JMenuItem miZuruecksetzen = new JMenuItem(tabellenZuruecksetzenAction);
		menu.add(miZuruecksetzen);
		mi92 = new JCheckBoxMenuItem("§ 92 Abs. 2 ZPO anwenden", true);
		mi92.setToolTipText(
				"Verhältnismäßig geringe Kostenquoten (unter 10%) werden nach § 92 II ZPO ignoriert und den anderen Beteiligten auferlegt.");
		mi92.setMnemonic(KeyEvent.VK_9);
		menu.add(mi92);
		menu.add(new JMenuItem(skktxSpeichern));
		JMenu oeffnenMenu = new JMenu("Öffnen");
		menu.add(oeffnenMenu);
		oeffnenMenu.add(new JMenuItem(skktxLaden));
		oeffnenMenu.add(new JMenuItem(xmlLesenAction));
		menu.addSeparator();
		JMenuItem miStandardSzenario = new JMenuItem(klageruecknahmeTerminAction);
		menu.add(miStandardSzenario);
		menu.addSeparator();
		menu.add(new JMenuItem(baumbachAssistentAction));
		menu.add(new JMenuItem(gesamtschuldnerschaftExtrahierenAction));
		menu.add(new JMenuItem(gesamtschuldnerschaftDwbkExtrahierenAction));
		menu.addSeparator();
		menu.add(new JMenuItem(saeumnisGebuehrenAction));
		menu.add(new JMenuItem(anerkenntnisGebuehrenAction));
		menu.add(new JMenuItem(normaleGebuehrenAction));
		menu.addSeparator();
		menu.add(new JMenuItem(ueberAction));
		menu.add(new JMenuItem(lizenzAnzeigenAktion));
		JMenu kontaktMenu = new JMenu("Kontakt");
		menu.add(kontaktMenu);
		kontaktMenu.add(new JMenuItem(emailSchreibenXML));
		kontaktMenu.add(new JMenuItem(emailSchreibenAllgemein));
		JMenuItem miEnde = new JMenuItem(beendenAction);
		menu.add(miEnde);
		setJMenuBar(menuBar);
	}

	/**
	 * Die Methode setzt die Oberflächenelemente für die Registerkarte 1
	 * Teilklagerücknahme
	 * 
	 */
	private void fuelleStreitwertReduktionPanel() {
		JPanel pnStreitwertreduktion = new JPanel();
		tabbedPane.addTab("Teilklagerücknahme (1)", null, pnStreitwertreduktion, "<html><b>Teilklagerücknahme</b>"
				+ "<br>Ein Tenor kann nach Quotenmethode oder nach Mehrkostenmethode erzeugt werden.</html>");
		tabbedPane.setEnabledAt(0, true);
		tabbedPane.setMnemonicAt(0, KeyEvent.VK_1);

		GridBagLayout gbl_tabStreitwertreduktion = new GridBagLayout();
		pnStreitwertreduktion.setLayout(gbl_tabStreitwertreduktion);

		JLabel lblParteien = new JLabel("Parteien:", SwingConstants.RIGHT);
		lblParteien.setDisplayedMnemonic(KeyEvent.VK_P);
		pnStreitwertreduktion.add(lblParteien,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] klBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.KLAEGER, true);
		cbKlaeger = new JComboBox<Beteiligter>(klBezeichnerListe);
		cbKlaeger.setRenderer(BETEILIGTEN_RENDERER_OHNE_LFDNR);
		lblParteien.setLabelFor(cbKlaeger);
		pnStreitwertreduktion.add(cbKlaeger,
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] bkBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.BEKLAGTE, true);
		cbBeklagte = new JComboBox<Beteiligter>(bkBezeichnerListe);
		cbBeklagte.setRenderer(BETEILIGTEN_RENDERER_OHNE_LFDNR);
		pnStreitwertreduktion.add(cbBeklagte,
				GitterBeutelBeschraenkungen.getInstance(2, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblStreitwertVorReduktion = new JLabel("Streitwert vor Reduktion", SwingConstants.RIGHT);
		pnStreitwertreduktion.add(lblStreitwertVorReduktion,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(0,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfStreitwertVorher = new JFormattedTextField(FORMATTER_INT);
		ftfStreitwertVorher.setColumns(10);
		ftfStreitwertVorher.setText("0");// "15000"
		ftfStreitwertVorher.addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		ftfStreitwertVorher.setHorizontalAlignment(JTextField.RIGHT);
		lblStreitwertVorReduktion.setLabelFor(ftfStreitwertVorher);
		pnStreitwertreduktion.add(ftfStreitwertVorher,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(2,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(1, 1, 2, 1, GridBagConstraints.HORIZONTAL, false));

		setzeStandardGebuehren(false);
		tabVorher = new JTable(gttmVorher);
		entferneTastaturFalle(tabVorher);
		setzeGebuehrenTabelleAussehen(tabVorher);
		JScrollPane scpVorher = new JScrollPane(tabVorher);
		pnStreitwertreduktion.add(scpVorher,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(0,
				// 3, 3,
				// 1, true, false));
				GitterBeutelBeschraenkungen.getInstance(0, 2, 3, 1, GridBagConstraints.BOTH, false));

		JButton btnNeueGebhrvorher = new JButton(neueGebuehrVorherAction);
		pnStreitwertreduktion.add(btnNeueGebhrvorher,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 3, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblStreitwertNachReduktion = new JLabel("Streitwert nach Reduktion", SwingConstants.RIGHT);
		pnStreitwertreduktion.add(lblStreitwertNachReduktion,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(3,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(0, 4, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfStreitwertNachher = new JFormattedTextField(FORMATTER_INT);
		ftfStreitwertNachher.setColumns(10);
		ftfStreitwertNachher.setText("0");// "10000"
		ftfStreitwertNachher.addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		ftfStreitwertNachher.addFocusListener(streitwertListener);
		ftfStreitwertNachher.setHorizontalAlignment(JTextField.RIGHT);
		lblStreitwertNachReduktion.setLabelFor(ftfStreitwertNachher);
		pnStreitwertreduktion.add(ftfStreitwertNachher,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(5,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(1, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		tabNachher = new JTable(gttmNachher);
		entferneTastaturFalle(tabNachher);
		setzeGebuehrenTabelleAussehen(tabNachher);
		JScrollPane scpNachher = new JScrollPane(tabNachher);
		pnStreitwertreduktion.add(scpNachher,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(3,
				// 3, 3,
				// 1, true, false));
				GitterBeutelBeschraenkungen.getInstance(0, 5, 3, 1, GridBagConstraints.BOTH, false));

		JButton btnNeueGebhrnachher = new JButton(neueGebuehrNachherAction);
		pnStreitwertreduktion.add(btnNeueGebhrnachher,
				GitterBeutelBeschraenkungen.getInstance(0, 6, 3, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblLetztlichObsiegt = new JLabel("Letztlich obsiegt:", SwingConstants.RIGHT);
		pnStreitwertreduktion.add(lblLetztlichObsiegt,
				GitterBeutelBeschraenkungen.getInstance(0, 8, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfObsiegen = new JFormattedTextField(FORMATTER_INT);
		ftfObsiegen.setText("0");// "10000"
		ftfObsiegen.addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		ftfObsiegen.setHorizontalAlignment(JTextField.RIGHT);
		lblLetztlichObsiegt.setLabelFor(ftfObsiegen);
		pnStreitwertreduktion.add(ftfObsiegen,
				GitterBeutelBeschraenkungen.getInstance(1, 8, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnBerechnequotenmethode = new JButton(quotenMethodeAction);
		pnStreitwertreduktion.add(btnBerechnequotenmethode,
				GitterBeutelBeschraenkungen.getInstance(0, 9, 3, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnBerechnemehrkostenmethode = new JButton(mehrkostenMethodeAction);
		pnStreitwertreduktion.add(btnBerechnemehrkostenmethode,
				GitterBeutelBeschraenkungen.getInstance(0, 10, 3, 1, GridBagConstraints.HORIZONTAL, false));

		// tabVorher.setTransferHandler(new
		// GebuehrenTransferHandler(tabVorher));
		// tabNachher.setTransferHandler(new
		// GebuehrenTransferHandler(tabNachher));

		// JButton btnBerechneGebuehrenSumme = new
		// JButton(gebuehrenSummierenAction);
		// pnStreitwertreduktion.add(btnBerechneGebuehrenSumme,
		// GitterBeutelBeschraenkungen.getInstance(0, 11, 3, 1,
		// GridBagConstraints.NONE, false));

		pmGttmVorher = new JPopupMenu(VORHER_POPUPLABEL);
		pmGttmVorher.add(new JMenuItem(VORHER_POPUPLABEL));
		pmGttmVorher.addSeparator();
		pmGttmVorher.add(new JMenuItem(neueGebuehrVorherAction));
		pmGttmVorher.add(new JMenuItem(klageruecknahmeTerminAction));
		pmGttmVorher.add(new JMenuItem(eintragEntfernenAktion));
		pmGttmVorher.add(new JMenuItem(gebuehrenTbVerschiebenAction));
		tabVorher.addMouseListener(new KontextMouseListener(tabVorher, pmGttmVorher));

		pmGttmNachher = new JPopupMenu(NACHHER_POPUPLABEL);
		pmGttmNachher.add(new JMenuItem(NACHHER_POPUPLABEL));
		pmGttmNachher.addSeparator();
		pmGttmNachher.add(new JMenuItem(neueGebuehrNachherAction));
		pmGttmNachher.add(new JMenuItem(klageruecknahmeTerminAction));
		pmGttmNachher.add(new JMenuItem(eintragEntfernenAktion));
		pmGttmNachher.add(new JMenuItem(gebuehrenTbVerschiebenAction));
		tabNachher.addMouseListener(new KontextMouseListener(tabNachher, pmGttmNachher));
	}

	/**
	 * Die Methode setzt die Oberflächenelemente für die Registerkarte 2
	 * Streitgenossen
	 * 
	 */
	private void fuelleBaumbachPanel() {
		JPanel pnBaumbach = new JPanel();
		tabbedPane.addTab("Streitgenossen (2)", null, pnBaumbach,
				"<html><b>Streitgenossenschaft</b><br>Der Tenor für Teilunterliegen bei Streitgenossenschaft kann nach der Baumbach’schen Kostenformel erzeugt werden.</html>");
		tabbedPane.setEnabledAt(1, true);
		tabbedPane.setMnemonicAt(1, KeyEvent.VK_2);
		GridBagLayout gbl_tabBaumbach = new GridBagLayout();
		pnBaumbach.setLayout(gbl_tabBaumbach);

		widerKlageAnzeigen = false;

		setzeBaumbachTabellen();

		tabKlaeger = new JTable(btmKlaeger);
		setzeBeteiligtenRenderer(tabKlaeger);
		JScrollPane scpKlaeger = new JScrollPane(tabKlaeger);
		pnBaumbach.add(scpKlaeger, GitterBeutelBeschraenkungen.getInstance(0, 0, 2, 1, GridBagConstraints.BOTH, false));
		entferneTastaturFalle(tabKlaeger);

		tabBeklagte = new JTable(btmBeklagte);
		setzeBeteiligtenRenderer(tabBeklagte);
		JScrollPane scpBeklagte = new JScrollPane(tabBeklagte);
		pnBaumbach.add(scpBeklagte,
				GitterBeutelBeschraenkungen.getInstance(2, 0, 2, 3, GridBagConstraints.BOTH, false));
		entferneTastaturFalle(tabBeklagte);

		JButton btnKlgerNeu = new JButton(neuerKlaegerAction);
		pnBaumbach.add(btnKlgerNeu,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		tabDrittwiderbeklagte = new JTable(btmDrittwiderbeklagte);
		setzeBeteiligtenRenderer(tabDrittwiderbeklagte);
		scpDwbk = new JScrollPane(tabDrittwiderbeklagte);
		pnBaumbach.add(scpDwbk, GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));
		entferneTastaturFalle(tabDrittwiderbeklagte);

		btnDrittwiderbeklagterNeu = new JButton(neuerDrittwiderbeklagterAction);
		pnBaumbach.add(btnDrittwiderbeklagterNeu,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnBeklagterNeu = new JButton(neuerBeklagterAction);
		pnBaumbach.add(btnBeklagterNeu,
				GitterBeutelBeschraenkungen.getInstance(2, 3, 2, 1, GridBagConstraints.HORIZONTAL, false));

		btnKlDwblGesamtsch = new JButton(klDwbkGesamtschAction);
		pnBaumbach.add(btnKlDwblGesamtsch,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		btnWiderklageAnzeigen = new JButton(widerKlageAktivierenAction);
		pnBaumbach.add(btnWiderklageAnzeigen,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		setWiderKlageAnzeigen(false);

		JButton btnBkGesamtsch = new JButton(bkGesamtschuldnerAction);
		pnBaumbach.add(btnBkGesamtsch,
				GitterBeutelBeschraenkungen.getInstance(2, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnBerechnen = new JButton(baumbachAction);
		pnBaumbach.add(btnBerechnen,
				GitterBeutelBeschraenkungen.getInstance(0, 5, 4, 1, GridBagConstraints.HORIZONTAL, false));

		pmBeklagtenKontextmenu = new JPopupMenu(BEKLAGTE_POPUPLABEL);
		pmBeklagtenKontextmenu.add(new JMenuItem(BEKLAGTE_POPUPLABEL));
		pmBeklagtenKontextmenu.addSeparator();
		pmBeklagtenKontextmenu.add(new JMenuItem(neuerBeklagterAction));
		pmBeklagtenKontextmenu.add(new JMenuItem(bkGesamtschuldnerAction));
		pmBeklagtenKontextmenu.add(new JMenuItem(gesamtschuldnerschaftExtrahierenAction));
		pmBeklagtenKontextmenu.add(new JMenuItem(eintragEntfernenAktion));
		tabBeklagte.addMouseListener(new KontextMouseListener(tabBeklagte, pmBeklagtenKontextmenu));

		pmKlaegerKontextmenu = new JPopupMenu(KLAEGER_POPUPLABEL);
		pmKlaegerKontextmenu.add(new JMenuItem(KLAEGER_POPUPLABEL));
		pmKlaegerKontextmenu.addSeparator();
		pmKlaegerKontextmenu.add(new JMenuItem(neuerKlaegerAction));
		pmKlaegerKontextmenu.add(new JMenuItem(klDwbkGesamtschAction));
		pmKlaegerKontextmenu.add(new JMenuItem(gesamtschuldnerschaftDwbkExtrahierenAction));
		pmKlaegerKontextmenu.add(new JMenuItem(eintragEntfernenAktion));
		tabKlaeger.addMouseListener(new KontextMouseListener(tabKlaeger, pmKlaegerKontextmenu));

		pmDwbkKontextmenu = new JPopupMenu(DRITTWIDERBEKLAGTE_POPUPLABEL);
		pmDwbkKontextmenu.add(new JMenuItem(DRITTWIDERBEKLAGTE_POPUPLABEL));
		pmDwbkKontextmenu.addSeparator();
		pmDwbkKontextmenu.add(new JMenuItem(neuerDrittwiderbeklagterAction));
		pmDwbkKontextmenu.add(new JMenuItem(klDwbkGesamtschAction));
		pmDwbkKontextmenu.add(new JMenuItem(gesamtschuldnerschaftDwbkExtrahierenAction));
		pmDwbkKontextmenu.add(new JMenuItem(eintragEntfernenAktion));
		tabDrittwiderbeklagte.addMouseListener(new KontextMouseListener(tabDrittwiderbeklagte, pmDwbkKontextmenu));
		// pnBaumbach.setTransferHandler(new XmlTransferHandler(this));
	}

	/**
	 * Die Methode setzt die Oberflächenelemente für die Registerkarte 3
	 * Hilfsaufrechnung.
	 * 
	 */
	private void fuelleHilfsaufrechnungPanel() {
		pnHilfsaufrechnung = new JPanel();
		tabbedPane.addTab("Hilfsaufrechnung (3)", null, pnHilfsaufrechnung,
				"<html><b>Hilfsaufrechnung</b><br>Der Tenor für die Staffelung von Klage und Aufrechnungen kann erzeugt werden.</html>");
		tabbedPane.setMnemonicAt(2, KeyEvent.VK_3);
		pnHilfsaufrechnung.setLayout(new GridBagLayout());

		JLabel lblParteien = new JLabel("Parteien:", SwingConstants.RIGHT);
		lblParteien.setDisplayedMnemonic(KeyEvent.VK_P);
		pnHilfsaufrechnung.add(lblParteien,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] klBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.KLAEGER, true);
		cbAngriffsKlaeger = new JComboBox<Beteiligter>(klBezeichnerListe);
		cbAngriffsKlaeger.setRenderer(BETEILIGTEN_RENDERER_OHNE_LFDNR);
		lblParteien.setLabelFor(cbAngriffsKlaeger);
		pnHilfsaufrechnung.add(cbAngriffsKlaeger,
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] bkBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.BEKLAGTE, true);
		cbVerteidigungsBeklagter = new JComboBox<Beteiligter>(bkBezeichnerListe);
		cbVerteidigungsBeklagter.setRenderer(BETEILIGTEN_RENDERER_OHNE_LFDNR);
		pnHilfsaufrechnung.add(cbVerteidigungsBeklagter,
				GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblForderungen = new JLabel("Bitte Angriffs- und Verteidigungsmittel angeben:");
		lblParteien.setDisplayedMnemonic(KeyEvent.VK_V);
		pnHilfsaufrechnung.add(lblForderungen,
				GitterBeutelBeschraenkungen.getInstance(0, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));

		tbForderungen = new JTable();
		setzeForderungsTabellen();
		entferneTastaturFalle(tbForderungen);
		// tbForderungen.setFillsViewportHeight(true);
		tbForderungen.setDefaultRenderer(Forderung.class, FORDERUNGEN_RENDERER);
		tbForderungen.setDefaultRenderer(Double.class, EURO_RENDERER);
		tbForderungen.setDefaultEditor(Double.class, EURO_EDITOR);
		JComboBox<Forderung> cbForderungen = new JComboBox<Forderung>(ForderungHinzufuegen.sammleForderungen());
		cbForderungen.setRenderer(FORDERUNGEN_RENDERER);
		tbForderungen.setDefaultEditor(Forderung.class, new DefaultCellEditor(cbForderungen));
		tbForderungen.getTableHeader().setDefaultRenderer(HEADER_RENDERER);

		JScrollPane scpForderungen = new JScrollPane(tbForderungen);
		lblForderungen.setLabelFor(scpForderungen);
		pnHilfsaufrechnung.add(scpForderungen,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 2, 1, GridBagConstraints.BOTH, false));

		JButton btnNeueForderung = new JButton(neueForderungAction);
		pnHilfsaufrechnung.add(btnNeueForderung,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnAngriffsVerteidigungsmittelStaffelung = new JButton(angriffsVerteidigungsmittelStaffelungAction);
		pnHilfsaufrechnung.add(btnAngriffsVerteidigungsmittelStaffelung,
				GitterBeutelBeschraenkungen.getInstance(0, 5, 2, 1, GridBagConstraints.HORIZONTAL, false));

		pmForderungen = new JPopupMenu(FORDERUNGEN_POPUPLABEL);
		pmForderungen.add(new JMenuItem(FORDERUNGEN_POPUPLABEL));
		pmForderungen.addSeparator();
		pmForderungen.add(new JMenuItem(neueForderungAction));
		pmForderungen.add(new JMenuItem(eintragEntfernenAktion));
		tbForderungen.addMouseListener(new KontextMouseListener(tbForderungen, pmForderungen));
	}

	/**
	 * Die Methode fuellt den GebuehrenRechnerPanel.
	 * 
	 */
	private void fuelleGebuehrenRechnerPanel() {
		JPanel pnGebuehrenRechner = new JPanel();
		tabbedPane.addTab("Freie Gebührenberechnung (4)", null, pnGebuehrenRechner,
				"Berechnung von Gebühren nach einem Streitwert.");
		tabbedPane.setMnemonicAt(3, KeyEvent.VK_4);
		pnGebuehrenRechner.setLayout(new GridBagLayout());

		// JFormattedTextField für den Streitwert, das sich ständig mit der
		// JTable synchronisiert
		JLabel lblStreitwert = new JLabel("Bitte geben Sie den Streitwert ein:", SwingConstants.RIGHT);
		ftfStreitwert = new JFormattedTextField(FORMATTER);
		ftfStreitwert.setColumns(10);
		ftfStreitwert.setText("0");
		ftfStreitwert.addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		ftfStreitwert.setHorizontalAlignment(JTextField.RIGHT);
		lblStreitwert.setDisplayedMnemonic(KeyEvent.VK_S);
		lblStreitwert.setDisplayedMnemonicIndex(20);
		lblStreitwert.setLabelFor(ftfStreitwert);

		// JScrollpane mit Gebührentabelle, die sich mit dem ftf synchronisiert
		JLabel lblStreitwertTabelle = new JLabel("Oder wählen Sie einen Streitwert aus:");
		StreitwertTableModel swtm = new StreitwertTableModel();
		tabStreitwertTabelle = new JTable(swtm);
		tabStreitwertTabelle.setDefaultRenderer(Long.class, EURO_RENDERER);
		tabStreitwertTabelle.setDefaultRenderer(Double.class, EURO_RENDERER);
		tabStreitwertTabelle.getTableHeader().setDefaultRenderer(HEADER_RENDERER);
		tabStreitwertTabelle.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		entferneTastaturFalle(tabStreitwertTabelle);
		JScrollPane scpStreitwertTabelle = new JScrollPane(tabStreitwertTabelle);
		lblStreitwertTabelle.setDisplayedMnemonic(KeyEvent.VK_W);
		lblStreitwertTabelle.setLabelFor(scpStreitwertTabelle);

		ftfStreitwert.addFocusListener(aktualisierungsHorcher);
		ftfStreitwert.addKeyListener(aktualisierungsHorcher);
		tabStreitwertTabelle.addFocusListener(aktualisierungsHorcher);
		tabStreitwertTabelle.addMouseListener(aktualisierungsHorcher);
		tabStreitwertTabelle.addKeyListener(aktualisierungsHorcher);
		tabStreitwertTabelle.getSelectionModel().addListSelectionListener(aktualisierungsHorcher);

		// JScrollpane mit JTable mit gttm + JLabel
		JLabel lblGebuehrenTatbestaende = new JLabel("Bitte geben Sie die angefallenen Gebühren an:");
		tabGebuehrenTatbestaende = new JTable();
		setzeGebuehrenRechnerTabelle(0);
		entferneTastaturFalle(tabGebuehrenTatbestaende);
		setzeGebuehrenTabelleAussehen(tabGebuehrenTatbestaende);
		JScrollPane scpGebuehrenTatbestaende = new JScrollPane(tabGebuehrenTatbestaende);
		lblGebuehrenTatbestaende.setDisplayedMnemonic(KeyEvent.VK_G);
		lblGebuehrenTatbestaende.setDisplayedMnemonicIndex(33);
		lblGebuehrenTatbestaende.setLabelFor(scpGebuehrenTatbestaende);

		// JButton Gebühr hinzufügen
		JButton btnNeueVerguetung = new JButton(neueRvgVerguetungAction);
		JButton btnNeueGebuehr = new JButton(neueGkgGebuehrAction);

		// JTextField Summe berechnen
		tfGebuehrenSumme = new JTextField();
		tfGebuehrenSumme.setEditable(false);
		tfGebuehrenSumme.setHorizontalAlignment(SwingConstants.RIGHT);
		JButton btnSummeBerechnen = new JButton(gebuehrenAufstellungAction);

		errechneGebuehrenSumme();

		// Der JPanel hinzufügen
		pnGebuehrenRechner.add(lblStreitwert,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		pnGebuehrenRechner.add(ftfStreitwert,
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		pnGebuehrenRechner.add(lblStreitwertTabelle,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
		pnGebuehrenRechner.add(scpStreitwertTabelle,
				GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, true));
		pnGebuehrenRechner.add(lblGebuehrenTatbestaende,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		pnGebuehrenRechner.add(tfGebuehrenSumme,
				GitterBeutelBeschraenkungen.getInstance(1, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		pnGebuehrenRechner.add(scpGebuehrenTatbestaende,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.BOTH, true));
		pnGebuehrenRechner.add(btnNeueGebuehr,
				GitterBeutelBeschraenkungen.getInstance(0, 5, 1, 1, GridBagConstraints.HORIZONTAL, false));
		pnGebuehrenRechner.add(btnNeueVerguetung,
				GitterBeutelBeschraenkungen.getInstance(1, 5, 1, 1, GridBagConstraints.HORIZONTAL, false));
		pnGebuehrenRechner.add(btnSummeBerechnen,
				GitterBeutelBeschraenkungen.getInstance(0, 6, 2, 1, GridBagConstraints.HORIZONTAL, false));

		// Kontextmenü für Streitwerttabelle
		pmStreitwert = new JPopupMenu(STREITWERTE_POPUPLABEL);
		pmStreitwert.add(new JMenuItem(STREITWERTE_POPUPLABEL));
		pmStreitwert.addSeparator();
		pmStreitwert.add(new JMenuItem(druckeSwTabelleAction));
		pmStreitwert.add(new JMenuItem(kopiereSwTabelleAction));
		tabStreitwertTabelle.addMouseListener(new KontextMouseListener(tabStreitwertTabelle, pmStreitwert));

		// Kontextmenü für gttm
		pmGttmRechner = new JPopupMenu(RECHNER_POPUPLABEL);
		pmGttmRechner.add(new JMenuItem(RECHNER_POPUPLABEL));
		pmGttmRechner.addSeparator();
		pmGttmRechner.add(new JMenuItem(neueGkgGebuehrAction));
		pmGttmRechner.add(new JMenuItem(neueRvgVerguetungAction));
		pmGttmRechner.add(new JMenuItem(saeumnisGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(anerkenntnisGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(normaleGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(mitInstanzGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(eintragEntfernenAktion));
		tabGebuehrenTatbestaende.addMouseListener(new KontextMouseListener(tabGebuehrenTatbestaende, pmGttmRechner));
	}

	/**
	 * Die Methode addiert die Gebühren der Tabelle.
	 * 
	 */
	private void errechneGebuehrenSumme() {
		// tfGebuehrenSumme.requestFocus();
		String streitwert = "";
		double gebuehrenSumme = 0.0;
		if (ftfStreitwert != null && gttmRechner != null) {
			try {
				streitwert = ftfStreitwert.getText();
				gebuehrenSumme = gttmRechner.errechneGebuehrenSumme(Long.parseLong(streitwert));
			} catch (NumberFormatException e) {
				// e.printStackTrace();

				new FehlerDialog(FehlerDialog.FEHLER_BLANKO,
						"Falsches Eingabeformat für den Streitwert: \"" + streitwert + "\"!\n ", e).showDialog();
			}
		}
		String text = String.format(SUMME_EUR, gebuehrenSumme);
		if (tfGebuehrenSumme != null)
			tfGebuehrenSumme.setText(text);
	}

	/**
	 * Die Methode setzt die Registerkarte 4 Hilfe mit einem einzigen
	 * JScrollPane statt eines JPanel
	 * 
	 */
	private void fuelleHilfePanel() {
		txtpnPanic = setHilfeText();
		scpPanic = new JScrollPane(txtpnPanic);
		tabbedPane.addTab("Hilfe (5)", null, scpPanic,
				"<html><b>Keine Panik!</b><br>Zeigt einen kurzen Hilfetext an.</html>");
		tabbedPane.setMnemonicAt(4, KeyEvent.VK_5);
	}

	/**
	 * Die Methode setzt ein JTextPane mit Hilfetext.
	 * 
	 * @return das JTextPane mit dem Hilfetext
	 */
	private JTextPane setHilfeText() {
		JTextPane textPane = new JTextPane();
		// RTFEditorKit rtf = new RTFEditorKit();
		HTMLEditorKit html = new HTMLEditorKit();
		textPane.setEditorKit(html);
		textPane.setEditable(false);
		StyledDocument doc = (StyledDocument) html.createDefaultDocument();
		textPane.setStyledDocument(doc);
		// leseRtf(doc, rtf);
		leseHtml(doc, html);
		// Caret ganz nach oben setzen
		textPane.setCaretPosition(0);
		textPane.addHyperlinkListener(HinzufuegenDialog.HYPERLINK_LISTENER);
		return textPane;
	}

	/**
	 * Liest den Inhalt einer HTML-Datei ein und schreibt ihn formatiert in das
	 * übergebene StyledDocument
	 * 
	 * @param doc
	 *            das StyledDocument des JTextPanes, das gefüllt werden soll
	 * @param html
	 *            das HTMLEditorKit des JTextPanes, das gefüllt werden soll.
	 */
	private void leseHtml(StyledDocument doc, HTMLEditorKit html) {
		InputStream is;
		try {
			is = getClass().getClassLoader().getResourceAsStream("resources/hilfetext.html");
			doc.remove(0, doc.getLength());
			doc.putProperty("IgnoreCharsetDirective", new Boolean(true));
			html.read(is, doc, 0);
		} catch (FileNotFoundException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
			// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
			// e.printStackTrace();
		} catch (IOException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
			// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
			// e.printStackTrace();
		} catch (BadLocationException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
			// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
			// e.printStackTrace();
		}
	}

	// private void leseRtf(StyledDocument doc, RTFEditorKit rtf) {
	// InputStream is;
	// try {
	// is = getClass().getClassLoader().getResourceAsStream(
	// "resources/hilfetext.rtf");
	// doc.remove(0, doc.getLength());
	// rtf.read(is, doc, 0);
	// } catch (FileNotFoundException e) {
	// //
	// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
	// e.printStackTrace();
	// } catch (IOException e) {
	// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
	// e.printStackTrace();
	// } catch (BadLocationException e) {
	// //
	// Kostenrechner.zeigeFehler(e.getLocalizedMessage());
	// e.printStackTrace();
	// }
	// }
	// pnPanic = new JPanel();
	// tabbedPane.addTab("Don't Panic", null, pnPanic, null);
	// GridBagLayout gbl_pnPanic = new GridBagLayout();
	// pnHilfsaufrechnung.setLayout(gbl_pnPanic);
	//
	// setHilfeText();
	// scpPanic = new JScrollPane(txtpnPanic);
	// pnPanic.add(scpPanic,
	// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(0, 0, 1, 1,
	// true,
	// false));
	//
	// txtpnKostentenor = new JTextPane();
	// scpKostentenor = new JScrollPane(txtpnKostentenor);
	// txtpnKostentenor.setText("Kostentenor");
	// pnTenor.add(scpKostentenor,
	// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(4, 4, 0,
	// 0,
	// true, false));

	// txtpnKostentenor.setContentType("text/rtf");
	// rtf = new RTFEditorKit();
	// txtpnKostentenor.setEditorKit(rtf);
	// txtpnKostentenor_1.getEditorKit().
	// txtpnKostentenor.setDocument(rtf.createDefaultDocument());
	// StyledDocument doc = new
	// DefaultStyledDocument(rtf.createDefaultDocument());
	// txtpnKostentenor.setStyledDocument((StyledDocument) rtf
	// .createDefaultDocument());
	// setKostentenorText("Kostentenor");

	/**
	 * Die Methode setWiderklageAnzeigen dient zum Aktivieren der Widerklagen.
	 * Bei allen TableModels widerKlageAnzeigen setzen, Dwbk-Tabelle und
	 * Schaltflächen wieder auf Visible setzen, eigene Schaltflächen ausblenden.
	 * 
	 * @param widerKlageAnzeigen
	 *            der Wert für widerKlageAnzeigen, der gesetzt werden soll.
	 */
	void setWiderKlageAnzeigen(boolean widerKlageAnzeigen) {
		this.widerKlageAnzeigen = widerKlageAnzeigen;
		scpDwbk.setVisible(widerKlageAnzeigen);
		btnDrittwiderbeklagterNeu.setVisible(widerKlageAnzeigen);
		btnKlDwblGesamtsch.setVisible(widerKlageAnzeigen);
		btnWiderklageAnzeigen.setVisible(!widerKlageAnzeigen);
	}

	/**
	 * Die Methode leert die Tabellen der BaumbachBeteiligten
	 * 
	 */
	void setzeBaumbachTabellen() {
		btmKlaeger = new BeteiligtenTableModel(Beteiligter.KLAEGER, widerKlageAnzeigen, new BaumbachBeteiligtenListe());
		btmDrittwiderbeklagte = new BeteiligtenTableModel(Beteiligter.DRITTWIDERBEKLAGTE, widerKlageAnzeigen);
		btmBeklagte = new BeteiligtenTableModel(Beteiligter.BEKLAGTE, widerKlageAnzeigen,
				new BaumbachBeteiligtenListe());
		if (tabKlaeger != null)
			tabKlaeger.setModel(btmKlaeger);
		if (tabBeklagte != null)
			tabBeklagte.setModel(btmBeklagte);
		if (tabDrittwiderbeklagte != null)
			tabDrittwiderbeklagte.setModel(btmDrittwiderbeklagte);
	}

	/*
	 * Die Methode leert die Baumbach-Tabellen und füllt sie mit den Daten aus
	 * dem {@link KostenTenorContainer}
	 * 
	 * @param vdc der KostenTenorContainer
	 */
	// private void setzeBaumbachTabellen(VerfahrensDatenContainer vdc) {
	// if (vdc == null)
	// return;
	// if (vdc.gibtsKlaeger()
	// && vdc.getKlaegerListe().enthaeltAnWiderklageBeteiligte())
	// setWiderKlageAnzeigen(true);
	// if (vdc.gibtsDrittwiderbeklagte())
	// setWiderKlageAnzeigen(true);
	// if (vdc.gibtsBeklagte()
	// && vdc.getBeklagtenListe().enthaeltAnWiderklageBeteiligte())
	// setWiderKlageAnzeigen(true);
	// btmKlaeger = new BeteiligtenTableModel(Beteiligter.KLAEGER,
	// widerKlageAnzeigen, vdc.getKlaegerListe());
	// tabKlaeger.setModel(btmKlaeger);
	// if (vdc.gibtsDrittwiderbeklagte())
	// btmDrittwiderbeklagte = new BeteiligtenTableModel(
	// Beteiligter.DRITTWIDERBEKLAGTE, widerKlageAnzeigen,
	// vdc.getDrittwiderbeklagtenListe());
	// else
	// btmDrittwiderbeklagte = new BeteiligtenTableModel(
	// Beteiligter.DRITTWIDERBEKLAGTE, widerKlageAnzeigen);
	// tabDrittwiderbeklagte.setModel(btmDrittwiderbeklagte);
	// btmBeklagte = new BeteiligtenTableModel(Beteiligter.BEKLAGTE,
	// widerKlageAnzeigen, vdc.getBeklagtenListe());
	// tabBeklagte.setModel(btmBeklagte);
	// tabbedPane.setSelectedIndex(1);
	// }

	/**
	 * Die Methode leert die Baumbach-Tabellen und füllt sie mit den Daten aus
	 * dem Untercontainer
	 * {@link eu.gronos.kostenrechner.VerfahrensDatenContainer.Streitgenossen}
	 * 
	 * @param streitgenossen
	 *            ein Untercontainer
	 *            {@link eu.gronos.kostenrechner.VerfahrensDatenContainer.Streitgenossen}
	 */
	private void setzeBaumbachTabellen(final VerfahrensDatenContainer.Streitgenossen streitgenossen) {
		setWiderKlageAnzeigen(streitgenossen.widerklage);
		wirtschaftlichIdentisch = streitgenossen.wirtschaftlichIdentisch;
		btmKlaeger = new BeteiligtenTableModel(Beteiligter.KLAEGER, widerKlageAnzeigen, streitgenossen.klaeger);
		tabKlaeger.setModel(btmKlaeger);
		btmDrittwiderbeklagte = new BeteiligtenTableModel(Beteiligter.DRITTWIDERBEKLAGTE, widerKlageAnzeigen,
				streitgenossen.drittwiderbeklagte);
		tabDrittwiderbeklagte.setModel(btmDrittwiderbeklagte);
		btmBeklagte = new BeteiligtenTableModel(Beteiligter.BEKLAGTE, widerKlageAnzeigen, streitgenossen.beklagte);
		tabBeklagte.setModel(btmBeklagte);

	}

	/**
	 * Die Methode setzt die DefaultRenderer für Beteiligten-JTables
	 * (BeteiligtenRenderer und EuroRenderer) und bestimmt, dass sie den
	 * Viewport ausfüllen müssen.
	 * 
	 * @param table
	 *            JTable für Beteiligtendaten
	 * 
	 */
	private void setzeBeteiligtenRenderer(JTable table) {
		// table.setFillsViewportHeight(true);
		table.setDefaultRenderer(Beteiligter.class, BETEILIGTEN_RENDERER);
		JComboBox<Beteiligter> comboBox = new JComboBox<Beteiligter>(Beteiligter
				.getBeteiligtenAuswahlListe(((BeteiligtenTableModel) table.getModel()).getBeteiligtenTyp(), false));
		comboBox.setRenderer(BETEILIGTEN_RENDERER_OHNE_LFDNR);
		table.setDefaultEditor(Beteiligter.class, new DefaultCellEditor(comboBox));
		table.setDefaultRenderer(Double.class, EURO_RENDERER);
		table.setDefaultEditor(Double.class, EURO_EDITOR);
		table.setDefaultRenderer(Boolean.class, WIDERKLAGE_RENDERER_EDITOR);
		table.setDefaultEditor(Boolean.class, WIDERKLAGE_RENDERER_EDITOR);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getTableHeader().setDefaultRenderer(HEADER_RENDERER);
	}

	/**
	 * Die Methode prüft, ob die Tabellen für die BaumbachBeteiligten auf
	 * Kläger- und auf Beklagtenseite Werte enthalten.
	 * 
	 * @return true, wenn btmKlaeger und btmBeklagte Beteiligte enthalten;
	 *         false, wenn auch nur eine der beiden leer ist.
	 */
	private boolean pruefeBaumbachTabellen() {
		return btmKlaeger.getRowCount() > 0 && btmBeklagte.getRowCount() > 0;
	}

	/*
	 * Die Methode dient dazu, alle JFormattedTextFields zu Streitwertangaben
	 * auf den im {@link KostenTenorContainer} <code>vdc</code> angegebenen
	 * Streitwert zu setzen.
	 * 
	 * @param vdc der vom {@link BaumbachAssistentDialog} oder vom {@link
	 * XjustizContentHandler} übergebene {@link KostenTenorContainer}, aus dem
	 * der Streitwert gezogen wird.
	 */
	// protected void setzeAlleStreitwertTextfelder(VerfahrensDatenContainer
	// vdc) {
	// if (vdc == null)
	// return;
	// final double streitwert = vdc.getStreitwert();
	// if (streitwert <= 0.0)
	// return;
	// /*
	// * Muss long werden, sonst haben die hinterher die Parser für den
	// * Streitwert Probleme
	// */
	// String toString = Long.toString((long) streitwert);
	// ftfStreitwert.setText(toString);
	// aktualisierungsHorcher.focusLost(new FocusEvent(ftfStreitwert,
	// FocusEvent.FOCUS_LOST));
	// ftfStreitwertVorher.setText(toString);
	// ftfStreitwertNachher.setText(toString);
	// ftfObsiegen.setText(toString);
	// }

	/**
	 * Die Methode leert alle Tabellen der Streitwertreduktion oder füllt sie
	 * mit den Tatbeständen zum Standardfall
	 * 
	 * @param mitInhalt
	 *            true, wenn die Tabellen mit den Tatbeständen zum Standardfall
	 *            gefüllt werden sollen.
	 * 
	 */
	void setzeStandardGebuehren(boolean mitInhalt) {
		if (mitInhalt) {
			gttmVorher = new GebuehrenTatbestandTableModel(new GebuehrenSatzTatbestand[] {
					GerichtsGebuehrenTabelle.KV1210, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3100 });

			gttmNachher = new GebuehrenTatbestandTableModel(
					new GebuehrenSatzTatbestand[] { AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV3104 });
		} else {
			gttmVorher = new GebuehrenTatbestandTableModel(new GebuehrenSatzTatbestand[0]);
			gttmNachher = new GebuehrenTatbestandTableModel(new GebuehrenSatzTatbestand[0]);
		}
		if (tabVorher != null)
			tabVorher.setModel(gttmVorher);
		if (tabNachher != null)
			tabNachher.setModel(gttmNachher);
	}

	/**
	 * Die Methode leert alle Tabellen der Streitwertreduktion oder füllt sie
	 * mit den Tatbeständen zum aus dem
	 * {@link VerfahrensDatenContainer.Teilklageruecknahme}-Behälter. Zudem
	 * füllt sie die {@link JFormattedTextField} mit den Streitwertangaben.
	 * 
	 * @param teilklageruecknahme
	 *            ein {@link VerfahrensDatenContainer.Teilklageruecknahme}
	 *            -Behälter
	 */
	private void setzeTeilklageruecknahme(final Teilklageruecknahme teilklageruecknahme) {
		cbKlaeger.setSelectedItem(teilklageruecknahme.parteien.get(0));
		cbBeklagte.setSelectedItem(teilklageruecknahme.parteien.get(1));
		ftfStreitwertVorher.setText(teilklageruecknahme.streitwerteUndObsiegen.get(0).toString());
		ftfStreitwertNachher.setText(teilklageruecknahme.streitwerteUndObsiegen.get(1).toString());
		ftfObsiegen.setText(teilklageruecknahme.streitwerteUndObsiegen.get(2).toString());
		gttmVorher = new GebuehrenTatbestandTableModel(teilklageruecknahme.vorher);
		tabVorher.setModel(gttmVorher);
		gttmNachher = new GebuehrenTatbestandTableModel(teilklageruecknahme.nachher);
		tabNachher.setModel(gttmNachher);
	}

	/**
	 * Die Methode leert die GebuehrenRechnerTabelle oder füllt sie mit
	 * üblicherweise anfallenden Gebühren, lässt aber die Listener in Ruhe.
	 * 
	 * @param key
	 *            0 = Tabelle leeren, {@link #GEBUEHREN_ANERKENNTNIS} = Tabelle
	 *            mit den bei einem Anerkenntnis üblicherweise anfallenden
	 *            Gebühren füllen, {@link #GEBUEHREN_SAEUMNIS} Tabelle mit den
	 *            bei einer Säumnis üblicherweise anfallenden Gebühren füllen
	 * 
	 */
	void setzeGebuehrenRechnerTabelle(int key) {
		GebuehrenTatbestand[] values;
		switch (key) {
		case GEBUEHREN_STANDARD:
			values = new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210, AnwaltsGebuehrenTabelle.VV3100,
					AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R };
			break;
		case GEBUEHREN_ANERKENNTNIS:
			values = new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1211, AnwaltsGebuehrenTabelle.VV3100,
					AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R };
			break;
		case GEBUEHREN_SAEUMNIS:
			values = new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210, AnwaltsGebuehrenTabelle.VV3100,
					AnwaltsGebuehrenTabelle.VV3105, AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R };
			break;
		case GEBUEHREN_2INSTANZ:
			values = new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210, GerichtsGebuehrenTabelle.KV1220,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV3200, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV3200, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R };
			break;
		default:
			values = new GebuehrenTatbestand[0];
			break;
		}
		if (gttmRechner == null) {
			gttmRechner = new GebuehrenTatbestandTableModel(values);
			gttmRechner.addTableModelListener(aktualisierungsHorcher);
			tabGebuehrenTatbestaende.setModel(gttmRechner);
		} else {
			gttmRechner.clear();
			gttmRechner.addAll(values);
		}
	}

	/**
	 * Die Methode leert die GebuehrenRechnerTabelle, füllt sie mit den Gebühren
	 * aus
	 * {@link eu.gronos.kostenrechner.VerfahrensDatenContainer.GebuehrenBerechnung}
	 * und setzt auch den Streitwert entsprechend.
	 * 
	 * @param gebuehrenberechnung
	 *            ein
	 *            {@link eu.gronos.kostenrechner.VerfahrensDatenContainer.GebuehrenBerechnung}
	 */
	private void setzeGebuehrenRechnerTabelle(final VerfahrensDatenContainer.GebuehrenBerechnung gebuehrenberechnung) {
		/* Erst den Streitwert eintragen und die Tabelle springen lassen */
		ftfStreitwert.setText(Long.toString((long) gebuehrenberechnung.streitwert));
		aktualisierungsHorcher.focusLost(new FocusEvent(ftfStreitwert, FocusEvent.FOCUS_LOST));
		/* Dann die Tatbestände in die Gebührentabelle eintragen */
		if (gttmRechner == null) {
			gttmRechner = new GebuehrenTatbestandTableModel(gebuehrenberechnung.gebuehren);
			gttmRechner.addTableModelListener(aktualisierungsHorcher);
			tabGebuehrenTatbestaende.setModel(gttmRechner);
		} else {
			gttmRechner.clear();
			gttmRechner.addAll(gebuehrenberechnung.gebuehren);
		}
	}

	/**
	 * Die Methode leert die Tabelle mit den Forderungen.
	 * 
	 */
	private void setzeForderungsTabellen() {
		if (tbForderungen != null) {
			ftm = new ForderungenTableModel();
			tbForderungen.setModel(ftm);
		}
	}

	/**
	 * Die Methode leert die Tabelle mit den Forderungen und ersetzt mit denen
	 * aus dem {@link ForderungsStaffelung}-Behälter. Zudem werden
	 * {@link #cbAngriffsKlaeger} und {@link #cbVerteidigungsBeklagter}
	 * angepasst.
	 * 
	 * @param staffelung
	 *            {@link VerfahrensDatenContainer.ForderungsStaffelung}
	 */
	private void setzeForderungsTabellen(final ForderungsStaffelung staffelung) {
		cbAngriffsKlaeger.setSelectedItem(staffelung.parteien.get(0));
		cbVerteidigungsBeklagter.setSelectedItem(staffelung.parteien.get(1));
		ftm = new ForderungenTableModel();
		ftm.setAllValues(staffelung.forderungen);
		tbForderungen.setModel(ftm);
	}

	/**
	 * Die Methode beteiligtenHinzufuegen dient dazu, den JDialog
	 * BeteiligtenHinzufuegen zum Hinzufügen eines neuen BaumbaachBeteiligten
	 * auszulesen. Dazu liest sie BaumbachTableModel aus, sofern dieses Werte
	 * enthält, kopiert den letzten Eintrag und fügt den Rückgabewert wieder als
	 * neuen Eintrag der Tabelle hinzu.
	 * 
	 * @param btm
	 *            das {@link BeteiligtenTableModel}, in das der Beteiligte
	 *            eingefügt werden soll.
	 */
	private void beteiligtenHinzufuegen(BeteiligtenTableModel btm) {
		BaumbachBeteiligter bt;
		if (btm.getRowCount() > 0)
			bt = btm.getRow(btm.getRowCount() - 1);
		else
			bt = new BaumbachBeteiligter(btm.getBeteiligtenTyp(), Beteiligter.WEIBLICH, 0.0, 0.0, false);
		BeteiligtenHinzufuegenDialog btd = new BeteiligtenHinzufuegenDialog(Kostenrechner.this, bt, widerKlageAnzeigen);
		bt = btd.showDialog();
		// return bt;
		/*
		 * Wenn versucht wird, mehr als einen Kläger zum btm hinzuzufügen, wirft
		 * dieses eine IllegalArgumentException
		 */
		if (bt != null)
			try {
				btm.addRow(bt);
			} catch (IllegalArgumentException e) {
				new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
			}
	}

	/**
	 * Die Methode parst die JFormattedTextFields für die Streitwertreduktion.
	 * 
	 * @return ein long[] mit den Werten in der Reihenfolge Streitwert vorher,
	 *         Streitwert nachher, letztlich obsiegt.
	 */
	private long[] parseVorherNachherLetztendlich() {
		long[] zwischen = new long[3];
		long vorher;
		long nachher;
		long letztendlich;
		// TODO geht das vielleicht auch mit dem NumberFormatter?
		try {
			vorher = Long.parseLong(ftfStreitwertVorher.getText().replace(".", ""));
			nachher = Long.parseLong(ftfStreitwertNachher.getText().replace(".", ""));
			letztendlich = Long.parseLong(ftfObsiegen.getText().replace(".", ""));
		} catch (NumberFormatException nfe) {
			new FehlerDialog("Bitte Ganzzahlen eingeben!",
					"Konnte Eingabe nicht als Ganzzahl lesen: " + nfe.getLocalizedMessage(), nfe).showDialog();
			return new long[] { 0L, 0L, 0L };
		}
		zwischen = new long[] { vorher, nachher, letztendlich };
		return zwischen;
	}

	/**
	 * Die Methode prüft, ob wenigstens eine der beiden
	 * GebuehrenTatbestandTableModels auf dem Tab für die
	 * Teilerledigung/-rücknahme Daten enhtält.
	 * 
	 * @return true, wenn eine der beiden GebuehrenTatbestandTableModels Daten
	 *         enthält; false, wenn beide leer sind.
	 */
	private boolean pruefeGebuehrenTabellen() {
		return gttmVorher.getRowCount() > 0 || gttmNachher.getRowCount() > 0;
	}

	/**
	 * Die Methode setzt das Look&Feel von Windows über den UIManager
	 * 
	 * @see javax.swing.UIManager
	 * @see javax.swing.UIManager#setLookAndFeel(String)
	 */
	private static void setWindowsLookAndFeel() {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (ClassNotFoundException | InstantiationException | IllegalAccessException
				| UnsupportedLookAndFeelException e1) {
			zeigeFehler(FehlerDialog.FEHLER_BLANKO, e1.getLocalizedMessage(), e1);
		}
	}

	/**
	 * Die Methode setzt das Look And Feel von Mac OS X und sorgt dafür, dass
	 * die native MacOSX-Menüzeile benutzt werden kann
	 * 
	 * @url "http://stackoverflow.com/questions/307024/native-swing-menu-bar-support-for-macos-x-in-java"
	 *      Native Swing Menu Bar Support For MacOS X In Java
	 * @url "https://developer.apple.com/library/mac/documentation/Java/Conceptual/Java14Development/07-NativePlatformIntegration/NativePlatformIntegration.html#//apple_ref/doc/uid/TP40001909-SW1"
	 *      OS X Integration for Java
	 * 
	 * @url "http://stackoverflow.com/questions/6006173/how-do-you-change-the-dock-icon-of-a-java-program"
	 *      Dock-Icon setzen
	 * @url "http://stackoverflow.com/questions/8361947/how-to-get-getclass-getresource-from-a-static-context"
	 *      getResource aus einer static Methode heraus
	 */
	private static void setMacOsLookAndFeel() {
		try {
			/*
			 * url "https://bugs.openjdk.java.net/browse/JDK-8029440"
			 * JDK-Fehler: Xdock:name setzt den Dock-Namen nicht in Mac OS X
			 * Mavericks
			 */
			System.setProperty("com.apple.mrj.application.apple.menu.about.name", Kostenrechner.KOSTENRECHNER_TITLE);
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			System.setProperty("apple.laf.useScreenMenuBar", "true");
			System.setProperty("apple.awt.brushMetalLook", "true");
			MacToolkiste.dockIconSetzen();
			MacToolkiste.aboutMenuSetzen();
			MacToolkiste.quitSetzen();
			// Statt CTRL_MASK muss META_MASK genommen werden
			ctrlMask = ActionEvent.META_MASK;

			// MRJApplicationUtils.registerAboutHandler(handleAbout);
		} catch (ClassNotFoundException | InstantiationException | IllegalAccessException
				| UnsupportedLookAndFeelException e1) {
			zeigeFehler(FehlerDialog.FEHLER_BLANKO, e1.getLocalizedMessage(), e1);
			// e1.printStackTrace();
		}
	}

	/**
	 * Die Methode zeigt über JOptionPane einen Fehlertext an.
	 * 
	 * @param title
	 *            der Titel des Dialogs, standardmäßig:
	 *            {@link FehlerDialog#FEHLER_BLANKO}
	 * @param meldung
	 *            den anzuzeigenden Fehlertext
	 * @deprecated Use {@link #zeigeFehler(String,String,Throwable)} instead
	 */
	static void zeigeFehler(String title, String meldung) {
		if (selbst == null)
			System.err.println(meldung);
		else
			JOptionPane.showMessageDialog(selbst, meldung, title, JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Die Methode zeigt über {@link FehlerDialog} einen Fehlertext an.
	 * 
	 * @param title
	 *            der Titel des Dialogs, standardmäßig:
	 *            {@link FehlerDialog#FEHLER_BLANKO}
	 * @param meldung
	 *            den anzuzeigenden Fehlertext
	 * @param throwable
	 *            das {@link Throwable} (Oberklasse von {@link Exception}), das
	 *            geworfen wurde.
	 */
	static void zeigeFehler(String title, String meldung, Throwable throwable) {
		if (selbst == null) {
			System.err.println(meldung);
			throwable.printStackTrace();
		} else
			new FehlerDialog(title, meldung, throwable).showDialog();
	}

	/**
	 * Die Methode dient dazu, bei den JTables für die GebührenTatbestände die
	 * Spaltenbreite einzustellen, den Renderer und den Transferhandler zu
	 * setzen.
	 * 
	 * @param table
	 *            eine JTable für GebührenTatbestände
	 * 
	 */
	private void setzeGebuehrenTabelleAussehen(JTable table) {
		table.getColumnModel().getColumn(0).setMinWidth(180);
		table.getColumnModel().getColumn(0).setPreferredWidth(180);
		table.getColumnModel().getColumn(1).setPreferredWidth(40);
		table.getColumnModel().getColumn(2).setPreferredWidth(60);
		table.setDefaultRenderer(Double.class, new GebuehrenRenderer());
		table.getTableHeader().setDefaultRenderer(HEADER_RENDERER);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setTransferHandler(new GebuehrenTransferHandler(table));
	}

	/**
	 * Verhindert, dass die übergegebene <code>JTable table</code> die
	 * Tabulatortaste und Umschalt+Tabulatortaste "frisst".
	 * 
	 * @param table
	 *            die JTable
	 * @url "http://www.coderanch.com/t/342895/GUI/java/JTable-focus" aus: Big
	 *      Moose Saloon, A friendly place for programming greenhorns!
	 */
	private void entferneTastaturFalle(JTable table) {
		table.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), "tabNext");
		table.getActionMap().put("tabNext", aaTabNext);
		table.getInputMap(JComponent.WHEN_FOCUSED)
				.put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_DOWN_MASK), "tabBefore");
		table.getActionMap().put("tabBefore", aaTabBefore);
	}

	/**
	 * Die Methode stellt eine einfache "Info über"-Meldung dar.
	 * 
	 */
	static void ueberKostenrechner() {
		JOptionPane.showMessageDialog(Kostenrechner.selbst,
				Kostenrechner.KOSTENRECHNER_TITLE + Kostenrechner.KOSTENRECHNER_VERSION
						+ "\n(C) 2014-2015 by Dr. Peter Felix Schuster",
				"Über Kostenrechner", JOptionPane.INFORMATION_MESSAGE,
				new ImageIcon(Kostenrechner.class.getClassLoader().getResource("resources/rechnerhammer-64.png")));
	}

	/**
	 * Die Methode fragt vor dem Beenden ab
	 * 
	 */
	static void beendeKostenrechner() {
		if (selbst == null)
			System.exit(0);
		else {
			int antwort;
			antwort = JOptionPane
					.showConfirmDialog(selbst, "Sind Sie sicher?\nDies beendet " + Kostenrechner.KOSTENRECHNER_TITLE,
							Kostenrechner.KOSTENRECHNER_TITLE
									+ " beenden?",
							JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
							new ImageIcon(Kostenrechner.class.getClassLoader()
									.getResource("resources/rechnerhammer-64.png")));
			if (antwort == JOptionPane.OK_OPTION)
				System.exit(0);
		}
	}

	/**
	 * Definiert die Programm-Icons
	 * 
	 */
	void definiereIcons() {
		List<Image> alleIcons = new ArrayList<Image>();
		;
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-16.png")).getImage());
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-20.png")).getImage());
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-32.png")).getImage());
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-40.png")).getImage());
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-64.png")).getImage());
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-80.png")).getImage());
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-96.png")).getImage());
		alleIcons.add(
				new ImageIcon(getClass().getClassLoader().getResource("resources/rechnerhammer-128.png")).getImage());
		setIconImages(alleIcons);
	}

	/**
	 * Entfernt eine Zeile aus einer Beteiligten-Tabelle. Es wird zuvor geprüft,
	 * ob die entsprechende Tabelle bereits Gesamtschuldnerschaften enthält;
	 * dann ist ein Entfernen aus Gründen möglicher Verweise nicht möglicht. Es
	 * sei denn: die Gesamtschuldnerschaft selbst wird entfernt.
	 * 
	 * @param table
	 *            die JTable, aus der die Zeile gelöscht werden soll
	 * @param model
	 *            das BeteiligtenTableModel, aus der die Zeile gelöscht werden
	 *            soll.
	 * 
	 */
	private void entferneBeteiligtenEintrag(JTable table, BeteiligtenTableModel model) {
		if (model == null)
			return;
		if (model.getAllValues() != null && model.getAllValues().enthaeltGesamtschuldner()) {
			if (table.getSelectedRow() > -1 && !model.getRow(table.getSelectedRow()).isGesamtschuldnerschaft()) {
				zeigeFehler(FehlerDialog.FEHLER_BLANKO, FEHLER_GESAMTSCHULD_NIX_LOESCHEN,
						new IllegalArgumentException(FEHLER_GESAMTSCHULD_NIX_LOESCHEN));
				return;
			}
		} else if (model.getBeteiligtenTyp() == Beteiligter.KLAEGER && (btmDrittwiderbeklagte.getAllValues() != null
				&& btmDrittwiderbeklagte.getAllValues().enthaeltGesamtschuldner())) {
			if (table.getSelectedRow() > -1 && !model.getRow(table.getSelectedRow()).isGesamtschuldnerschaft()) {
				zeigeFehler(FehlerDialog.FEHLER_BLANKO, FEHLER_GESAMTSCHULD_NIX_LOESCHEN,
						new IllegalArgumentException(FEHLER_GESAMTSCHULD_NIX_LOESCHEN));
				return;
			}
		}
		// int selectedRow = table.getSelectedRow();
		// model.removeRow(selectedRow);
		entferneTabellenEintrag(table, model);
	}

	/**
	 * Entfernt eine Zeile aus einer Gebühren- oder Forderungs-Tabelle.
	 * 
	 * @param table
	 *            die JTable, aus der die Zeile gelöscht werden soll
	 * @param model
	 *            das RowHandler-implementierende TableModel, aus der die Zeile
	 *            gelöscht werden soll
	 */
	private void entferneTabellenEintrag(JTable table, RowHandler<?> model) {
		if (table == null || model == null || table.getSelectedRow() < 0)
			return;
		int selectedRow = table.getSelectedRow();
		model.removeRow(selectedRow);
	}

	/**
	 * Die Methode verschiebt einen Gebührentatbestand aus einer JTable in die
	 * andere.
	 * 
	 * @param ausgang
	 *            die JTable, aus der die Zeile gelöscht werden soll
	 * @param ziel
	 *            die JTable, in die die Zeile verschoben werden soll
	 */
	private void verschiebeGebuehrenTatbestand(JTable ausgang, JTable ziel) {
		if (ausgang == null || ziel == null || ausgang.getSelectedRow() < 0
				|| !(ausgang.getModel() instanceof GebuehrenTatbestandTableModel)
				|| !(ziel.getModel() instanceof GebuehrenTatbestandTableModel))
			return;
		GebuehrenTatbestandTableModel ausgangsModel = (GebuehrenTatbestandTableModel) ausgang.getModel();
		GebuehrenTatbestandTableModel zielModel = (GebuehrenTatbestandTableModel) ziel.getModel();
		int selectedRow = ausgang.getSelectedRow();
		zielModel.addRow(ausgangsModel.getRow(selectedRow));
		ausgangsModel.removeRow(selectedRow);
	}

	/**
	 * Ruft für eine gegebene Datei den Xjustiz-Import von
	 * {@link XjustizContentHandler} auf und im Anschluss den
	 * {@link BaumbachAssistenzDialog}, um das Ergebnis zu verarbeiten.
	 * 
	 * @param datei
	 *            ein File
	 * @return einen {@link VerfahrensDatenContainer}
	 */
	private VerfahrensDatenContainer xJustizLesen(File datei) {
		VerfahrensDatenContainer container = null;
		BaumbachAssistentDialog bad;
		try {
			XjustizContentHandler xj = new XjustizContentHandler(XjustizContentHandler.vonFileNachURI(datei));
			container = xj.parseXjustiz();
		} catch (ParserConfigurationException | SAXException | IOException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		}
		if (container != null) {
			/*
			 * TODO Die übrigen Streitwertfelder erst einmal mit dem Wert aus
			 * der XJustiz-Datei bestücken. So stehen sie schon einmal drin,
			 * auch wenn der Assistent später abgebrochen wird.
			 */
			// TODO setzeAlleStreitwertTextfelder(vdc);
			bad = new BaumbachAssistentDialog(Kostenrechner.this, container);
		} else
			bad = new BaumbachAssistentDialog(Kostenrechner.this, widerKlageAnzeigen);
		container = bad.showDialog();
		/*
		 * Wenn der Assistent mit OK beendet wird, dann die übrigen
		 * Streitwertfelder mit dem Streitwert füllen
		 */
		return container;
	}

	/**
	 * Die Methode prüft, ob eine Xjustiz-XML-Datei als Parameter zum
	 * Programmaufruf angegeben wurde oder eine Berechnungs-SKKTX-Datei
	 * 
	 * @param args
	 *            der String[] mit den Aufrufparametern aus
	 *            {@link Kostenrechner#main(String[]) main}
	 * @return eine Datei
	 */
	private static File parseDateiname(String[] args) {
		for (int i = 0; i < args.length; i++) {
			File datei = null;
			datei = new File(args[i]);
			/* Prüfen nach Endung XML oder SKKTX */
			if (datei.getName().toLowerCase().contains(XmlTransferHandler.XML_SUFFIX)
					|| datei.getName().toLowerCase().contains(BerechnungLadenSpeichernDialog.DATEIENDUNG_GANZ)) {
				if (datei.exists())
					return datei;
			} else {
				datei = new File(args[i] + BerechnungLadenSpeichernDialog.DATEIENDUNG_GANZ);
				if (datei.exists())
					return datei;
				datei = new File(args[i] + XmlTransferHandler.XML_SUFFIX);
				if (datei.exists())
					return datei;
			}
		}
		return null;
	}

	/**
	 * Die Methode prüft, ob das das Programm mit Schalter -debug aufgerufen
	 * wurde.
	 * 
	 * @param args
	 *            der String[] mit den Aufrufparametern aus
	 *            {@link Kostenrechner#main(String[]) main}
	 * @return true, wenn debug
	 */
	private static boolean parseDebug(final String[] args) {
		if (args != null && args.length > 0)
			for (String s : args)
				if ("-debug".equalsIgnoreCase(s))
					return true;
		return false;
	}

	/**
	 * Die Methode dient zum Xjustiz-Import bzw Laden von Xml-Berechnungen aus
	 * {@link Kostenrechner#main(String[]) main} und aus anderen Klassen.
	 * 
	 * @param xml
	 *            ein {@link File}
	 */
	void nimmXmlDatei(File xml) {
		VerfahrensDatenContainer container = null;
		if (xml != null) {
			System.out.println("nimmXmlDatei");
			if (XmlTransferHandler.isXml(xml))
				container = selbst.xJustizLesen(xml);
			else if (XmlTransferHandler.isBerechnung(xml))
				container = new BerechnungLadenSpeichernDialog().ladeXml(xml);
			if (container != null)
				leseContainer(container);
		}
	}

	/**
	 * Die Methode dient dazu, alle Programmzustände dem
	 * {@link VerfahrensDatenContainer} anzupassen.
	 * 
	 * @param container
	 *            ein {@link VerfahrensDatenContainer}
	 */
	private void leseContainer(VerfahrensDatenContainer container) {
		final GregorianCalendar gregorianCalendar = container.allgemein.speicherdatum.toGregorianCalendar();
		System.out.println(container.dateikopf.toString() + "\nBerechnung vom "
				+ gregorianCalendar.get(Calendar.DAY_OF_MONTH) + "." + (gregorianCalendar.get(Calendar.MONTH) + 1) + "."
				+ gregorianCalendar.get(Calendar.YEAR) + ", zu Az. " + container.allgemein.aktenzeichen);

		/* Allgemeine Einstellungen */
		mi92.setSelected(container.allgemein.b92ii);

		/* Erste Registerkarte Teilklagerücknahme */
		setzeTeilklageruecknahme(container.teilklageruecknahme);

		/* Zweite Registerkarte Streitgenossen */
		setzeBaumbachTabellen(container.streitgenossen);

		/* Dritte Registerkarte Forderungsstaffelung */
		setzeForderungsTabellen(container.staffelung);

		/* Vierte Registerkarte Freie Gebührenberechnung */
		setzeGebuehrenRechnerTabelle(container.gebuehrenBerechnung);

	}

	/**
	 * Die Methode dient dazu, alle Programmzustände in einen
	 * {@link VerfahrensDatenContainer} zu packen
	 * 
	 * @return einen {@link VerfahrensDatenContainer}
	 */
	private VerfahrensDatenContainer erstelleContainer() {
		VerfahrensDatenContainer container = new VerfahrensDatenContainer();

		/*
		 * Allgemeine Programmeinstellungen, die nicht vom Container selbst
		 * gesetzt werden
		 */
		container.allgemein.b92ii = mi92.isSelected();

		/* Erste Registerkarte Teilklagerücknahme */
		container.teilklageruecknahme.parteien.add((Beteiligter) cbKlaeger.getSelectedItem());
		container.teilklageruecknahme.parteien.add((Beteiligter) cbBeklagte.getSelectedItem());
		long[] vorherNachherLetztendlich = parseVorherNachherLetztendlich();
		container.teilklageruecknahme.streitwerteUndObsiegen.add(vorherNachherLetztendlich[0]);
		container.teilklageruecknahme.streitwerteUndObsiegen.add(vorherNachherLetztendlich[1]);
		container.teilklageruecknahme.streitwerteUndObsiegen.add(vorherNachherLetztendlich[2]);
		container.teilklageruecknahme.vorher = new GebuehrenTatbestand[] {};
		container.teilklageruecknahme.vorher = gttmVorher.toArray();
		container.teilklageruecknahme.nachher = new GebuehrenTatbestand[] {};
		container.teilklageruecknahme.nachher = gttmNachher.toArray();

		/* Zweite Registerkarte Streitgenossen */
		container.streitgenossen.widerklage = widerKlageAnzeigen;
		container.streitgenossen.wirtschaftlichIdentisch = wirtschaftlichIdentisch;
		container.streitgenossen.klaeger = btmKlaeger.getAllValues();
		if (widerKlageAnzeigen)
			container.streitgenossen.drittwiderbeklagte = btmDrittwiderbeklagte.getAllValues();
		else
			container.streitgenossen.drittwiderbeklagte = new BaumbachBeteiligtenListe();
		container.streitgenossen.beklagte = btmBeklagte.getAllValues();

		/* Dritte Registerkarte Forderungsstaffelung */
		container.staffelung.parteien.add((Beteiligter) cbAngriffsKlaeger.getSelectedItem());
		container.staffelung.parteien.add((Beteiligter) cbVerteidigungsBeklagter.getSelectedItem());
		container.staffelung.forderungen = ftm.getAllValues();

		/* Vierte Registerkarte Freie Gebührenberechnung */
		container.gebuehrenBerechnung.streitwert = 0L;
		container.gebuehrenBerechnung.gebuehren = new GebuehrenTatbestand[] {};
		try {
			container.gebuehrenBerechnung.streitwert = Long.parseLong(ftfStreitwert.getText());
		} catch (NumberFormatException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		}
		container.gebuehrenBerechnung.gebuehren = gttmRechner.toArray();

		/* Höchster Wert im Verfahren */
		container.allgemein.streitwert = Math
				.max(container.streitgenossen.klaeger.getHoechstenStreitwert(Beteiligter.KLAEGER),
						Math.max(
								container.streitgenossen.drittwiderbeklagte.getHoechstenStreitwert(
										Beteiligter.DRITTWIDERBEKLAGTE),
						Math.max(container.streitgenossen.beklagte.getHoechstenStreitwert(Beteiligter.BEKLAGTE),
								Math.max((double) container.gebuehrenBerechnung.streitwert,
										container.teilklageruecknahme.streitwerteUndObsiegen.get(0).doubleValue()))));

		/* und weg damit */
		return container;
	}
}
