/**
 * MacToolkiste.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import javax.swing.ImageIcon;

import com.apple.eawt.AboutHandler;
import com.apple.eawt.AppEvent.QuitEvent;
import com.apple.eawt.QuitHandler;
import com.apple.eawt.AppEvent.AboutEvent;
import com.apple.eawt.Application;
import com.apple.eawt.QuitResponse;

/**
 * In diese Klasse mussten die Besonderheiten für Mac OS X, damit die
 * Hauptklasse auch unter Windows ausgeführt werden kann.
 * 
 * @author Peter Schuster (setrok)
 * @date 18.07.2014
 * 
 */
class MacToolkiste {

	/**
	 * @url 
	 *      "http://www.javabeginners.de/Mac-Besonderheiten/MacOSX-Menuleiste.php"
	 * 
	 */
	static class MacHandler implements AboutHandler, QuitHandler {

		/**
		 * Die Methode ruft {@link Kostenrechner#beendeKostenrechner()} als
		 * Reaktion auf Apfel+Q auf.
		 * 
		 * @param qe
		 *            ein QuitEvent
		 * @param qr
		 *            eine QuitResponse
		 * 
		 * @see com.apple.eawt.QuitHandler#handleQuitRequestWith(com.apple.eawt.AppEvent.QuitEvent,
		 *      com.apple.eawt.QuitResponse)
		 */
		@Override
		public void handleQuitRequestWith(QuitEvent qe, QuitResponse qr) {
			Kostenrechner.beendeKostenrechner();
		}

		/**
		 * Die Methode ruft {@link Kostenrechner#ueberKostenrechner()} als
		 * Reaktion auf "About Kostentenor" auf.
		 * 
		 * @param ae
		 *            ein AboutEvent
		 * 
		 * @see com.apple.eawt.AboutHandler#handleAbout(com.apple.eawt.AppEvent.AboutEvent)
		 */
		@Override
		public void handleAbout(AboutEvent ae) {
			Kostenrechner.ueberKostenrechner();
		}

	}

	static private final MacHandler MAC_HANDLER = new MacHandler();

	/**
	 * Die Methoden der Klasse sind statisch, der Konstruktor daher private
	 * 
	 */
	private MacToolkiste() {
		// nichts zu tun
	}

	static void dockIconSetzen() {
		// Dock-Icon setzen
		Application.getApplication().setDockIconImage(
				new ImageIcon(Kostenrechner.class.getClassLoader().getResource(
						"resources/rechnerhammer-128.png")).getImage());
	}

	static void aboutMenuSetzen() {
		// "About"-Menu setzen
		Application.getApplication().setAboutHandler(MAC_HANDLER);
	}

	static void quitSetzen() {
		Application.getApplication().setQuitHandler(MAC_HANDLER);
	}
}
