/**
 * Tabulierend.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

/**
 * Schnittstelle um Tenorierend-Klassen zu kennzeichen, die Begründungen auch
 * als Tabellendaten zurückgeben können.
 *
 * @author Peter Schuster (setrok)
 * @date 12.10.2014
 *
 */
interface Tabulierend {

	/**
	 * Die Methode liefert die eigentlichen Tabellendaten.
	 * 
	 * @return ein Array Object[][]
	 */
	Object[][] getTableCellValues();

	/**
	 * Die Methode sagt, welche Datentypen von {@link #getTableCellValues()}
	 * zurückgegeben werden.
	 * 
	 * @return ein Array Class&lt;?&gt;[]
	 */
	Class<?>[] getColumnClasses();

	/**
	 * Die Methode sagt, welche Spaltenüberschriften die Tabellendaten haben.
	 * 
	 * @return ein Array String[]
	 */
	String[] getColumnNames();

	/**
	 * Die Methode dient dazu, Start und Ende des Textbereichs, der durch die
	 * Tabelle ersetzt werden soll, zurückzugeben.
	 * 
	 * @return ein Array int[2], bei dem [0] der Start und [1] das Ende des
	 *         Bereichs ist, der ersetzt werden soll.
	 */
	int[] getRange();

}
