/**
 * TenorDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;
import javax.swing.text.rtf.RTFEditorKit;

/**
 * Ein JDialog, der einen Tenor anzeigt und diesen auf Wunsch als RTF speichert
 * und diesen auch in die Zwischenablage kopieren kann.
 * 
 * @author Peter Schuster (setrok)
 * @date 15.06.2014
 */
public class TenorDialog extends HinzufuegenDialog<Void>implements ClipboardOwner {

	private static final String POPUPLABEL_TENORTEXT = "Tenortext...";
	private static final String RTF_EXTENSION = ".rtf";
	// private static final EmptyBorder BORDER = new EmptyBorder(5, 5, 5, 5);
	private static final long serialVersionUID = -7570808761464484864L;
	private JScrollPane scpKostentenor;
	private JTextPane txtpnKostentenor;
	private RTFEditorKit rtf;
	private Tenorierend tenorierend;
	private final VerfahrensDatenContainer container;
	private final AbstractAction tenorInZwischenablage = new AbstractAction("Tenor in Zwischenablage kopieren") {
		private static final long serialVersionUID = -1470041456809868231L;

		{
			putValue(ACTION_COMMAND_KEY, "copy");
			putValue(SHORT_DESCRIPTION, "Kopiert den gesamten Tenortext in die Zwischenablage.");
			putValue(MNEMONIC_KEY, KeyEvent.VK_K);
			// putValue(ACCELERATOR_KEY, KeyEvent.VK_C);
			putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, Kostenrechner.ctrlMask));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			// kopiereInZwischenablage();
			kopiereRtfInZwischenablage();
		}
	};
	private final AbstractAction tenorSpeichern = new AbstractAction("Tenor speichern") {

		private static final long serialVersionUID = 1162624762983290295L;

		{
			putValue(ACTION_COMMAND_KEY, "save");
			putValue(SHORT_DESCRIPTION, "Speichert den Tenortext im Rich Text Format (RTF)");
			putValue(MNEMONIC_KEY, KeyEvent.VK_S);
			// putValue(ACCELERATOR_KEY, KeyEvent.VK_S);
			putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S, Kostenrechner.ctrlMask));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			speichereRTF();
		}
	};
	/**
	 * begruendungEinfuegen ist eine AbstractAction, die
	 * {@link #setBegruendungText()} aufruft.
	 */
	private final AbstractAction begruendungEinfuegen = new BeschriebeneAktion("Begründung hinzufügen",
			"Fügt die hinterlegte Begründung für den generierten Tenor an.") {
		private static final long serialVersionUID = 6000735647165897065L;

		@Override
		public void actionPerformed(ActionEvent e) {
			setBegruendungText();
			setEnabled(false);
		}
	};

	/**
	 * emailSchreibenTenorFalsch lässt eine Beschwerde-E-Mail im
	 * Standard-E-Mail-Client über {@link TenorUtils.emailSchreiben(String,
	 * String)} schreiben. Dabei wird der Text des Tenors und noch XML-Daten
	 * mitgeschickt.
	 * 
	 */
	private final AbstractAction emailSchreibenTenorFalsch = new BeschriebeneAktion("Tenor falsch",
			"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail an den Programmierer zu verfassen</html>",
			KeyEvent.VK_E, "e-mail") {

		private static final long serialVersionUID = -978875818285786710L;

		@Override
		public void actionPerformed(ActionEvent e) {
			final String subject = "Falscher Tenor in kostentenor.de " + Kostenrechner.KOSTENRECHNER_VERSION;
			// Im E-Mail-Text werden Tenor und noch XML-Daten mitgeschickt
			final String xmlString = BerechnungLadenSpeichernDialog.toXmlString(container);
			final String tenor = !(tenorierend instanceof UrteilsTenorierend) ? tenorierend.erzeugeTenor()
					: tenorierend.erzeugeTenor() + EmailUtils.NEUER_ABSATZ
							+ ((UrteilsTenorierend) tenorierend).getGruende();
			final String body = "Bitte beschreiben Sie, warum Sie den Tenor für falsch halten" + EmailUtils.NEUER_ABSATZ
					+ tenor + EmailUtils.NEUER_ABSATZ + xmlString;
			EmailUtils.emailSchreiben(subject, body);
		}
	};

	/**
	 * bereitsGruendeDa speichert, ob die Begründung bereits eingefügt wurde,
	 * als boolean
	 */
	private boolean bereitsGruendeDa = false;

	/**
	 * Create the dialog.
	 * 
	 * @param owner
	 *            der JFrame, der den JDialog aufruft
	 * @param container
	 *            TODO
	 * 
	 * @see javax.swing.JDialog#JDialog(JFrame,boolean)
	 */
	public TenorDialog(JFrame owner, Tenorierend tenorierend, VerfahrensDatenContainer container) {
		// Beschreibung vom "Tenorierend" holen
		super(owner, tenorierend.getTenorBeschreibung());
		this.tenorierend = tenorierend;
		this.container = container;
		super.setOkButtonToBeFocussed(true);
	}

	/**
	 * Die Methode fuellt das ContentPane mit einer {@link JTextPane} für den
	 * Tenortext des {@link UrteilsTenorierend} in einer {@link JScrollPane}.
	 * Zudem setzt sie ein paar mehr {@link JButton Schaltflächen}
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		txtpnKostentenor = new JTextPane();
		rtf = new RTFEditorKit();
		txtpnKostentenor.setEditorKit(rtf);
		txtpnKostentenor.setEditable(false);
		scpKostentenor = new JScrollPane(txtpnKostentenor);
		contentPanel.add(scpKostentenor,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 3, 1, GridBagConstraints.BOTH, true));
		getButtonPane().add(new JButton(tenorInZwischenablage));
		addToRootInputMap(tenorInZwischenablage);

		getButtonPane().add(new JButton(tenorSpeichern));
		addToRootInputMap(tenorSpeichern);

		getButtonPane().add(new JButton(emailSchreibenTenorFalsch));
		addToRootInputMap(emailSchreibenTenorFalsch);

		setKostentenorText();
		// txtpnKostentenor.setContentType("text/rtf; charset=8859_1");
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

		/* Kontextmenü zusammensetzen */
		JPopupMenu pmTenortext = new JPopupMenu(POPUPLABEL_TENORTEXT);
		pmTenortext.add(new JMenuItem(POPUPLABEL_TENORTEXT));
		pmTenortext.addSeparator();
		pmTenortext.add(new JMenuItem(tenorSpeichern));
		pmTenortext.add(new JMenuItem(tenorInZwischenablage));
		pmTenortext.add(new JMenuItem(begruendungEinfuegen));
		/*
		 * Das Einfügen von Begründungen geht nur bei Klassen, die
		 * UrteilsTenorierend implementieren; sonst fehlt die entsprechende
		 * Methode
		 */
		if (tenorierend instanceof UrteilsTenorierend)
			begruendungEinfuegen.setEnabled(true);
		else
			begruendungEinfuegen.setEnabled(false);
		txtpnKostentenor.addMouseListener(new KontextMouseListener(pmTenortext));
	}

	/**
	 * Die Methode setKostentenorText dient dazu, den Tenor der übergegebenen
	 * Tenorklasse im JTextPane mit der passenden Schriftart und Formatierungen
	 * zu setzen. Als Dialogtitel wird die Tenorbeschreibung des Tenorierend
	 * gesetzt.
	 * 
	 * @param tenorierend
	 *            die Tenor erzeugende Klasseninstanz, die "Tenorierend"
	 *            implementiert.
	 * 
	 * @see javax.swing.text.Document#insertString(int,String,AttributeSet)
	 */
	void setKostentenorText() {
		final String tenor = tenorierend.erzeugeTenor();
		// Beschreibung vom "Tenorierend" holen
		try {
			DefaultStyledDocument doc = (DefaultStyledDocument) getDocument();
			createStyles(doc);
			doc.remove(0, doc.getLength());
			doc.insertString(0, tenor, doc.getStyle("base"));
			setParagraphAttributes(doc, 0, doc.getLength(), !(tenorierend instanceof UrteilsTenorierend));
			// Die Beschreibung auch beim Speichern als Titel setzen
			doc.putProperty(Document.TitleProperty, tenorierend.getTenorBeschreibung());
			setMaximumSizeFromAttributeSet(doc.getStyle("base"));
		} catch (BadLocationException ble) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ble.getLocalizedMessage(), ble);
		}
	}

	/**
	 * Die Methode dient dazu, an den Tenortext Gründe anzuhängen, falls es sich
	 * beim {@link #tenorierend} um ein {@link UrteilsTenorierend} handelt und
	 * die Gründe nicht bereits angehängt wurden.
	 * 
	 */
	void setBegruendungText() {
		if (tenorierend instanceof UrteilsTenorierend && !bereitsGruendeDa) {
			final String gruende = ((UrteilsTenorierend) tenorierend).getGruende();
			try {
				DefaultStyledDocument doc = (DefaultStyledDocument) getDocument();
				int caret = txtpnKostentenor.getCaretPosition();
				int anfang = doc.getLength();
				doc.insertString(anfang, UrteilsTenorierend.GRUENDE_UEBERSCHRIFT, doc.getStyle("bold"));
				int nachUeberschrift = doc.getLength();
				doc.insertString(nachUeberschrift, gruende, doc.getStyle("base"));
				if (istTenorierendTabulierend()) {
					/*
					 * Wenn eine Begründungstabelle erzeugt werden kann, dann
					 * diese einfügen.
					 */
					Tabulierend tab = (Tabulierend) tenorierend;
					int[] range = tab.getRange();
					int anfangGruende = nachUeberschrift + range[0] + 1;
					int laenge = range[1] - range[0];
					/*
					 * Damit es keine Dopplungen gibt, erst einmal den alten
					 * Text herausnehmen
					 */
					doc.remove(anfangGruende, laenge - 1);
					/*
					 * Dann den neuen Text mit der richtigen Formatierung (d.h.
					 * mit JTable in JScrollpane) einfügen
					 */
					doc.insertString(anfangGruende, gruende.substring(range[0] + 1, range[1]), doc.getStyle("table"));
				}
				setParagraphAttributes(doc, anfang, doc.getLength() - anfang, false);
				txtpnKostentenor.setCaretPosition(caret);
				bereitsGruendeDa = true;
			} catch (BadLocationException ble) {
				Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ble.getLocalizedMessage(), ble);
			}
		}
	}

	/**
	 * Die Methode setzt die Absatzformatierung auf anderthalbfachen
	 * Zeilenabsabstand (ok, 1,33-fach) und 7px Abstand danach.
	 * 
	 * @see javax.swing.JTextPane#getParagraphAttributes()
	 */
	private void setParagraphAttributes(StyledDocument doc, int offset, int length, boolean grosseTabulatoren) {
		int caret = txtpnKostentenor.getCaretPosition();
		txtpnKostentenor.setCaretPosition(offset);
		if (grosseTabulatoren)
			doc.setParagraphAttributes(offset, length, doc.getStyle("tabulatoren"), false);
		else
			doc.setParagraphAttributes(offset, length, doc.getStyle("base"), false);
		txtpnKostentenor.setCaretPosition(caret);
	}

	/**
	 * Die Methode dient dazu, Styles zu definieren. Davon ist einer normal
	 * ("base"), einer fett ("bold") und einer mit Tabstopps ("tabulatoren").
	 * Außerdem einen Style mit einer JTable ("table"), sofern das übergebene
	 * {@link Tenorierend} auch das Interface {@link Tabulierend} implementiert.
	 * 
	 * @param doc
	 *            ein {@link StyledDocument}
	 * 
	 * @url "http://www.java-forums.org/awt-swing/2260-how-insert-tables-into-jtextpane.html#post4296"
	 *      "http://www.java2s.com/Tutorial/Java/0240__Swing/TabStopandTabSetClasses.htm"
	 *      "http://www.java2s.com/Code/Java/Swing-JFC/Createadecimalalignedtabstopat400pixelsfromtheleftmargin.htm"
	 */
	private void createStyles(StyledDocument doc) {
		Style baseStyle = doc.addStyle("base", null);
		setFontArial(baseStyle);
		setFontSize(baseStyle);
		StyleConstants.setLineSpacing(baseStyle, .33f);
		StyleConstants.setSpaceBelow(baseStyle, 7f);

		Style style = doc.addStyle("bold", baseStyle);
		StyleConstants.setBold(style, true);

		style = doc.addStyle("tabulatoren", baseStyle);
		StyleConstants.setTabSet(style,
				new TabSet(new TabStop[] { new TabStop(600, TabStop.ALIGN_RIGHT, TabStop.LEAD_DOTS),
						new TabStop(950, TabStop.ALIGN_RIGHT, TabStop.LEAD_DOTS) }));

		if (istTenorierendTabulierend()) {
			style = doc.addStyle("table", baseStyle);
			JTable table = new JTable(new TabulierendTableModel((Tabulierend) tenorierend));
			// Dimension d = table.getMinimumSize();
			Dimension d = table.getPreferredSize();
			d.width = txtpnKostentenor.getPreferredSize().width;
			table.setPreferredScrollableViewportSize(d);
			table.setDefaultRenderer(Double.class, new NachkommastellenRenderer());
			table.setDefaultRenderer(String.class, new StringButtonRenderer());
			JScrollPane jScrollPane = new JScrollPane(table);
			StyleConstants.setComponent(style, jScrollPane);
		}
	}

	/**
	 * @return true, sofern das übergebene {@link Tenorierend} auch das
	 *         Interface {@link Tabulierend} implementiert
	 */
	private boolean istTenorierendTabulierend() {
		return tenorierend instanceof Tabulierend;
	}

	/**
	 * Die Methode dient dazu, die Schriftart Arial , sonst eine verwandte
	 * Schriftart aus den Schriftarten des Systems zu finden. Sie geht
	 * nacheinander die Schriftnamen durch. Ist einer davon der aktuelle Font,
	 * gewinnt dieser immer, sonst wird einer der anderen aus der Liste
	 * genommen, sofern er existiert.
	 * 
	 * @param attrib
	 *            ein {@link MutableAttributeSet} oder {@link Style}
	 */
	private void setFontArial(MutableAttributeSet attrib) {
		boolean ok = false;
		final String ff = StyleConstants.getFontFamily(attrib);
		for (String arial : new String[] { "Arial", "Helvetica", "Helv" })
			if (arial.equals(ff)) {
				StyleConstants.setFontFamily(attrib, arial);
				ok = true;
			} else if (!ok && getFontForName(arial) != null) {
				StyleConstants.setFontFamily(attrib, arial);
				ok = true;
			}
	}

	/**
	 * Zusehen, dass das Fenster nicht breiter als 120 Zeichen wird.
	 * 
	 * @param attrib
	 *            ein {@link AttributeSet}
	 */
	private void setMaximumSizeFromAttributeSet(AttributeSet attrib) {
		final int maxSpalten = 120;
		final int charWidth = getCharWidth(attrib);
		Dimension maximumSize = super.getMaximumSize();
		maximumSize.width = charWidth > 0 && charWidth * maxSpalten < maximumSize.width ? charWidth * maxSpalten
				: maximumSize.width;
		super.setMaximumSize(maximumSize);
	}

	/**
	 * Die Methode dient dazu, die Breite eines Zeichens ('n') zu ermitteln
	 * 
	 * @param attrib
	 *            ein {@link AttributeSet}
	 * @return Zeichenbreite in Pixeln als {@link Integer int}
	 */
	private int getCharWidth(AttributeSet attrib) {
		FontMetrics fontMetrics = getFontMetrics(getFontForName(StyleConstants.getFontFamily(attrib)));
		int charWidth = fontMetrics.charWidth('n');
		System.out.println("Breite eines 'n':" + charWidth);
		return charWidth;
	}

	/**
	 * Die Methode findet die Schriftgröße heraus. Die Schriftgröße wird auf
	 * 12px gesetzt, solange nicht die Systemschriftgröße größer ist.
	 * 
	 * @param attrib
	 *            ein {@link MutableAttributeSet} oder {@link Style}
	 */
	private void setFontSize(MutableAttributeSet attrib) {
		// TODO Kann man herausfinden, ob die Schriftart die Größe unterstützt?
		final int fs = StyleConstants.getFontSize(attrib);
		StyleConstants.setFontSize(attrib, fs > getFont().getSize() ? fs : getFont().getSize());
	}

	/**
	 * Die Methode getFontForName dient dazu, herauszufinden, ob ein Font mit
	 * dem als <code>name</code> übergebenen Namen im System existiert.
	 * 
	 * @param name
	 * @return
	 */
	private Font getFontForName(String name) {
		if (name == null || "".equals(name))
			return null;
		Font f = null;
		for (String fonts : GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames())
			if (name.equals(fonts))
				f = new Font(fonts, Font.PLAIN, 12);
		return f;
	}

	/**
	 * Die Methode speichereRTF dient dazu, den Inhalt der JTextPane als RTF zu
	 * speichern. Die Variante ohne Parameter soll vom ActionListener der
	 * Schaltfläche selbst aufgerufen werden. Sie ruft nacheinander den
	 * Dateiauswahldialog auf, überprüft ob die Datei schon existiert und ruft
	 * dann die eigentliche Methode zum Speichern auf.
	 * 
	 * Write content from a document to the given stream in a format appropriate
	 * for this kind of content handler.
	 * 
	 */
	private boolean speichereRTF() {
		boolean zwischen = false;
		JFileChooser dateiAuswahl = getRtfFileChooser();
		int option = dateiAuswahl.showSaveDialog(this);
		if (option == JFileChooser.APPROVE_OPTION) {
			File datei = dateiAuswahl.getSelectedFile().getAbsoluteFile();
			if (!datei.exists())
				zwischen = speichereRTF(datei);
			else {
				int bestaetigung = JOptionPane.showConfirmDialog(this, "Die Datei existiert schon. \n\nFortfahren?");
				if (bestaetigung == JOptionPane.OK_OPTION)
					zwischen = speichereRTF(datei);
				else
					JOptionPane.showMessageDialog(this, "Die Datei wurde nicht gespeichert.");
			}
		}
		return zwischen;
	}

	/**
	 * Die Methode speichereRTF dient dazu, die <code>datei</code> wirklich zu
	 * speichern. Dazu wird
	 * {@link javax.swing.text.rtf.RTFEditorKit#write(OutputStream,Document,int,int)}
	 * benutzt.
	 * 
	 * @param datei
	 *            die Datei, in die gespeichert werden soll
	 * 
	 * @return hat's geklappt, dann true
	 * 
	 * @url "http://stackoverflow.com/questions/2725141/java-jtextpane-rtf-save"
	 * 
	 * @see javax.swing.text.rtf.RTFEditorKit#write(OutputStream,Document,int,int)
	 */
	private boolean speichereRTF(File datei) {
		// TODO Probleme mit dem Zeichensatz
		boolean zwischen = false;
		StyledDocument doc = getDocument();
		BufferedOutputStream bos = null;
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(datei);
			bos = new BufferedOutputStream(fos);
			rtf.write(bos, doc, doc.getStartPosition().getOffset(), doc.getLength());
			zwischen = true;
		} catch (FileNotFoundException e) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} catch (IOException e) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} catch (BadLocationException e) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} finally {
			if (bos != null)
				try {
					bos.close();
				} catch (IOException e) {
				}
			if (fos != null)
				try {
					fos.close();
				} catch (IOException e) {
				}
		}
		return zwischen;
	}

	/**
	 * Die Methode dient dazu, einen JFileChooser zu erzeugen, der fürs
	 * Speichern von RTF-Dateien vorkonfiguriert ist.
	 * 
	 * @return einen JFileChooser
	 */
	private JFileChooser getRtfFileChooser() {
		/**
		 * FileFilter für RTF-Dateien (und Verzeichnisse)
		 */
		FileFilter rtfFilter = new FileFilter() {
			@Override
			public boolean accept(File f) {
				return (f != null && (f.isDirectory() || f.getName().endsWith(RTF_EXTENSION)));
			}

			@Override
			public String getDescription() {
				return "Rich Text Format (RTF)";
			}
		};
		JFileChooser dateiAuswahl = new JFileChooser();
		dateiAuswahl.setMultiSelectionEnabled(false);
		dateiAuswahl.setDialogTitle("Tenor als RTF-Datei speichern...");
		dateiAuswahl.addChoosableFileFilter(rtfFilter);
		dateiAuswahl.setFileFilter(rtfFilter);
		dateiAuswahl.setAcceptAllFileFilterUsed(false);
		dateiAuswahl.setSelectedFile(new File("tenor" + RTF_EXTENSION));
		return dateiAuswahl;
	}

	/**
	 * Die Methode fügt den Text der JTextPane in die Zwischenablage ein.
	 * Versucht das jetzt mit text/rtf!
	 * 
	 */
	private void kopiereRtfInZwischenablage() {
		StyledDocument doc = getDocument();
		Clipboard zwischenAblage = Toolkit.getDefaultToolkit().getSystemClipboard();
		DataFlavor[] flavors = null;
		String[] inhalte = null;
		Transferable rtft = null;
		try {
			// FIXME: auch hier funktioniert der Zeichensatz nicht.
			inhalte = new String[] { getRtfAsString(doc), getAllText(doc) };
			flavors = new DataFlavor[] { new DataFlavor("text/rtf", "Rich Formatted Text"),
					new DataFlavor("text/plain", "Plain Text") };
			rtft = new RtfTransferable(flavors, inhalte);
			zwischenAblage.setContents(rtft, this);
		} catch (BadLocationException | UnsupportedEncodingException e) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
			// e.printStackTrace();
		}
	}

	/**
	 * Die Methode schreibt den RTF-Inhalt des StyledDocument in einen
	 * ByteArrayOutputStream.
	 * 
	 * @param doc
	 *            das StyledDocument, dessen Inhalt ausgesaugt werden soll
	 * 
	 * @return den Inhalt des ByteArrayOutputStream mittels toString()
	 * @throws UnsupportedEncodingException
	 */
	private String getRtfAsString(StyledDocument doc) throws UnsupportedEncodingException {
		ByteArrayOutputStream bos = null;
		try {
			bos = new ByteArrayOutputStream(doc.getLength());
			rtf.write(bos, doc, doc.getStartPosition().getOffset(), doc.getLength());
		} catch (IOException e) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} catch (BadLocationException e) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} finally {
			// der ByteArrayOutputStream muss nicht geschlossen werden
		}
		return bos != null ? bos.toString("UTF-8") : null;
	}

	/**
	 * Die Methode holt das StyledDocument aus der JTextPane
	 * 
	 * @return the model
	 */
	private StyledDocument getDocument() {
		return txtpnKostentenor.getStyledDocument();
	}

	/**
	 * Die Methode holt sich dem vollständigen Text der JTextPane
	 * 
	 * @param doc
	 *            ein {@link StyledDocument}
	 * @return den gesamten Text der {@link JTextPane} als {@link String}
	 * @throws BadLocationException
	 */
	private String getAllText(StyledDocument doc) throws BadLocationException {
		return doc.getText(doc.getStartPosition().getOffset(), doc.getLength());
	}

	/**
	 * 
	 * @see java.awt.datatransfer.ClipboardOwner#lostOwnership(java.awt.datatransfer.Clipboard,
	 *      java.awt.datatransfer.Transferable)
	 */
	@Override
	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		System.out.println("Och nö, gehört mir doch die Zwischeablage nicht mehr!");
	}

	/**
	 * Die Methode dient der Kompatibilität zur Oberklasse und tut nichts.
	 * 
	 * @return null
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected Void baueRueckgabewert() {
		return null;
	}

	/**
	 * Die Methode dient der Kompatibilität zur Oberklasse und liefert immer
	 * true
	 * 
	 * @return true
	 * 
	 * @see eu.gronos.kostenrechner.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	protected boolean parseEingabe() {
		return true;
	}

}
